#pragma once

#define BOYS_SHORTGRID_MAXN 31
#define BOYS_SHORTGRID_MAXX 36.5
#define BOYS_SHORTGRID_SPACE 0.1
#define BOYS_SHORTGRID_NPOINT 366
#define BOYS_SHORTGRID_LOOKUPFAC 10.0
#define BOYS_SHORTGRID_LOOKUPFAC2 0.05

