#include "simint/boys/boys.h"
#include "simint/ostei/gen/ostei_generated.h"
#include "simint/vectorization/vectorization.h"
#include <math.h>
#include <string.h>


int ostei_k_k_k_k(struct simint_multi_shellpair const P,
                  struct simint_multi_shellpair const Q,
                  double screen_tol,
                  double * const restrict work,
                  double * const restrict INT__k_k_k_k)
{

    SIMINT_ASSUME_ALIGN_DBL(work);
    SIMINT_ASSUME_ALIGN_DBL(INT__k_k_k_k);
    int ab, cd, abcd;
    int istart, jstart;
    int iprimcd, nprim_icd, icd;
    const int check_screen = (screen_tol > 0.0);
    int i, j;
    int n;
    int not_screened;
    int real_abcd;
    int iket;
    int ibra;

    // partition workspace
    double * const INT__k_s_k_s = work + (SIMINT_NSHELL_SIMD * 0);
    double * const INT__k_s_l_s = work + (SIMINT_NSHELL_SIMD * 1296);
    double * const INT__k_s_m_s = work + (SIMINT_NSHELL_SIMD * 2916);
    double * const INT__k_s_n_s = work + (SIMINT_NSHELL_SIMD * 4896);
    double * const INT__k_s_o_s = work + (SIMINT_NSHELL_SIMD * 7272);
    double * const INT__k_s_q_s = work + (SIMINT_NSHELL_SIMD * 10080);
    double * const INT__k_s_r_s = work + (SIMINT_NSHELL_SIMD * 13356);
    double * const INT__k_s_t_s = work + (SIMINT_NSHELL_SIMD * 17136);
    double * const INT__l_s_k_s = work + (SIMINT_NSHELL_SIMD * 21456);
    double * const INT__l_s_l_s = work + (SIMINT_NSHELL_SIMD * 23076);
    double * const INT__l_s_m_s = work + (SIMINT_NSHELL_SIMD * 25101);
    double * const INT__l_s_n_s = work + (SIMINT_NSHELL_SIMD * 27576);
    double * const INT__l_s_o_s = work + (SIMINT_NSHELL_SIMD * 30546);
    double * const INT__l_s_q_s = work + (SIMINT_NSHELL_SIMD * 34056);
    double * const INT__l_s_r_s = work + (SIMINT_NSHELL_SIMD * 38151);
    double * const INT__l_s_t_s = work + (SIMINT_NSHELL_SIMD * 42876);
    double * const INT__m_s_k_s = work + (SIMINT_NSHELL_SIMD * 48276);
    double * const INT__m_s_l_s = work + (SIMINT_NSHELL_SIMD * 50256);
    double * const INT__m_s_m_s = work + (SIMINT_NSHELL_SIMD * 52731);
    double * const INT__m_s_n_s = work + (SIMINT_NSHELL_SIMD * 55756);
    double * const INT__m_s_o_s = work + (SIMINT_NSHELL_SIMD * 59386);
    double * const INT__m_s_q_s = work + (SIMINT_NSHELL_SIMD * 63676);
    double * const INT__m_s_r_s = work + (SIMINT_NSHELL_SIMD * 68681);
    double * const INT__m_s_t_s = work + (SIMINT_NSHELL_SIMD * 74456);
    double * const INT__n_s_k_s = work + (SIMINT_NSHELL_SIMD * 81056);
    double * const INT__n_s_l_s = work + (SIMINT_NSHELL_SIMD * 83432);
    double * const INT__n_s_m_s = work + (SIMINT_NSHELL_SIMD * 86402);
    double * const INT__n_s_n_s = work + (SIMINT_NSHELL_SIMD * 90032);
    double * const INT__n_s_o_s = work + (SIMINT_NSHELL_SIMD * 94388);
    double * const INT__n_s_q_s = work + (SIMINT_NSHELL_SIMD * 99536);
    double * const INT__n_s_r_s = work + (SIMINT_NSHELL_SIMD * 105542);
    double * const INT__n_s_t_s = work + (SIMINT_NSHELL_SIMD * 112472);
    double * const INT__o_s_k_s = work + (SIMINT_NSHELL_SIMD * 120392);
    double * const INT__o_s_l_s = work + (SIMINT_NSHELL_SIMD * 123200);
    double * const INT__o_s_m_s = work + (SIMINT_NSHELL_SIMD * 126710);
    double * const INT__o_s_n_s = work + (SIMINT_NSHELL_SIMD * 131000);
    double * const INT__o_s_o_s = work + (SIMINT_NSHELL_SIMD * 136148);
    double * const INT__o_s_q_s = work + (SIMINT_NSHELL_SIMD * 142232);
    double * const INT__o_s_r_s = work + (SIMINT_NSHELL_SIMD * 149330);
    double * const INT__o_s_t_s = work + (SIMINT_NSHELL_SIMD * 157520);
    double * const INT__q_s_k_s = work + (SIMINT_NSHELL_SIMD * 166880);
    double * const INT__q_s_l_s = work + (SIMINT_NSHELL_SIMD * 170156);
    double * const INT__q_s_m_s = work + (SIMINT_NSHELL_SIMD * 174251);
    double * const INT__q_s_n_s = work + (SIMINT_NSHELL_SIMD * 179256);
    double * const INT__q_s_o_s = work + (SIMINT_NSHELL_SIMD * 185262);
    double * const INT__q_s_q_s = work + (SIMINT_NSHELL_SIMD * 192360);
    double * const INT__q_s_r_s = work + (SIMINT_NSHELL_SIMD * 200641);
    double * const INT__q_s_t_s = work + (SIMINT_NSHELL_SIMD * 210196);
    double * const INT__r_s_k_s = work + (SIMINT_NSHELL_SIMD * 221116);
    double * const INT__r_s_l_s = work + (SIMINT_NSHELL_SIMD * 224896);
    double * const INT__r_s_m_s = work + (SIMINT_NSHELL_SIMD * 229621);
    double * const INT__r_s_n_s = work + (SIMINT_NSHELL_SIMD * 235396);
    double * const INT__r_s_o_s = work + (SIMINT_NSHELL_SIMD * 242326);
    double * const INT__r_s_q_s = work + (SIMINT_NSHELL_SIMD * 250516);
    double * const INT__r_s_r_s = work + (SIMINT_NSHELL_SIMD * 260071);
    double * const INT__r_s_t_s = work + (SIMINT_NSHELL_SIMD * 271096);
    double * const INT__t_s_k_s = work + (SIMINT_NSHELL_SIMD * 283696);
    double * const INT__t_s_l_s = work + (SIMINT_NSHELL_SIMD * 288016);
    double * const INT__t_s_m_s = work + (SIMINT_NSHELL_SIMD * 293416);
    double * const INT__t_s_n_s = work + (SIMINT_NSHELL_SIMD * 300016);
    double * const INT__t_s_o_s = work + (SIMINT_NSHELL_SIMD * 307936);
    double * const INT__t_s_q_s = work + (SIMINT_NSHELL_SIMD * 317296);
    double * const INT__t_s_r_s = work + (SIMINT_NSHELL_SIMD * 328216);
    double * const INT__t_s_t_s = work + (SIMINT_NSHELL_SIMD * 340816);
    SIMINT_DBLTYPE * const primwork = (SIMINT_DBLTYPE *)(work + SIMINT_NSHELL_SIMD*355216);
    SIMINT_DBLTYPE * const restrict PRIM_INT__s_s_s_s = primwork + 0;
    SIMINT_DBLTYPE * const restrict PRIM_INT__s_s_p_s = primwork + 29;
    SIMINT_DBLTYPE * const restrict PRIM_INT__s_s_d_s = primwork + 71;
    SIMINT_DBLTYPE * const restrict PRIM_INT__s_s_f_s = primwork + 149;
    SIMINT_DBLTYPE * const restrict PRIM_INT__s_s_g_s = primwork + 269;
    SIMINT_DBLTYPE * const restrict PRIM_INT__s_s_h_s = primwork + 434;
    SIMINT_DBLTYPE * const restrict PRIM_INT__s_s_i_s = primwork + 644;
    SIMINT_DBLTYPE * const restrict PRIM_INT__s_s_k_s = primwork + 896;
    SIMINT_DBLTYPE * const restrict PRIM_INT__p_s_s_s = primwork + 1184;
    SIMINT_DBLTYPE * const restrict PRIM_INT__p_s_p_s = primwork + 1268;
    SIMINT_DBLTYPE * const restrict PRIM_INT__p_s_d_s = primwork + 1394;
    SIMINT_DBLTYPE * const restrict PRIM_INT__p_s_f_s = primwork + 1628;
    SIMINT_DBLTYPE * const restrict PRIM_INT__p_s_g_s = primwork + 1988;
    SIMINT_DBLTYPE * const restrict PRIM_INT__p_s_h_s = primwork + 2483;
    SIMINT_DBLTYPE * const restrict PRIM_INT__p_s_i_s = primwork + 3113;
    SIMINT_DBLTYPE * const restrict PRIM_INT__p_s_k_s = primwork + 3869;
    SIMINT_DBLTYPE * const restrict PRIM_INT__p_s_l_s = primwork + 4733;
    SIMINT_DBLTYPE * const restrict PRIM_INT__d_s_s_s = primwork + 5678;
    SIMINT_DBLTYPE * const restrict PRIM_INT__d_s_p_s = primwork + 5840;
    SIMINT_DBLTYPE * const restrict PRIM_INT__d_s_d_s = primwork + 6092;
    SIMINT_DBLTYPE * const restrict PRIM_INT__d_s_f_s = primwork + 6560;
    SIMINT_DBLTYPE * const restrict PRIM_INT__d_s_g_s = primwork + 7280;
    SIMINT_DBLTYPE * const restrict PRIM_INT__d_s_h_s = primwork + 8270;
    SIMINT_DBLTYPE * const restrict PRIM_INT__d_s_i_s = primwork + 9530;
    SIMINT_DBLTYPE * const restrict PRIM_INT__d_s_k_s = primwork + 11042;
    SIMINT_DBLTYPE * const restrict PRIM_INT__d_s_l_s = primwork + 12770;
    SIMINT_DBLTYPE * const restrict PRIM_INT__d_s_m_s = primwork + 14660;
    SIMINT_DBLTYPE * const restrict PRIM_INT__f_s_s_s = primwork + 16640;
    SIMINT_DBLTYPE * const restrict PRIM_INT__f_s_p_s = primwork + 16900;
    SIMINT_DBLTYPE * const restrict PRIM_INT__f_s_d_s = primwork + 17320;
    SIMINT_DBLTYPE * const restrict PRIM_INT__f_s_f_s = primwork + 18100;
    SIMINT_DBLTYPE * const restrict PRIM_INT__f_s_g_s = primwork + 19300;
    SIMINT_DBLTYPE * const restrict PRIM_INT__f_s_h_s = primwork + 20950;
    SIMINT_DBLTYPE * const restrict PRIM_INT__f_s_i_s = primwork + 23050;
    SIMINT_DBLTYPE * const restrict PRIM_INT__f_s_k_s = primwork + 25570;
    SIMINT_DBLTYPE * const restrict PRIM_INT__f_s_l_s = primwork + 28450;
    SIMINT_DBLTYPE * const restrict PRIM_INT__f_s_m_s = primwork + 31600;
    SIMINT_DBLTYPE * const restrict PRIM_INT__f_s_n_s = primwork + 34900;
    SIMINT_DBLTYPE * const restrict PRIM_INT__g_s_s_s = primwork + 38200;
    SIMINT_DBLTYPE * const restrict PRIM_INT__g_s_p_s = primwork + 38575;
    SIMINT_DBLTYPE * const restrict PRIM_INT__g_s_d_s = primwork + 39205;
    SIMINT_DBLTYPE * const restrict PRIM_INT__g_s_f_s = primwork + 40375;
    SIMINT_DBLTYPE * const restrict PRIM_INT__g_s_g_s = primwork + 42175;
    SIMINT_DBLTYPE * const restrict PRIM_INT__g_s_h_s = primwork + 44650;
    SIMINT_DBLTYPE * const restrict PRIM_INT__g_s_i_s = primwork + 47800;
    SIMINT_DBLTYPE * const restrict PRIM_INT__g_s_k_s = primwork + 51580;
    SIMINT_DBLTYPE * const restrict PRIM_INT__g_s_l_s = primwork + 55900;
    SIMINT_DBLTYPE * const restrict PRIM_INT__g_s_m_s = primwork + 60625;
    SIMINT_DBLTYPE * const restrict PRIM_INT__g_s_n_s = primwork + 65575;
    SIMINT_DBLTYPE * const restrict PRIM_INT__g_s_o_s = primwork + 70525;
    SIMINT_DBLTYPE * const restrict PRIM_INT__h_s_s_s = primwork + 75205;
    SIMINT_DBLTYPE * const restrict PRIM_INT__h_s_p_s = primwork + 75709;
    SIMINT_DBLTYPE * const restrict PRIM_INT__h_s_d_s = primwork + 76591;
    SIMINT_DBLTYPE * const restrict PRIM_INT__h_s_f_s = primwork + 78229;
    SIMINT_DBLTYPE * const restrict PRIM_INT__h_s_g_s = primwork + 80749;
    SIMINT_DBLTYPE * const restrict PRIM_INT__h_s_h_s = primwork + 84214;
    SIMINT_DBLTYPE * const restrict PRIM_INT__h_s_i_s = primwork + 88624;
    SIMINT_DBLTYPE * const restrict PRIM_INT__h_s_k_s = primwork + 93916;
    SIMINT_DBLTYPE * const restrict PRIM_INT__h_s_l_s = primwork + 99964;
    SIMINT_DBLTYPE * const restrict PRIM_INT__h_s_m_s = primwork + 106579;
    SIMINT_DBLTYPE * const restrict PRIM_INT__h_s_n_s = primwork + 113509;
    SIMINT_DBLTYPE * const restrict PRIM_INT__h_s_o_s = primwork + 120439;
    SIMINT_DBLTYPE * const restrict PRIM_INT__h_s_q_s = primwork + 126991;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_s_s = primwork + 132724;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_p_s = primwork + 133368;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_d_s = primwork + 134544;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_f_s = primwork + 136728;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_g_s = primwork + 140088;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_h_s = primwork + 144708;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_i_s = primwork + 150588;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_k_s = primwork + 157644;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_l_s = primwork + 165708;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_m_s = primwork + 174528;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_n_s = primwork + 183768;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_o_s = primwork + 193008;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_q_s = primwork + 201744;
    SIMINT_DBLTYPE * const restrict PRIM_INT__i_s_r_s = primwork + 209388;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_s_s = primwork + 215268;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_p_s = primwork + 216060;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_d_s = primwork + 217572;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_f_s = primwork + 220380;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_g_s = primwork + 224700;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_h_s = primwork + 230640;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_i_s = primwork + 238200;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_k_s = primwork + 247272;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_l_s = primwork + 257640;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_m_s = primwork + 268980;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_n_s = primwork + 280860;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_o_s = primwork + 292740;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_q_s = primwork + 303972;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_r_s = primwork + 313800;
    SIMINT_DBLTYPE * const restrict PRIM_INT__k_s_t_s = primwork + 321360;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_s_s = primwork + 325680;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_p_s = primwork + 326625;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_d_s = primwork + 328515;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_f_s = primwork + 332025;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_g_s = primwork + 337425;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_h_s = primwork + 344850;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_i_s = primwork + 354300;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_k_s = primwork + 365640;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_l_s = primwork + 378600;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_m_s = primwork + 392775;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_n_s = primwork + 407625;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_o_s = primwork + 422475;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_q_s = primwork + 436515;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_r_s = primwork + 448800;
    SIMINT_DBLTYPE * const restrict PRIM_INT__l_s_t_s = primwork + 458250;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_s_s = primwork + 463650;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_p_s = primwork + 464750;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_d_s = primwork + 467060;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_f_s = primwork + 471350;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_g_s = primwork + 477950;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_h_s = primwork + 487025;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_i_s = primwork + 498575;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_k_s = primwork + 512435;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_l_s = primwork + 528275;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_m_s = primwork + 545600;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_n_s = primwork + 563750;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_o_s = primwork + 581900;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_q_s = primwork + 599060;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_r_s = primwork + 614075;
    SIMINT_DBLTYPE * const restrict PRIM_INT__m_s_t_s = primwork + 625625;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_s_s = primwork + 632225;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_p_s = primwork + 633479;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_d_s = primwork + 636251;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_f_s = primwork + 641399;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_g_s = primwork + 649319;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_h_s = primwork + 660209;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_i_s = primwork + 674069;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_k_s = primwork + 690701;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_l_s = primwork + 709709;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_m_s = primwork + 730499;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_n_s = primwork + 752279;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_o_s = primwork + 774059;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_q_s = primwork + 794651;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_r_s = primwork + 812669;
    SIMINT_DBLTYPE * const restrict PRIM_INT__n_s_t_s = primwork + 826529;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_s_s = primwork + 834449;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_p_s = primwork + 835853;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_d_s = primwork + 839129;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_f_s = primwork + 845213;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_g_s = primwork + 854573;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_h_s = primwork + 867443;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_i_s = primwork + 883823;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_k_s = primwork + 903479;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_l_s = primwork + 925943;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_m_s = primwork + 950513;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_n_s = primwork + 976253;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_o_s = primwork + 1001993;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_q_s = primwork + 1026329;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_r_s = primwork + 1047623;
    SIMINT_DBLTYPE * const restrict PRIM_INT__o_s_t_s = primwork + 1064003;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_s_s = primwork + 1073363;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_p_s = primwork + 1074910;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_d_s = primwork + 1078732;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_f_s = primwork + 1085830;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_g_s = primwork + 1096750;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_h_s = primwork + 1111765;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_i_s = primwork + 1130875;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_k_s = primwork + 1153807;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_l_s = primwork + 1180015;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_m_s = primwork + 1208680;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_n_s = primwork + 1238710;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_o_s = primwork + 1268740;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_q_s = primwork + 1297132;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_r_s = primwork + 1321975;
    SIMINT_DBLTYPE * const restrict PRIM_INT__q_s_t_s = primwork + 1341085;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_s_s = primwork + 1352005;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_p_s = primwork + 1353685;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_d_s = primwork + 1358095;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_f_s = primwork + 1366285;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_g_s = primwork + 1378885;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_h_s = primwork + 1396210;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_i_s = primwork + 1418260;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_k_s = primwork + 1444720;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_l_s = primwork + 1474960;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_m_s = primwork + 1508035;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_n_s = primwork + 1542685;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_o_s = primwork + 1577335;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_q_s = primwork + 1610095;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_r_s = primwork + 1638760;
    SIMINT_DBLTYPE * const restrict PRIM_INT__r_s_t_s = primwork + 1660810;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_s_s = primwork + 1673410;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_p_s = primwork + 1675210;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_d_s = primwork + 1680250;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_f_s = primwork + 1689610;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_g_s = primwork + 1704010;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_h_s = primwork + 1723810;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_i_s = primwork + 1749010;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_k_s = primwork + 1779250;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_l_s = primwork + 1813810;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_m_s = primwork + 1851610;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_n_s = primwork + 1891210;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_o_s = primwork + 1930810;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_q_s = primwork + 1968250;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_r_s = primwork + 2001010;
    SIMINT_DBLTYPE * const restrict PRIM_INT__t_s_t_s = primwork + 2026210;
    double * const hrrwork = (double *)(primwork + 2040610);
    double * const HRR_INT__k_p_k_s = hrrwork + 0;
    double * const HRR_INT__k_p_l_s = hrrwork + 3888;
    double * const HRR_INT__k_p_m_s = hrrwork + 8748;
    double * const HRR_INT__k_p_n_s = hrrwork + 14688;
    double * const HRR_INT__k_p_o_s = hrrwork + 21816;
    double * const HRR_INT__k_p_q_s = hrrwork + 30240;
    double * const HRR_INT__k_p_r_s = hrrwork + 40068;
    double * const HRR_INT__k_p_t_s = hrrwork + 51408;
    double * const HRR_INT__k_d_k_s = hrrwork + 64368;
    double * const HRR_INT__k_d_l_s = hrrwork + 72144;
    double * const HRR_INT__k_d_m_s = hrrwork + 81864;
    double * const HRR_INT__k_d_n_s = hrrwork + 93744;
    double * const HRR_INT__k_d_o_s = hrrwork + 108000;
    double * const HRR_INT__k_d_q_s = hrrwork + 124848;
    double * const HRR_INT__k_d_r_s = hrrwork + 144504;
    double * const HRR_INT__k_d_t_s = hrrwork + 167184;
    double * const HRR_INT__k_f_k_s = hrrwork + 193104;
    double * const HRR_INT__k_f_l_s = hrrwork + 206064;
    double * const HRR_INT__k_f_m_s = hrrwork + 222264;
    double * const HRR_INT__k_f_n_s = hrrwork + 242064;
    double * const HRR_INT__k_f_o_s = hrrwork + 265824;
    double * const HRR_INT__k_f_q_s = hrrwork + 293904;
    double * const HRR_INT__k_f_r_s = hrrwork + 326664;
    double * const HRR_INT__k_f_t_s = hrrwork + 364464;
    double * const HRR_INT__k_g_k_s = hrrwork + 407664;
    double * const HRR_INT__k_g_l_s = hrrwork + 427104;
    double * const HRR_INT__k_g_m_s = hrrwork + 451404;
    double * const HRR_INT__k_g_n_s = hrrwork + 481104;
    double * const HRR_INT__k_g_o_s = hrrwork + 516744;
    double * const HRR_INT__k_g_q_s = hrrwork + 558864;
    double * const HRR_INT__k_g_r_s = hrrwork + 608004;
    double * const HRR_INT__k_g_t_s = hrrwork + 664704;
    double * const HRR_INT__k_h_k_s = hrrwork + 729504;
    double * const HRR_INT__k_h_l_s = hrrwork + 756720;
    double * const HRR_INT__k_h_m_s = hrrwork + 790740;
    double * const HRR_INT__k_h_n_s = hrrwork + 832320;
    double * const HRR_INT__k_h_o_s = hrrwork + 882216;
    double * const HRR_INT__k_h_q_s = hrrwork + 941184;
    double * const HRR_INT__k_h_r_s = hrrwork + 1009980;
    double * const HRR_INT__k_h_t_s = hrrwork + 1089360;
    double * const HRR_INT__k_i_k_s = hrrwork + 1180080;
    double * const HRR_INT__k_i_l_s = hrrwork + 1216368;
    double * const HRR_INT__k_i_m_s = hrrwork + 1261728;
    double * const HRR_INT__k_i_n_s = hrrwork + 1317168;
    double * const HRR_INT__k_i_o_s = hrrwork + 1383696;
    double * const HRR_INT__k_i_q_s = hrrwork + 1462320;
    double * const HRR_INT__k_i_r_s = hrrwork + 1554048;
    double * const HRR_INT__k_i_t_s = hrrwork + 1659888;
    double * const HRR_INT__k_k_k_s = hrrwork + 1780848;
    double * const HRR_INT__k_k_k_p = hrrwork + 1827504;
    double * const HRR_INT__k_k_k_d = hrrwork + 1967472;
    double * const HRR_INT__k_k_k_f = hrrwork + 2247408;
    double * const HRR_INT__k_k_k_g = hrrwork + 2713968;
    double * const HRR_INT__k_k_k_h = hrrwork + 3413808;
    double * const HRR_INT__k_k_k_i = hrrwork + 4393584;
    double * const HRR_INT__k_k_l_s = hrrwork + 5699952;
    double * const HRR_INT__k_k_l_p = hrrwork + 5758272;
    double * const HRR_INT__k_k_l_d = hrrwork + 5933232;
    double * const HRR_INT__k_k_l_f = hrrwork + 6283152;
    double * const HRR_INT__k_k_l_g = hrrwork + 6866352;
    double * const HRR_INT__k_k_l_h = hrrwork + 7741152;
    double * const HRR_INT__k_k_l_i = hrrwork + 8965872;
    double * const HRR_INT__k_k_m_s = hrrwork + 10598832;
    double * const HRR_INT__k_k_m_p = hrrwork + 10670112;
    double * const HRR_INT__k_k_m_d = hrrwork + 10883952;
    double * const HRR_INT__k_k_m_f = hrrwork + 11311632;
    double * const HRR_INT__k_k_m_g = hrrwork + 12024432;
    double * const HRR_INT__k_k_m_h = hrrwork + 13093632;
    double * const HRR_INT__k_k_n_s = hrrwork + 14590512;
    double * const HRR_INT__k_k_n_p = hrrwork + 14676048;
    double * const HRR_INT__k_k_n_d = hrrwork + 14932656;
    double * const HRR_INT__k_k_n_f = hrrwork + 15445872;
    double * const HRR_INT__k_k_n_g = hrrwork + 16301232;
    double * const HRR_INT__k_k_o_s = hrrwork + 17584272;
    double * const HRR_INT__k_k_o_p = hrrwork + 17685360;
    double * const HRR_INT__k_k_o_d = hrrwork + 17988624;
    double * const HRR_INT__k_k_o_f = hrrwork + 18595152;
    double * const HRR_INT__k_k_q_s = hrrwork + 19606032;
    double * const HRR_INT__k_k_q_p = hrrwork + 19723968;
    double * const HRR_INT__k_k_q_d = hrrwork + 20077776;
    double * const HRR_INT__k_k_r_s = hrrwork + 20785392;
    double * const HRR_INT__k_k_r_p = hrrwork + 20921472;
    double * const HRR_INT__k_k_t_s = hrrwork + 21329712;
    double * const HRR_INT__l_p_k_s = hrrwork + 21485232;
    double * const HRR_INT__l_p_l_s = hrrwork + 21490092;
    double * const HRR_INT__l_p_m_s = hrrwork + 21496167;
    double * const HRR_INT__l_p_n_s = hrrwork + 21503592;
    double * const HRR_INT__l_p_o_s = hrrwork + 21512502;
    double * const HRR_INT__l_p_q_s = hrrwork + 21523032;
    double * const HRR_INT__l_p_r_s = hrrwork + 21535317;
    double * const HRR_INT__l_p_t_s = hrrwork + 21549492;
    double * const HRR_INT__l_d_k_s = hrrwork + 21565692;
    double * const HRR_INT__l_d_l_s = hrrwork + 21575412;
    double * const HRR_INT__l_d_m_s = hrrwork + 21587562;
    double * const HRR_INT__l_d_n_s = hrrwork + 21602412;
    double * const HRR_INT__l_d_o_s = hrrwork + 21620232;
    double * const HRR_INT__l_d_q_s = hrrwork + 21641292;
    double * const HRR_INT__l_d_r_s = hrrwork + 21665862;
    double * const HRR_INT__l_d_t_s = hrrwork + 21694212;
    double * const HRR_INT__l_f_k_s = hrrwork + 21726612;
    double * const HRR_INT__l_f_l_s = hrrwork + 21742812;
    double * const HRR_INT__l_f_m_s = hrrwork + 21763062;
    double * const HRR_INT__l_f_n_s = hrrwork + 21787812;
    double * const HRR_INT__l_f_o_s = hrrwork + 21817512;
    double * const HRR_INT__l_f_q_s = hrrwork + 21852612;
    double * const HRR_INT__l_f_r_s = hrrwork + 21893562;
    double * const HRR_INT__l_f_t_s = hrrwork + 21940812;
    double * const HRR_INT__l_g_k_s = hrrwork + 21994812;
    double * const HRR_INT__l_g_l_s = hrrwork + 22019112;
    double * const HRR_INT__l_g_m_s = hrrwork + 22049487;
    double * const HRR_INT__l_g_n_s = hrrwork + 22086612;
    double * const HRR_INT__l_g_o_s = hrrwork + 22131162;
    double * const HRR_INT__l_g_q_s = hrrwork + 22183812;
    double * const HRR_INT__l_g_r_s = hrrwork + 22245237;
    double * const HRR_INT__l_g_t_s = hrrwork + 22316112;
    double * const HRR_INT__l_h_k_s = hrrwork + 22397112;
    double * const HRR_INT__l_h_l_s = hrrwork + 22431132;
    double * const HRR_INT__l_h_m_s = hrrwork + 22473657;
    double * const HRR_INT__l_h_n_s = hrrwork + 22525632;
    double * const HRR_INT__l_h_o_s = hrrwork + 22588002;
    double * const HRR_INT__l_h_q_s = hrrwork + 22661712;
    double * const HRR_INT__l_h_r_s = hrrwork + 22747707;
    double * const HRR_INT__l_h_t_s = hrrwork + 22846932;
    double * const HRR_INT__l_i_k_s = hrrwork + 22960332;
    double * const HRR_INT__l_i_l_s = hrrwork + 23005692;
    double * const HRR_INT__l_i_m_s = hrrwork + 23062392;
    double * const HRR_INT__l_i_n_s = hrrwork + 23131692;
    double * const HRR_INT__l_i_o_s = hrrwork + 23214852;
    double * const HRR_INT__l_i_q_s = hrrwork + 23313132;
    double * const HRR_INT__l_i_r_s = hrrwork + 23427792;
    double * const HRR_INT__l_i_t_s = hrrwork + 23560092;
    double * const HRR_INT__m_p_k_s = hrrwork + 23711292;
    double * const HRR_INT__m_p_l_s = hrrwork + 23717232;
    double * const HRR_INT__m_p_m_s = hrrwork + 23724657;
    double * const HRR_INT__m_p_n_s = hrrwork + 23733732;
    double * const HRR_INT__m_p_o_s = hrrwork + 23744622;
    double * const HRR_INT__m_p_q_s = hrrwork + 23757492;
    double * const HRR_INT__m_p_r_s = hrrwork + 23772507;
    double * const HRR_INT__m_p_t_s = hrrwork + 23789832;
    double * const HRR_INT__m_d_k_s = hrrwork + 23809632;
    double * const HRR_INT__m_d_l_s = hrrwork + 23821512;
    double * const HRR_INT__m_d_m_s = hrrwork + 23836362;
    double * const HRR_INT__m_d_n_s = hrrwork + 23854512;
    double * const HRR_INT__m_d_o_s = hrrwork + 23876292;
    double * const HRR_INT__m_d_q_s = hrrwork + 23902032;
    double * const HRR_INT__m_d_r_s = hrrwork + 23932062;
    double * const HRR_INT__m_d_t_s = hrrwork + 23966712;
    double * const HRR_INT__m_f_k_s = hrrwork + 24006312;
    double * const HRR_INT__m_f_l_s = hrrwork + 24026112;
    double * const HRR_INT__m_f_m_s = hrrwork + 24050862;
    double * const HRR_INT__m_f_n_s = hrrwork + 24081112;
    double * const HRR_INT__m_f_o_s = hrrwork + 24117412;
    double * const HRR_INT__m_f_q_s = hrrwork + 24160312;
    double * const HRR_INT__m_f_r_s = hrrwork + 24210362;
    double * const HRR_INT__m_f_t_s = hrrwork + 24268112;
    double * const HRR_INT__m_g_k_s = hrrwork + 24334112;
    double * const HRR_INT__m_g_l_s = hrrwork + 24363812;
    double * const HRR_INT__m_g_m_s = hrrwork + 24400937;
    double * const HRR_INT__m_g_n_s = hrrwork + 24446312;
    double * const HRR_INT__m_g_o_s = hrrwork + 24500762;
    double * const HRR_INT__m_g_q_s = hrrwork + 24565112;
    double * const HRR_INT__m_g_r_s = hrrwork + 24640187;
    double * const HRR_INT__m_g_t_s = hrrwork + 24726812;
    double * const HRR_INT__m_h_k_s = hrrwork + 24825812;
    double * const HRR_INT__m_h_l_s = hrrwork + 24867392;
    double * const HRR_INT__m_h_m_s = hrrwork + 24919367;
    double * const HRR_INT__m_h_n_s = hrrwork + 24982892;
    double * const HRR_INT__m_h_o_s = hrrwork + 25059122;
    double * const HRR_INT__m_h_q_s = hrrwork + 25149212;
    double * const HRR_INT__m_h_r_s = hrrwork + 25254317;
    double * const HRR_INT__m_h_t_s = hrrwork + 25375592;
    double * const HRR_INT__n_p_k_s = hrrwork + 25514192;
    double * const HRR_INT__n_p_l_s = hrrwork + 25521320;
    double * const HRR_INT__n_p_m_s = hrrwork + 25530230;
    double * const HRR_INT__n_p_n_s = hrrwork + 25541120;
    double * const HRR_INT__n_p_o_s = hrrwork + 25554188;
    double * const HRR_INT__n_p_q_s = hrrwork + 25569632;
    double * const HRR_INT__n_p_r_s = hrrwork + 25587650;
    double * const HRR_INT__n_p_t_s = hrrwork + 25608440;
    double * const HRR_INT__n_d_k_s = hrrwork + 25632200;
    double * const HRR_INT__n_d_l_s = hrrwork + 25646456;
    double * const HRR_INT__n_d_m_s = hrrwork + 25664276;
    double * const HRR_INT__n_d_n_s = hrrwork + 25686056;
    double * const HRR_INT__n_d_o_s = hrrwork + 25712192;
    double * const HRR_INT__n_d_q_s = hrrwork + 25743080;
    double * const HRR_INT__n_d_r_s = hrrwork + 25779116;
    double * const HRR_INT__n_d_t_s = hrrwork + 25820696;
    double * const HRR_INT__n_f_k_s = hrrwork + 25868216;
    double * const HRR_INT__n_f_l_s = hrrwork + 25891976;
    double * const HRR_INT__n_f_m_s = hrrwork + 25921676;
    double * const HRR_INT__n_f_n_s = hrrwork + 25957976;
    double * const HRR_INT__n_f_o_s = hrrwork + 26001536;
    double * const HRR_INT__n_f_q_s = hrrwork + 26053016;
    double * const HRR_INT__n_f_r_s = hrrwork + 26113076;
    double * const HRR_INT__n_f_t_s = hrrwork + 26182376;
    double * const HRR_INT__n_g_k_s = hrrwork + 26261576;
    double * const HRR_INT__n_g_l_s = hrrwork + 26297216;
    double * const HRR_INT__n_g_m_s = hrrwork + 26341766;
    double * const HRR_INT__n_g_n_s = hrrwork + 26396216;
    double * const HRR_INT__n_g_o_s = hrrwork + 26461556;
    double * const HRR_INT__n_g_q_s = hrrwork + 26538776;
    double * const HRR_INT__n_g_r_s = hrrwork + 26628866;
    double * const HRR_INT__n_g_t_s = hrrwork + 26732816;
    double * const HRR_INT__o_p_k_s = hrrwork + 26851616;
    double * const HRR_INT__o_p_l_s = hrrwork + 26860040;
    double * const HRR_INT__o_p_m_s = hrrwork + 26870570;
    double * const HRR_INT__o_p_n_s = hrrwork + 26883440;
    double * const HRR_INT__o_p_o_s = hrrwork + 26898884;
    double * const HRR_INT__o_p_q_s = hrrwork + 26917136;
    double * const HRR_INT__o_p_r_s = hrrwork + 26938430;
    double * const HRR_INT__o_p_t_s = hrrwork + 26963000;
    double * const HRR_INT__o_d_k_s = hrrwork + 26991080;
    double * const HRR_INT__o_d_l_s = hrrwork + 27007928;
    double * const HRR_INT__o_d_m_s = hrrwork + 27028988;
    double * const HRR_INT__o_d_n_s = hrrwork + 27054728;
    double * const HRR_INT__o_d_o_s = hrrwork + 27085616;
    double * const HRR_INT__o_d_q_s = hrrwork + 27122120;
    double * const HRR_INT__o_d_r_s = hrrwork + 27164708;
    double * const HRR_INT__o_d_t_s = hrrwork + 27213848;
    double * const HRR_INT__o_f_k_s = hrrwork + 27270008;
    double * const HRR_INT__o_f_l_s = hrrwork + 27298088;
    double * const HRR_INT__o_f_m_s = hrrwork + 27333188;
    double * const HRR_INT__o_f_n_s = hrrwork + 27376088;
    double * const HRR_INT__o_f_o_s = hrrwork + 27427568;
    double * const HRR_INT__o_f_q_s = hrrwork + 27488408;
    double * const HRR_INT__o_f_r_s = hrrwork + 27559388;
    double * const HRR_INT__o_f_t_s = hrrwork + 27641288;
    double * const HRR_INT__q_p_k_s = hrrwork + 27734888;
    double * const HRR_INT__q_p_l_s = hrrwork + 27744716;
    double * const HRR_INT__q_p_m_s = hrrwork + 27757001;
    double * const HRR_INT__q_p_n_s = hrrwork + 27772016;
    double * const HRR_INT__q_p_o_s = hrrwork + 27790034;
    double * const HRR_INT__q_p_q_s = hrrwork + 27811328;
    double * const HRR_INT__q_p_r_s = hrrwork + 27836171;
    double * const HRR_INT__q_p_t_s = hrrwork + 27864836;
    double * const HRR_INT__q_d_k_s = hrrwork + 27897596;
    double * const HRR_INT__q_d_l_s = hrrwork + 27917252;
    double * const HRR_INT__q_d_m_s = hrrwork + 27941822;
    double * const HRR_INT__q_d_n_s = hrrwork + 27971852;
    double * const HRR_INT__q_d_o_s = hrrwork + 28007888;
    double * const HRR_INT__q_d_q_s = hrrwork + 28050476;
    double * const HRR_INT__q_d_r_s = hrrwork + 28100162;
    double * const HRR_INT__q_d_t_s = hrrwork + 28157492;
    double * const HRR_INT__r_p_k_s = hrrwork + 28223012;
    double * const HRR_INT__r_p_l_s = hrrwork + 28234352;
    double * const HRR_INT__r_p_m_s = hrrwork + 28248527;
    double * const HRR_INT__r_p_n_s = hrrwork + 28265852;
    double * const HRR_INT__r_p_o_s = hrrwork + 28286642;
    double * const HRR_INT__r_p_q_s = hrrwork + 28311212;
    double * const HRR_INT__r_p_r_s = hrrwork + 28339877;
    double * const HRR_INT__r_p_t_s = hrrwork + 28372952;


    // Create constants
    const SIMINT_DBLTYPE const_1 = SIMINT_DBLSET1(1);
    const SIMINT_DBLTYPE const_10 = SIMINT_DBLSET1(10);
    const SIMINT_DBLTYPE const_11 = SIMINT_DBLSET1(11);
    const SIMINT_DBLTYPE const_12 = SIMINT_DBLSET1(12);
    const SIMINT_DBLTYPE const_13 = SIMINT_DBLSET1(13);
    const SIMINT_DBLTYPE const_14 = SIMINT_DBLSET1(14);
    const SIMINT_DBLTYPE const_2 = SIMINT_DBLSET1(2);
    const SIMINT_DBLTYPE const_3 = SIMINT_DBLSET1(3);
    const SIMINT_DBLTYPE const_4 = SIMINT_DBLSET1(4);
    const SIMINT_DBLTYPE const_5 = SIMINT_DBLSET1(5);
    const SIMINT_DBLTYPE const_6 = SIMINT_DBLSET1(6);
    const SIMINT_DBLTYPE const_7 = SIMINT_DBLSET1(7);
    const SIMINT_DBLTYPE const_8 = SIMINT_DBLSET1(8);
    const SIMINT_DBLTYPE const_9 = SIMINT_DBLSET1(9);
    const SIMINT_DBLTYPE one_half = SIMINT_DBLSET1(0.5);


    ////////////////////////////////////////
    // Loop over shells and primitives
    ////////////////////////////////////////

    real_abcd = 0;
    istart = 0;
    for(ab = 0; ab < P.nshell12_clip; ++ab)
    {
        const int iend = istart + P.nprim12[ab];

        cd = 0;
        jstart = 0;

        for(cd = 0; cd < Q.nshell12_clip; cd += SIMINT_NSHELL_SIMD)
        {
            const int nshellbatch = ((cd + SIMINT_NSHELL_SIMD) > Q.nshell12_clip) ? Q.nshell12_clip - cd : SIMINT_NSHELL_SIMD;
            int jend = jstart;
            for(i = 0; i < nshellbatch; i++)
                jend += Q.nprim12[cd+i];

            // Clear the beginning of the workspace (where we are accumulating integrals)
            memset(work, 0, SIMINT_NSHELL_SIMD * 355216 * sizeof(double));
            abcd = 0;


            for(i = istart; i < iend; ++i)
            {
                SIMINT_DBLTYPE bra_screen_max;  // only used if check_screen

                if(check_screen)
                {
                    // Skip this whole thing if always insignificant
                    if((P.screen[i] * Q.screen_max) < screen_tol)
                        continue;
                    bra_screen_max = SIMINT_DBLSET1(P.screen[i]);
                }

                icd = 0;
                iprimcd = 0;
                nprim_icd = Q.nprim12[cd];
                double * restrict PRIM_PTR_INT__k_s_k_s = INT__k_s_k_s + abcd * 1296;
                double * restrict PRIM_PTR_INT__k_s_l_s = INT__k_s_l_s + abcd * 1620;
                double * restrict PRIM_PTR_INT__k_s_m_s = INT__k_s_m_s + abcd * 1980;
                double * restrict PRIM_PTR_INT__k_s_n_s = INT__k_s_n_s + abcd * 2376;
                double * restrict PRIM_PTR_INT__k_s_o_s = INT__k_s_o_s + abcd * 2808;
                double * restrict PRIM_PTR_INT__k_s_q_s = INT__k_s_q_s + abcd * 3276;
                double * restrict PRIM_PTR_INT__k_s_r_s = INT__k_s_r_s + abcd * 3780;
                double * restrict PRIM_PTR_INT__k_s_t_s = INT__k_s_t_s + abcd * 4320;
                double * restrict PRIM_PTR_INT__l_s_k_s = INT__l_s_k_s + abcd * 1620;
                double * restrict PRIM_PTR_INT__l_s_l_s = INT__l_s_l_s + abcd * 2025;
                double * restrict PRIM_PTR_INT__l_s_m_s = INT__l_s_m_s + abcd * 2475;
                double * restrict PRIM_PTR_INT__l_s_n_s = INT__l_s_n_s + abcd * 2970;
                double * restrict PRIM_PTR_INT__l_s_o_s = INT__l_s_o_s + abcd * 3510;
                double * restrict PRIM_PTR_INT__l_s_q_s = INT__l_s_q_s + abcd * 4095;
                double * restrict PRIM_PTR_INT__l_s_r_s = INT__l_s_r_s + abcd * 4725;
                double * restrict PRIM_PTR_INT__l_s_t_s = INT__l_s_t_s + abcd * 5400;
                double * restrict PRIM_PTR_INT__m_s_k_s = INT__m_s_k_s + abcd * 1980;
                double * restrict PRIM_PTR_INT__m_s_l_s = INT__m_s_l_s + abcd * 2475;
                double * restrict PRIM_PTR_INT__m_s_m_s = INT__m_s_m_s + abcd * 3025;
                double * restrict PRIM_PTR_INT__m_s_n_s = INT__m_s_n_s + abcd * 3630;
                double * restrict PRIM_PTR_INT__m_s_o_s = INT__m_s_o_s + abcd * 4290;
                double * restrict PRIM_PTR_INT__m_s_q_s = INT__m_s_q_s + abcd * 5005;
                double * restrict PRIM_PTR_INT__m_s_r_s = INT__m_s_r_s + abcd * 5775;
                double * restrict PRIM_PTR_INT__m_s_t_s = INT__m_s_t_s + abcd * 6600;
                double * restrict PRIM_PTR_INT__n_s_k_s = INT__n_s_k_s + abcd * 2376;
                double * restrict PRIM_PTR_INT__n_s_l_s = INT__n_s_l_s + abcd * 2970;
                double * restrict PRIM_PTR_INT__n_s_m_s = INT__n_s_m_s + abcd * 3630;
                double * restrict PRIM_PTR_INT__n_s_n_s = INT__n_s_n_s + abcd * 4356;
                double * restrict PRIM_PTR_INT__n_s_o_s = INT__n_s_o_s + abcd * 5148;
                double * restrict PRIM_PTR_INT__n_s_q_s = INT__n_s_q_s + abcd * 6006;
                double * restrict PRIM_PTR_INT__n_s_r_s = INT__n_s_r_s + abcd * 6930;
                double * restrict PRIM_PTR_INT__n_s_t_s = INT__n_s_t_s + abcd * 7920;
                double * restrict PRIM_PTR_INT__o_s_k_s = INT__o_s_k_s + abcd * 2808;
                double * restrict PRIM_PTR_INT__o_s_l_s = INT__o_s_l_s + abcd * 3510;
                double * restrict PRIM_PTR_INT__o_s_m_s = INT__o_s_m_s + abcd * 4290;
                double * restrict PRIM_PTR_INT__o_s_n_s = INT__o_s_n_s + abcd * 5148;
                double * restrict PRIM_PTR_INT__o_s_o_s = INT__o_s_o_s + abcd * 6084;
                double * restrict PRIM_PTR_INT__o_s_q_s = INT__o_s_q_s + abcd * 7098;
                double * restrict PRIM_PTR_INT__o_s_r_s = INT__o_s_r_s + abcd * 8190;
                double * restrict PRIM_PTR_INT__o_s_t_s = INT__o_s_t_s + abcd * 9360;
                double * restrict PRIM_PTR_INT__q_s_k_s = INT__q_s_k_s + abcd * 3276;
                double * restrict PRIM_PTR_INT__q_s_l_s = INT__q_s_l_s + abcd * 4095;
                double * restrict PRIM_PTR_INT__q_s_m_s = INT__q_s_m_s + abcd * 5005;
                double * restrict PRIM_PTR_INT__q_s_n_s = INT__q_s_n_s + abcd * 6006;
                double * restrict PRIM_PTR_INT__q_s_o_s = INT__q_s_o_s + abcd * 7098;
                double * restrict PRIM_PTR_INT__q_s_q_s = INT__q_s_q_s + abcd * 8281;
                double * restrict PRIM_PTR_INT__q_s_r_s = INT__q_s_r_s + abcd * 9555;
                double * restrict PRIM_PTR_INT__q_s_t_s = INT__q_s_t_s + abcd * 10920;
                double * restrict PRIM_PTR_INT__r_s_k_s = INT__r_s_k_s + abcd * 3780;
                double * restrict PRIM_PTR_INT__r_s_l_s = INT__r_s_l_s + abcd * 4725;
                double * restrict PRIM_PTR_INT__r_s_m_s = INT__r_s_m_s + abcd * 5775;
                double * restrict PRIM_PTR_INT__r_s_n_s = INT__r_s_n_s + abcd * 6930;
                double * restrict PRIM_PTR_INT__r_s_o_s = INT__r_s_o_s + abcd * 8190;
                double * restrict PRIM_PTR_INT__r_s_q_s = INT__r_s_q_s + abcd * 9555;
                double * restrict PRIM_PTR_INT__r_s_r_s = INT__r_s_r_s + abcd * 11025;
                double * restrict PRIM_PTR_INT__r_s_t_s = INT__r_s_t_s + abcd * 12600;
                double * restrict PRIM_PTR_INT__t_s_k_s = INT__t_s_k_s + abcd * 4320;
                double * restrict PRIM_PTR_INT__t_s_l_s = INT__t_s_l_s + abcd * 5400;
                double * restrict PRIM_PTR_INT__t_s_m_s = INT__t_s_m_s + abcd * 6600;
                double * restrict PRIM_PTR_INT__t_s_n_s = INT__t_s_n_s + abcd * 7920;
                double * restrict PRIM_PTR_INT__t_s_o_s = INT__t_s_o_s + abcd * 9360;
                double * restrict PRIM_PTR_INT__t_s_q_s = INT__t_s_q_s + abcd * 10920;
                double * restrict PRIM_PTR_INT__t_s_r_s = INT__t_s_r_s + abcd * 12600;
                double * restrict PRIM_PTR_INT__t_s_t_s = INT__t_s_t_s + abcd * 14400;



                // Load these one per loop over i
                const SIMINT_DBLTYPE P_alpha = SIMINT_DBLSET1(P.alpha[i]);
                const SIMINT_DBLTYPE P_prefac = SIMINT_DBLSET1(P.prefac[i]);
                const SIMINT_DBLTYPE Pxyz[3] = { SIMINT_DBLSET1(P.x[i]), SIMINT_DBLSET1(P.y[i]), SIMINT_DBLSET1(P.z[i]) };

                const SIMINT_DBLTYPE P_PA[3] = { SIMINT_DBLSET1(P.PA_x[i]), SIMINT_DBLSET1(P.PA_y[i]), SIMINT_DBLSET1(P.PA_z[i]) };

                for(j = jstart; j < jend; j += SIMINT_SIMD_LEN)
                {
                    // calculate the shell offsets
                    // these are the offset from the shell pointed to by cd
                    // for each element
                    int shelloffsets[SIMINT_SIMD_LEN] = {0};
                    int lastoffset = 0;
                    const int nlane = ( ((j + SIMINT_SIMD_LEN) < jend) ? SIMINT_SIMD_LEN : (jend - j));

                    if((iprimcd + SIMINT_SIMD_LEN) >= nprim_icd)
                    {
                        // Handle if the first element of the vector is a new shell
                        if(iprimcd >= nprim_icd && ((icd+1) < nshellbatch))
                        {
                            nprim_icd += Q.nprim12[cd + (++icd)];
                            PRIM_PTR_INT__k_s_k_s += 1296;
                            PRIM_PTR_INT__k_s_l_s += 1620;
                            PRIM_PTR_INT__k_s_m_s += 1980;
                            PRIM_PTR_INT__k_s_n_s += 2376;
                            PRIM_PTR_INT__k_s_o_s += 2808;
                            PRIM_PTR_INT__k_s_q_s += 3276;
                            PRIM_PTR_INT__k_s_r_s += 3780;
                            PRIM_PTR_INT__k_s_t_s += 4320;
                            PRIM_PTR_INT__l_s_k_s += 1620;
                            PRIM_PTR_INT__l_s_l_s += 2025;
                            PRIM_PTR_INT__l_s_m_s += 2475;
                            PRIM_PTR_INT__l_s_n_s += 2970;
                            PRIM_PTR_INT__l_s_o_s += 3510;
                            PRIM_PTR_INT__l_s_q_s += 4095;
                            PRIM_PTR_INT__l_s_r_s += 4725;
                            PRIM_PTR_INT__l_s_t_s += 5400;
                            PRIM_PTR_INT__m_s_k_s += 1980;
                            PRIM_PTR_INT__m_s_l_s += 2475;
                            PRIM_PTR_INT__m_s_m_s += 3025;
                            PRIM_PTR_INT__m_s_n_s += 3630;
                            PRIM_PTR_INT__m_s_o_s += 4290;
                            PRIM_PTR_INT__m_s_q_s += 5005;
                            PRIM_PTR_INT__m_s_r_s += 5775;
                            PRIM_PTR_INT__m_s_t_s += 6600;
                            PRIM_PTR_INT__n_s_k_s += 2376;
                            PRIM_PTR_INT__n_s_l_s += 2970;
                            PRIM_PTR_INT__n_s_m_s += 3630;
                            PRIM_PTR_INT__n_s_n_s += 4356;
                            PRIM_PTR_INT__n_s_o_s += 5148;
                            PRIM_PTR_INT__n_s_q_s += 6006;
                            PRIM_PTR_INT__n_s_r_s += 6930;
                            PRIM_PTR_INT__n_s_t_s += 7920;
                            PRIM_PTR_INT__o_s_k_s += 2808;
                            PRIM_PTR_INT__o_s_l_s += 3510;
                            PRIM_PTR_INT__o_s_m_s += 4290;
                            PRIM_PTR_INT__o_s_n_s += 5148;
                            PRIM_PTR_INT__o_s_o_s += 6084;
                            PRIM_PTR_INT__o_s_q_s += 7098;
                            PRIM_PTR_INT__o_s_r_s += 8190;
                            PRIM_PTR_INT__o_s_t_s += 9360;
                            PRIM_PTR_INT__q_s_k_s += 3276;
                            PRIM_PTR_INT__q_s_l_s += 4095;
                            PRIM_PTR_INT__q_s_m_s += 5005;
                            PRIM_PTR_INT__q_s_n_s += 6006;
                            PRIM_PTR_INT__q_s_o_s += 7098;
                            PRIM_PTR_INT__q_s_q_s += 8281;
                            PRIM_PTR_INT__q_s_r_s += 9555;
                            PRIM_PTR_INT__q_s_t_s += 10920;
                            PRIM_PTR_INT__r_s_k_s += 3780;
                            PRIM_PTR_INT__r_s_l_s += 4725;
                            PRIM_PTR_INT__r_s_m_s += 5775;
                            PRIM_PTR_INT__r_s_n_s += 6930;
                            PRIM_PTR_INT__r_s_o_s += 8190;
                            PRIM_PTR_INT__r_s_q_s += 9555;
                            PRIM_PTR_INT__r_s_r_s += 11025;
                            PRIM_PTR_INT__r_s_t_s += 12600;
                            PRIM_PTR_INT__t_s_k_s += 4320;
                            PRIM_PTR_INT__t_s_l_s += 5400;
                            PRIM_PTR_INT__t_s_m_s += 6600;
                            PRIM_PTR_INT__t_s_n_s += 7920;
                            PRIM_PTR_INT__t_s_o_s += 9360;
                            PRIM_PTR_INT__t_s_q_s += 10920;
                            PRIM_PTR_INT__t_s_r_s += 12600;
                            PRIM_PTR_INT__t_s_t_s += 14400;
                        }
                        iprimcd++;
                        for(n = 1; n < SIMINT_SIMD_LEN; ++n)
                        {
                            if(iprimcd >= nprim_icd && ((icd+1) < nshellbatch))
                            {
                                shelloffsets[n] = shelloffsets[n-1] + 1;
                                lastoffset++;
                                nprim_icd += Q.nprim12[cd + (++icd)];
                            }
                            else
                                shelloffsets[n] = shelloffsets[n-1];
                            iprimcd++;
                        }
                    }
                    else
                        iprimcd += SIMINT_SIMD_LEN;

                    // Do we have to compute this vector (or has it been screened out)?
                    // (not_screened != 0 means we have to do this vector)
                    if(check_screen)
                    {
                        const double vmax = vector_max(SIMINT_MUL(bra_screen_max, SIMINT_DBLLOAD(Q.screen, j)));
                        if(vmax < screen_tol)
                        {
                            PRIM_PTR_INT__k_s_k_s += lastoffset*1296;
                            PRIM_PTR_INT__k_s_l_s += lastoffset*1620;
                            PRIM_PTR_INT__k_s_m_s += lastoffset*1980;
                            PRIM_PTR_INT__k_s_n_s += lastoffset*2376;
                            PRIM_PTR_INT__k_s_o_s += lastoffset*2808;
                            PRIM_PTR_INT__k_s_q_s += lastoffset*3276;
                            PRIM_PTR_INT__k_s_r_s += lastoffset*3780;
                            PRIM_PTR_INT__k_s_t_s += lastoffset*4320;
                            PRIM_PTR_INT__l_s_k_s += lastoffset*1620;
                            PRIM_PTR_INT__l_s_l_s += lastoffset*2025;
                            PRIM_PTR_INT__l_s_m_s += lastoffset*2475;
                            PRIM_PTR_INT__l_s_n_s += lastoffset*2970;
                            PRIM_PTR_INT__l_s_o_s += lastoffset*3510;
                            PRIM_PTR_INT__l_s_q_s += lastoffset*4095;
                            PRIM_PTR_INT__l_s_r_s += lastoffset*4725;
                            PRIM_PTR_INT__l_s_t_s += lastoffset*5400;
                            PRIM_PTR_INT__m_s_k_s += lastoffset*1980;
                            PRIM_PTR_INT__m_s_l_s += lastoffset*2475;
                            PRIM_PTR_INT__m_s_m_s += lastoffset*3025;
                            PRIM_PTR_INT__m_s_n_s += lastoffset*3630;
                            PRIM_PTR_INT__m_s_o_s += lastoffset*4290;
                            PRIM_PTR_INT__m_s_q_s += lastoffset*5005;
                            PRIM_PTR_INT__m_s_r_s += lastoffset*5775;
                            PRIM_PTR_INT__m_s_t_s += lastoffset*6600;
                            PRIM_PTR_INT__n_s_k_s += lastoffset*2376;
                            PRIM_PTR_INT__n_s_l_s += lastoffset*2970;
                            PRIM_PTR_INT__n_s_m_s += lastoffset*3630;
                            PRIM_PTR_INT__n_s_n_s += lastoffset*4356;
                            PRIM_PTR_INT__n_s_o_s += lastoffset*5148;
                            PRIM_PTR_INT__n_s_q_s += lastoffset*6006;
                            PRIM_PTR_INT__n_s_r_s += lastoffset*6930;
                            PRIM_PTR_INT__n_s_t_s += lastoffset*7920;
                            PRIM_PTR_INT__o_s_k_s += lastoffset*2808;
                            PRIM_PTR_INT__o_s_l_s += lastoffset*3510;
                            PRIM_PTR_INT__o_s_m_s += lastoffset*4290;
                            PRIM_PTR_INT__o_s_n_s += lastoffset*5148;
                            PRIM_PTR_INT__o_s_o_s += lastoffset*6084;
                            PRIM_PTR_INT__o_s_q_s += lastoffset*7098;
                            PRIM_PTR_INT__o_s_r_s += lastoffset*8190;
                            PRIM_PTR_INT__o_s_t_s += lastoffset*9360;
                            PRIM_PTR_INT__q_s_k_s += lastoffset*3276;
                            PRIM_PTR_INT__q_s_l_s += lastoffset*4095;
                            PRIM_PTR_INT__q_s_m_s += lastoffset*5005;
                            PRIM_PTR_INT__q_s_n_s += lastoffset*6006;
                            PRIM_PTR_INT__q_s_o_s += lastoffset*7098;
                            PRIM_PTR_INT__q_s_q_s += lastoffset*8281;
                            PRIM_PTR_INT__q_s_r_s += lastoffset*9555;
                            PRIM_PTR_INT__q_s_t_s += lastoffset*10920;
                            PRIM_PTR_INT__r_s_k_s += lastoffset*3780;
                            PRIM_PTR_INT__r_s_l_s += lastoffset*4725;
                            PRIM_PTR_INT__r_s_m_s += lastoffset*5775;
                            PRIM_PTR_INT__r_s_n_s += lastoffset*6930;
                            PRIM_PTR_INT__r_s_o_s += lastoffset*8190;
                            PRIM_PTR_INT__r_s_q_s += lastoffset*9555;
                            PRIM_PTR_INT__r_s_r_s += lastoffset*11025;
                            PRIM_PTR_INT__r_s_t_s += lastoffset*12600;
                            PRIM_PTR_INT__t_s_k_s += lastoffset*4320;
                            PRIM_PTR_INT__t_s_l_s += lastoffset*5400;
                            PRIM_PTR_INT__t_s_m_s += lastoffset*6600;
                            PRIM_PTR_INT__t_s_n_s += lastoffset*7920;
                            PRIM_PTR_INT__t_s_o_s += lastoffset*9360;
                            PRIM_PTR_INT__t_s_q_s += lastoffset*10920;
                            PRIM_PTR_INT__t_s_r_s += lastoffset*12600;
                            PRIM_PTR_INT__t_s_t_s += lastoffset*14400;
                            continue;
                        }
                    }

                    const SIMINT_DBLTYPE Q_alpha = SIMINT_DBLLOAD(Q.alpha, j);
                    const SIMINT_DBLTYPE PQalpha_mul = SIMINT_MUL(P_alpha, Q_alpha);
                    const SIMINT_DBLTYPE PQalpha_sum = SIMINT_ADD(P_alpha, Q_alpha);
                    const SIMINT_DBLTYPE one_over_PQalpha_sum = SIMINT_DIV(const_1, PQalpha_sum);


                    /* construct R2 = (Px - Qx)**2 + (Py - Qy)**2 + (Pz -Qz)**2 */
                    SIMINT_DBLTYPE PQ[3];
                    PQ[0] = SIMINT_SUB(Pxyz[0], SIMINT_DBLLOAD(Q.x, j));
                    PQ[1] = SIMINT_SUB(Pxyz[1], SIMINT_DBLLOAD(Q.y, j));
                    PQ[2] = SIMINT_SUB(Pxyz[2], SIMINT_DBLLOAD(Q.z, j));
                    SIMINT_DBLTYPE R2 = SIMINT_MUL(PQ[0], PQ[0]);
                    R2 = SIMINT_FMADD(PQ[1], PQ[1], R2);
                    R2 = SIMINT_FMADD(PQ[2], PQ[2], R2);

                    const SIMINT_DBLTYPE alpha = SIMINT_MUL(PQalpha_mul, one_over_PQalpha_sum); // alpha from MEST
                    const SIMINT_DBLTYPE one_over_p = SIMINT_DIV(const_1, P_alpha);
                    const SIMINT_DBLTYPE one_over_q = SIMINT_DIV(const_1, Q_alpha);
                    const SIMINT_DBLTYPE one_over_2p = SIMINT_MUL(one_half, one_over_p);
                    const SIMINT_DBLTYPE one_over_2q = SIMINT_MUL(one_half, one_over_q);
                    const SIMINT_DBLTYPE one_over_2pq = SIMINT_MUL(one_half, one_over_PQalpha_sum);
                    const SIMINT_DBLTYPE Q_PA[3] = { SIMINT_DBLLOAD(Q.PA_x, j), SIMINT_DBLLOAD(Q.PA_y, j), SIMINT_DBLLOAD(Q.PA_z, j) };

                    // NOTE: Minus sign!
                    const SIMINT_DBLTYPE a_over_p = SIMINT_MUL(SIMINT_NEG(alpha), one_over_p);
                    SIMINT_DBLTYPE aop_PQ[3];
                    aop_PQ[0] = SIMINT_MUL(a_over_p, PQ[0]);
                    aop_PQ[1] = SIMINT_MUL(a_over_p, PQ[1]);
                    aop_PQ[2] = SIMINT_MUL(a_over_p, PQ[2]);

                    SIMINT_DBLTYPE a_over_q = SIMINT_MUL(alpha, one_over_q);
                    SIMINT_DBLTYPE aoq_PQ[3];
                    aoq_PQ[0] = SIMINT_MUL(a_over_q, PQ[0]);
                    aoq_PQ[1] = SIMINT_MUL(a_over_q, PQ[1]);
                    aoq_PQ[2] = SIMINT_MUL(a_over_q, PQ[2]);
                    // Put a minus sign here so we don't have to in RR routines
                    a_over_q = SIMINT_NEG(a_over_q);


                    //////////////////////////////////////////////
                    // Fjt function section
                    // Maximum v value: 28
                    //////////////////////////////////////////////
                    // The parameter to the Fjt function
                    const SIMINT_DBLTYPE F_x = SIMINT_MUL(R2, alpha);


                    const SIMINT_DBLTYPE Q_prefac = mask_load(nlane, Q.prefac + j);


                    boys_F_split(PRIM_INT__s_s_s_s, F_x, 28);
                    SIMINT_DBLTYPE prefac = SIMINT_SQRT(one_over_PQalpha_sum);
                    prefac = SIMINT_MUL(SIMINT_MUL(P_prefac, Q_prefac), prefac);
                    for(n = 0; n <= 28; n++)
                        PRIM_INT__s_s_s_s[n] = SIMINT_MUL(PRIM_INT__s_s_s_s[n], prefac);

                    //////////////////////////////////////////////
                    // Primitive integrals: Vertical recurrance
                    //////////////////////////////////////////////

                    const SIMINT_DBLTYPE vrr_const_1_over_2p = one_over_2p;
                    const SIMINT_DBLTYPE vrr_const_2_over_2p = SIMINT_MUL(const_2, one_over_2p);
                    const SIMINT_DBLTYPE vrr_const_3_over_2p = SIMINT_MUL(const_3, one_over_2p);
                    const SIMINT_DBLTYPE vrr_const_4_over_2p = SIMINT_MUL(const_4, one_over_2p);
                    const SIMINT_DBLTYPE vrr_const_5_over_2p = SIMINT_MUL(const_5, one_over_2p);
                    const SIMINT_DBLTYPE vrr_const_6_over_2p = SIMINT_MUL(const_6, one_over_2p);
                    const SIMINT_DBLTYPE vrr_const_7_over_2p = SIMINT_MUL(const_7, one_over_2p);
                    const SIMINT_DBLTYPE vrr_const_8_over_2p = SIMINT_MUL(const_8, one_over_2p);
                    const SIMINT_DBLTYPE vrr_const_9_over_2p = SIMINT_MUL(const_9, one_over_2p);
                    const SIMINT_DBLTYPE vrr_const_10_over_2p = SIMINT_MUL(const_10, one_over_2p);
                    const SIMINT_DBLTYPE vrr_const_11_over_2p = SIMINT_MUL(const_11, one_over_2p);
                    const SIMINT_DBLTYPE vrr_const_12_over_2p = SIMINT_MUL(const_12, one_over_2p);
                    const SIMINT_DBLTYPE vrr_const_13_over_2p = SIMINT_MUL(const_13, one_over_2p);
                    const SIMINT_DBLTYPE vrr_const_1_over_2q = one_over_2q;
                    const SIMINT_DBLTYPE vrr_const_2_over_2q = SIMINT_MUL(const_2, one_over_2q);
                    const SIMINT_DBLTYPE vrr_const_3_over_2q = SIMINT_MUL(const_3, one_over_2q);
                    const SIMINT_DBLTYPE vrr_const_4_over_2q = SIMINT_MUL(const_4, one_over_2q);
                    const SIMINT_DBLTYPE vrr_const_5_over_2q = SIMINT_MUL(const_5, one_over_2q);
                    const SIMINT_DBLTYPE vrr_const_6_over_2q = SIMINT_MUL(const_6, one_over_2q);
                    const SIMINT_DBLTYPE vrr_const_7_over_2q = SIMINT_MUL(const_7, one_over_2q);
                    const SIMINT_DBLTYPE vrr_const_8_over_2q = SIMINT_MUL(const_8, one_over_2q);
                    const SIMINT_DBLTYPE vrr_const_9_over_2q = SIMINT_MUL(const_9, one_over_2q);
                    const SIMINT_DBLTYPE vrr_const_10_over_2q = SIMINT_MUL(const_10, one_over_2q);
                    const SIMINT_DBLTYPE vrr_const_11_over_2q = SIMINT_MUL(const_11, one_over_2q);
                    const SIMINT_DBLTYPE vrr_const_12_over_2q = SIMINT_MUL(const_12, one_over_2q);
                    const SIMINT_DBLTYPE vrr_const_13_over_2q = SIMINT_MUL(const_13, one_over_2q);
                    const SIMINT_DBLTYPE vrr_const_1_over_2pq = one_over_2pq;
                    const SIMINT_DBLTYPE vrr_const_2_over_2pq = SIMINT_MUL(const_2, one_over_2pq);
                    const SIMINT_DBLTYPE vrr_const_3_over_2pq = SIMINT_MUL(const_3, one_over_2pq);
                    const SIMINT_DBLTYPE vrr_const_4_over_2pq = SIMINT_MUL(const_4, one_over_2pq);
                    const SIMINT_DBLTYPE vrr_const_5_over_2pq = SIMINT_MUL(const_5, one_over_2pq);
                    const SIMINT_DBLTYPE vrr_const_6_over_2pq = SIMINT_MUL(const_6, one_over_2pq);
                    const SIMINT_DBLTYPE vrr_const_7_over_2pq = SIMINT_MUL(const_7, one_over_2pq);
                    const SIMINT_DBLTYPE vrr_const_8_over_2pq = SIMINT_MUL(const_8, one_over_2pq);
                    const SIMINT_DBLTYPE vrr_const_9_over_2pq = SIMINT_MUL(const_9, one_over_2pq);
                    const SIMINT_DBLTYPE vrr_const_10_over_2pq = SIMINT_MUL(const_10, one_over_2pq);
                    const SIMINT_DBLTYPE vrr_const_11_over_2pq = SIMINT_MUL(const_11, one_over_2pq);
                    const SIMINT_DBLTYPE vrr_const_12_over_2pq = SIMINT_MUL(const_12, one_over_2pq);
                    const SIMINT_DBLTYPE vrr_const_13_over_2pq = SIMINT_MUL(const_13, one_over_2pq);
                    const SIMINT_DBLTYPE vrr_const_14_over_2pq = SIMINT_MUL(const_14, one_over_2pq);



                    // Forming PRIM_INT__p_s_s_s[28 * 3];
                    for(n = 0; n < 28; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__p_s_s_s[n * 3 + 0] = SIMINT_MUL(P_PA[0], PRIM_INT__s_s_s_s[n * 1 + 0]);
                        PRIM_INT__p_s_s_s[n * 3 + 0] = SIMINT_FMADD( aop_PQ[0], PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__p_s_s_s[n * 3 + 0]);

                        PRIM_INT__p_s_s_s[n * 3 + 1] = SIMINT_MUL(P_PA[1], PRIM_INT__s_s_s_s[n * 1 + 0]);
                        PRIM_INT__p_s_s_s[n * 3 + 1] = SIMINT_FMADD( aop_PQ[1], PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__p_s_s_s[n * 3 + 1]);

                        PRIM_INT__p_s_s_s[n * 3 + 2] = SIMINT_MUL(P_PA[2], PRIM_INT__s_s_s_s[n * 1 + 0]);
                        PRIM_INT__p_s_s_s[n * 3 + 2] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__p_s_s_s[n * 3 + 2]);

                    }



                    // Forming PRIM_INT__d_s_s_s[27 * 6];
                    for(n = 0; n < 27; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__d_s_s_s[n * 6 + 0] = SIMINT_MUL(P_PA[0], PRIM_INT__p_s_s_s[n * 3 + 0]);
                        PRIM_INT__d_s_s_s[n * 6 + 0] = SIMINT_FMADD( aop_PQ[0], PRIM_INT__p_s_s_s[(n+1) * 3 + 0], PRIM_INT__d_s_s_s[n * 6 + 0]);
                        PRIM_INT__d_s_s_s[n * 6 + 0] = SIMINT_FMADD( vrr_const_1_over_2p, SIMINT_FMADD(a_over_p, PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__s_s_s_s[n * 1 + 0]), PRIM_INT__d_s_s_s[n * 6 + 0]);

                        PRIM_INT__d_s_s_s[n * 6 + 1] = SIMINT_MUL(P_PA[1], PRIM_INT__p_s_s_s[n * 3 + 0]);
                        PRIM_INT__d_s_s_s[n * 6 + 1] = SIMINT_FMADD( aop_PQ[1], PRIM_INT__p_s_s_s[(n+1) * 3 + 0], PRIM_INT__d_s_s_s[n * 6 + 1]);

                        PRIM_INT__d_s_s_s[n * 6 + 2] = SIMINT_MUL(P_PA[2], PRIM_INT__p_s_s_s[n * 3 + 0]);
                        PRIM_INT__d_s_s_s[n * 6 + 2] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__p_s_s_s[(n+1) * 3 + 0], PRIM_INT__d_s_s_s[n * 6 + 2]);

                        PRIM_INT__d_s_s_s[n * 6 + 3] = SIMINT_MUL(P_PA[1], PRIM_INT__p_s_s_s[n * 3 + 1]);
                        PRIM_INT__d_s_s_s[n * 6 + 3] = SIMINT_FMADD( aop_PQ[1], PRIM_INT__p_s_s_s[(n+1) * 3 + 1], PRIM_INT__d_s_s_s[n * 6 + 3]);
                        PRIM_INT__d_s_s_s[n * 6 + 3] = SIMINT_FMADD( vrr_const_1_over_2p, SIMINT_FMADD(a_over_p, PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__s_s_s_s[n * 1 + 0]), PRIM_INT__d_s_s_s[n * 6 + 3]);

                        PRIM_INT__d_s_s_s[n * 6 + 4] = SIMINT_MUL(P_PA[2], PRIM_INT__p_s_s_s[n * 3 + 1]);
                        PRIM_INT__d_s_s_s[n * 6 + 4] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__p_s_s_s[(n+1) * 3 + 1], PRIM_INT__d_s_s_s[n * 6 + 4]);

                        PRIM_INT__d_s_s_s[n * 6 + 5] = SIMINT_MUL(P_PA[2], PRIM_INT__p_s_s_s[n * 3 + 2]);
                        PRIM_INT__d_s_s_s[n * 6 + 5] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__p_s_s_s[(n+1) * 3 + 2], PRIM_INT__d_s_s_s[n * 6 + 5]);
                        PRIM_INT__d_s_s_s[n * 6 + 5] = SIMINT_FMADD( vrr_const_1_over_2p, SIMINT_FMADD(a_over_p, PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__s_s_s_s[n * 1 + 0]), PRIM_INT__d_s_s_s[n * 6 + 5]);

                    }



                    // Forming PRIM_INT__f_s_s_s[26 * 10];
                    for(n = 0; n < 26; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__f_s_s_s[n * 10 + 0] = SIMINT_MUL(P_PA[0], PRIM_INT__d_s_s_s[n * 6 + 0]);
                        PRIM_INT__f_s_s_s[n * 10 + 0] = SIMINT_FMADD( aop_PQ[0], PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__f_s_s_s[n * 10 + 0]);
                        PRIM_INT__f_s_s_s[n * 10 + 0] = SIMINT_FMADD( vrr_const_2_over_2p, SIMINT_FMADD(a_over_p, PRIM_INT__p_s_s_s[(n+1) * 3 + 0], PRIM_INT__p_s_s_s[n * 3 + 0]), PRIM_INT__f_s_s_s[n * 10 + 0]);

                        PRIM_INT__f_s_s_s[n * 10 + 1] = SIMINT_MUL(P_PA[1], PRIM_INT__d_s_s_s[n * 6 + 0]);
                        PRIM_INT__f_s_s_s[n * 10 + 1] = SIMINT_FMADD( aop_PQ[1], PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__f_s_s_s[n * 10 + 1]);

                        PRIM_INT__f_s_s_s[n * 10 + 2] = SIMINT_MUL(P_PA[2], PRIM_INT__d_s_s_s[n * 6 + 0]);
                        PRIM_INT__f_s_s_s[n * 10 + 2] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__f_s_s_s[n * 10 + 2]);

                        PRIM_INT__f_s_s_s[n * 10 + 3] = SIMINT_MUL(P_PA[0], PRIM_INT__d_s_s_s[n * 6 + 3]);
                        PRIM_INT__f_s_s_s[n * 10 + 3] = SIMINT_FMADD( aop_PQ[0], PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__f_s_s_s[n * 10 + 3]);

                        PRIM_INT__f_s_s_s[n * 10 + 4] = SIMINT_MUL(P_PA[2], PRIM_INT__d_s_s_s[n * 6 + 1]);
                        PRIM_INT__f_s_s_s[n * 10 + 4] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__d_s_s_s[(n+1) * 6 + 1], PRIM_INT__f_s_s_s[n * 10 + 4]);

                        PRIM_INT__f_s_s_s[n * 10 + 5] = SIMINT_MUL(P_PA[0], PRIM_INT__d_s_s_s[n * 6 + 5]);
                        PRIM_INT__f_s_s_s[n * 10 + 5] = SIMINT_FMADD( aop_PQ[0], PRIM_INT__d_s_s_s[(n+1) * 6 + 5], PRIM_INT__f_s_s_s[n * 10 + 5]);

                        PRIM_INT__f_s_s_s[n * 10 + 6] = SIMINT_MUL(P_PA[1], PRIM_INT__d_s_s_s[n * 6 + 3]);
                        PRIM_INT__f_s_s_s[n * 10 + 6] = SIMINT_FMADD( aop_PQ[1], PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__f_s_s_s[n * 10 + 6]);
                        PRIM_INT__f_s_s_s[n * 10 + 6] = SIMINT_FMADD( vrr_const_2_over_2p, SIMINT_FMADD(a_over_p, PRIM_INT__p_s_s_s[(n+1) * 3 + 1], PRIM_INT__p_s_s_s[n * 3 + 1]), PRIM_INT__f_s_s_s[n * 10 + 6]);

                        PRIM_INT__f_s_s_s[n * 10 + 7] = SIMINT_MUL(P_PA[2], PRIM_INT__d_s_s_s[n * 6 + 3]);
                        PRIM_INT__f_s_s_s[n * 10 + 7] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__f_s_s_s[n * 10 + 7]);

                        PRIM_INT__f_s_s_s[n * 10 + 8] = SIMINT_MUL(P_PA[1], PRIM_INT__d_s_s_s[n * 6 + 5]);
                        PRIM_INT__f_s_s_s[n * 10 + 8] = SIMINT_FMADD( aop_PQ[1], PRIM_INT__d_s_s_s[(n+1) * 6 + 5], PRIM_INT__f_s_s_s[n * 10 + 8]);

                        PRIM_INT__f_s_s_s[n * 10 + 9] = SIMINT_MUL(P_PA[2], PRIM_INT__d_s_s_s[n * 6 + 5]);
                        PRIM_INT__f_s_s_s[n * 10 + 9] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__d_s_s_s[(n+1) * 6 + 5], PRIM_INT__f_s_s_s[n * 10 + 9]);
                        PRIM_INT__f_s_s_s[n * 10 + 9] = SIMINT_FMADD( vrr_const_2_over_2p, SIMINT_FMADD(a_over_p, PRIM_INT__p_s_s_s[(n+1) * 3 + 2], PRIM_INT__p_s_s_s[n * 3 + 2]), PRIM_INT__f_s_s_s[n * 10 + 9]);

                    }



                    // Forming PRIM_INT__g_s_s_s[25 * 15];
                    for(n = 0; n < 25; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__g_s_s_s[n * 15 + 0] = SIMINT_MUL(P_PA[0], PRIM_INT__f_s_s_s[n * 10 + 0]);
                        PRIM_INT__g_s_s_s[n * 15 + 0] = SIMINT_FMADD( aop_PQ[0], PRIM_INT__f_s_s_s[(n+1) * 10 + 0], PRIM_INT__g_s_s_s[n * 15 + 0]);
                        PRIM_INT__g_s_s_s[n * 15 + 0] = SIMINT_FMADD( vrr_const_3_over_2p, SIMINT_FMADD(a_over_p, PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__d_s_s_s[n * 6 + 0]), PRIM_INT__g_s_s_s[n * 15 + 0]);

                        PRIM_INT__g_s_s_s[n * 15 + 1] = SIMINT_MUL(P_PA[1], PRIM_INT__f_s_s_s[n * 10 + 0]);
                        PRIM_INT__g_s_s_s[n * 15 + 1] = SIMINT_FMADD( aop_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 0], PRIM_INT__g_s_s_s[n * 15 + 1]);

                        PRIM_INT__g_s_s_s[n * 15 + 2] = SIMINT_MUL(P_PA[2], PRIM_INT__f_s_s_s[n * 10 + 0]);
                        PRIM_INT__g_s_s_s[n * 15 + 2] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 0], PRIM_INT__g_s_s_s[n * 15 + 2]);

                        PRIM_INT__g_s_s_s[n * 15 + 3] = SIMINT_MUL(P_PA[1], PRIM_INT__f_s_s_s[n * 10 + 1]);
                        PRIM_INT__g_s_s_s[n * 15 + 3] = SIMINT_FMADD( aop_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 1], PRIM_INT__g_s_s_s[n * 15 + 3]);
                        PRIM_INT__g_s_s_s[n * 15 + 3] = SIMINT_FMADD( vrr_const_1_over_2p, SIMINT_FMADD(a_over_p, PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__d_s_s_s[n * 6 + 0]), PRIM_INT__g_s_s_s[n * 15 + 3]);

                        PRIM_INT__g_s_s_s[n * 15 + 4] = SIMINT_MUL(P_PA[2], PRIM_INT__f_s_s_s[n * 10 + 1]);
                        PRIM_INT__g_s_s_s[n * 15 + 4] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 1], PRIM_INT__g_s_s_s[n * 15 + 4]);

                        PRIM_INT__g_s_s_s[n * 15 + 5] = SIMINT_MUL(P_PA[2], PRIM_INT__f_s_s_s[n * 10 + 2]);
                        PRIM_INT__g_s_s_s[n * 15 + 5] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 2], PRIM_INT__g_s_s_s[n * 15 + 5]);
                        PRIM_INT__g_s_s_s[n * 15 + 5] = SIMINT_FMADD( vrr_const_1_over_2p, SIMINT_FMADD(a_over_p, PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__d_s_s_s[n * 6 + 0]), PRIM_INT__g_s_s_s[n * 15 + 5]);

                        PRIM_INT__g_s_s_s[n * 15 + 6] = SIMINT_MUL(P_PA[0], PRIM_INT__f_s_s_s[n * 10 + 6]);
                        PRIM_INT__g_s_s_s[n * 15 + 6] = SIMINT_FMADD( aop_PQ[0], PRIM_INT__f_s_s_s[(n+1) * 10 + 6], PRIM_INT__g_s_s_s[n * 15 + 6]);

                        PRIM_INT__g_s_s_s[n * 15 + 7] = SIMINT_MUL(P_PA[2], PRIM_INT__f_s_s_s[n * 10 + 3]);
                        PRIM_INT__g_s_s_s[n * 15 + 7] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 3], PRIM_INT__g_s_s_s[n * 15 + 7]);

                        PRIM_INT__g_s_s_s[n * 15 + 8] = SIMINT_MUL(P_PA[1], PRIM_INT__f_s_s_s[n * 10 + 5]);
                        PRIM_INT__g_s_s_s[n * 15 + 8] = SIMINT_FMADD( aop_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 5], PRIM_INT__g_s_s_s[n * 15 + 8]);

                        PRIM_INT__g_s_s_s[n * 15 + 9] = SIMINT_MUL(P_PA[0], PRIM_INT__f_s_s_s[n * 10 + 9]);
                        PRIM_INT__g_s_s_s[n * 15 + 9] = SIMINT_FMADD( aop_PQ[0], PRIM_INT__f_s_s_s[(n+1) * 10 + 9], PRIM_INT__g_s_s_s[n * 15 + 9]);

                        PRIM_INT__g_s_s_s[n * 15 + 10] = SIMINT_MUL(P_PA[1], PRIM_INT__f_s_s_s[n * 10 + 6]);
                        PRIM_INT__g_s_s_s[n * 15 + 10] = SIMINT_FMADD( aop_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 6], PRIM_INT__g_s_s_s[n * 15 + 10]);
                        PRIM_INT__g_s_s_s[n * 15 + 10] = SIMINT_FMADD( vrr_const_3_over_2p, SIMINT_FMADD(a_over_p, PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__d_s_s_s[n * 6 + 3]), PRIM_INT__g_s_s_s[n * 15 + 10]);

                        PRIM_INT__g_s_s_s[n * 15 + 11] = SIMINT_MUL(P_PA[2], PRIM_INT__f_s_s_s[n * 10 + 6]);
                        PRIM_INT__g_s_s_s[n * 15 + 11] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 6], PRIM_INT__g_s_s_s[n * 15 + 11]);

                        PRIM_INT__g_s_s_s[n * 15 + 12] = SIMINT_MUL(P_PA[2], PRIM_INT__f_s_s_s[n * 10 + 7]);
                        PRIM_INT__g_s_s_s[n * 15 + 12] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 7], PRIM_INT__g_s_s_s[n * 15 + 12]);
                        PRIM_INT__g_s_s_s[n * 15 + 12] = SIMINT_FMADD( vrr_const_1_over_2p, SIMINT_FMADD(a_over_p, PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__d_s_s_s[n * 6 + 3]), PRIM_INT__g_s_s_s[n * 15 + 12]);

                        PRIM_INT__g_s_s_s[n * 15 + 13] = SIMINT_MUL(P_PA[1], PRIM_INT__f_s_s_s[n * 10 + 9]);
                        PRIM_INT__g_s_s_s[n * 15 + 13] = SIMINT_FMADD( aop_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 9], PRIM_INT__g_s_s_s[n * 15 + 13]);

                        PRIM_INT__g_s_s_s[n * 15 + 14] = SIMINT_MUL(P_PA[2], PRIM_INT__f_s_s_s[n * 10 + 9]);
                        PRIM_INT__g_s_s_s[n * 15 + 14] = SIMINT_FMADD( aop_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 9], PRIM_INT__g_s_s_s[n * 15 + 14]);
                        PRIM_INT__g_s_s_s[n * 15 + 14] = SIMINT_FMADD( vrr_const_3_over_2p, SIMINT_FMADD(a_over_p, PRIM_INT__d_s_s_s[(n+1) * 6 + 5], PRIM_INT__d_s_s_s[n * 6 + 5]), PRIM_INT__g_s_s_s[n * 15 + 14]);

                    }


                    VRR_I_h_s_s_s(
                            PRIM_INT__h_s_s_s,
                            PRIM_INT__g_s_s_s,
                            PRIM_INT__f_s_s_s,
                            P_PA,
                            a_over_p,
                            aop_PQ,
                            one_over_2p,
                            24);


                    VRR_I_i_s_s_s(
                            PRIM_INT__i_s_s_s,
                            PRIM_INT__h_s_s_s,
                            PRIM_INT__g_s_s_s,
                            P_PA,
                            a_over_p,
                            aop_PQ,
                            one_over_2p,
                            23);


                    VRR_I_k_s_s_s(
                            PRIM_INT__k_s_s_s,
                            PRIM_INT__i_s_s_s,
                            PRIM_INT__h_s_s_s,
                            P_PA,
                            a_over_p,
                            aop_PQ,
                            one_over_2p,
                            22);


                    ostei_general_vrr_K(7, 0, 1, 0, 14,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_s_s, NULL, NULL, PRIM_INT__i_s_s_s, NULL, PRIM_INT__k_s_p_s);


                    VRR_K_i_s_p_s(
                            PRIM_INT__i_s_p_s,
                            PRIM_INT__i_s_s_s,
                            PRIM_INT__h_s_s_s,
                            Q_PA,
                            aoq_PQ,
                            one_over_2pq,
                            14);


                    ostei_general_vrr_K(7, 0, 2, 0, 13,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_p_s, PRIM_INT__k_s_s_s, NULL, PRIM_INT__i_s_p_s, NULL, PRIM_INT__k_s_d_s);


                    VRR_K_h_s_p_s(
                            PRIM_INT__h_s_p_s,
                            PRIM_INT__h_s_s_s,
                            PRIM_INT__g_s_s_s,
                            Q_PA,
                            aoq_PQ,
                            one_over_2pq,
                            14);


                    ostei_general_vrr_K(6, 0, 2, 0, 13,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__i_s_p_s, PRIM_INT__i_s_s_s, NULL, PRIM_INT__h_s_p_s, NULL, PRIM_INT__i_s_d_s);


                    ostei_general_vrr_K(7, 0, 3, 0, 12,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_d_s, PRIM_INT__k_s_p_s, NULL, PRIM_INT__i_s_d_s, NULL, PRIM_INT__k_s_f_s);


                    VRR_K_g_s_p_s(
                            PRIM_INT__g_s_p_s,
                            PRIM_INT__g_s_s_s,
                            PRIM_INT__f_s_s_s,
                            Q_PA,
                            aoq_PQ,
                            one_over_2pq,
                            14);


                    VRR_K_h_s_d_s(
                            PRIM_INT__h_s_d_s,
                            PRIM_INT__h_s_p_s,
                            PRIM_INT__h_s_s_s,
                            PRIM_INT__g_s_p_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2pq,
                            one_over_2q,
                            13);


                    ostei_general_vrr_K(6, 0, 3, 0, 12,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__i_s_d_s, PRIM_INT__i_s_p_s, NULL, PRIM_INT__h_s_d_s, NULL, PRIM_INT__i_s_f_s);


                    ostei_general_vrr_K(7, 0, 4, 0, 11,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_f_s, PRIM_INT__k_s_d_s, NULL, PRIM_INT__i_s_f_s, NULL, PRIM_INT__k_s_g_s);



                    // Forming PRIM_INT__f_s_p_s[14 * 30];
                    for(n = 0; n < 14; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__f_s_p_s[n * 30 + 0] = SIMINT_MUL(Q_PA[0], PRIM_INT__f_s_s_s[n * 10 + 0]);
                        PRIM_INT__f_s_p_s[n * 30 + 0] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__f_s_s_s[(n+1) * 10 + 0], PRIM_INT__f_s_p_s[n * 30 + 0]);
                        PRIM_INT__f_s_p_s[n * 30 + 0] = SIMINT_FMADD( vrr_const_3_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__f_s_p_s[n * 30 + 0]);

                        PRIM_INT__f_s_p_s[n * 30 + 1] = SIMINT_MUL(Q_PA[1], PRIM_INT__f_s_s_s[n * 10 + 0]);
                        PRIM_INT__f_s_p_s[n * 30 + 1] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 0], PRIM_INT__f_s_p_s[n * 30 + 1]);

                        PRIM_INT__f_s_p_s[n * 30 + 2] = SIMINT_MUL(Q_PA[2], PRIM_INT__f_s_s_s[n * 10 + 0]);
                        PRIM_INT__f_s_p_s[n * 30 + 2] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 0], PRIM_INT__f_s_p_s[n * 30 + 2]);

                        PRIM_INT__f_s_p_s[n * 30 + 3] = SIMINT_MUL(Q_PA[0], PRIM_INT__f_s_s_s[n * 10 + 1]);
                        PRIM_INT__f_s_p_s[n * 30 + 3] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__f_s_s_s[(n+1) * 10 + 1], PRIM_INT__f_s_p_s[n * 30 + 3]);
                        PRIM_INT__f_s_p_s[n * 30 + 3] = SIMINT_FMADD( vrr_const_2_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 1], PRIM_INT__f_s_p_s[n * 30 + 3]);

                        PRIM_INT__f_s_p_s[n * 30 + 4] = SIMINT_MUL(Q_PA[1], PRIM_INT__f_s_s_s[n * 10 + 1]);
                        PRIM_INT__f_s_p_s[n * 30 + 4] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 1], PRIM_INT__f_s_p_s[n * 30 + 4]);
                        PRIM_INT__f_s_p_s[n * 30 + 4] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__f_s_p_s[n * 30 + 4]);

                        PRIM_INT__f_s_p_s[n * 30 + 5] = SIMINT_MUL(Q_PA[2], PRIM_INT__f_s_s_s[n * 10 + 1]);
                        PRIM_INT__f_s_p_s[n * 30 + 5] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 1], PRIM_INT__f_s_p_s[n * 30 + 5]);

                        PRIM_INT__f_s_p_s[n * 30 + 6] = SIMINT_MUL(Q_PA[0], PRIM_INT__f_s_s_s[n * 10 + 2]);
                        PRIM_INT__f_s_p_s[n * 30 + 6] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__f_s_s_s[(n+1) * 10 + 2], PRIM_INT__f_s_p_s[n * 30 + 6]);
                        PRIM_INT__f_s_p_s[n * 30 + 6] = SIMINT_FMADD( vrr_const_2_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 2], PRIM_INT__f_s_p_s[n * 30 + 6]);

                        PRIM_INT__f_s_p_s[n * 30 + 7] = SIMINT_MUL(Q_PA[1], PRIM_INT__f_s_s_s[n * 10 + 2]);
                        PRIM_INT__f_s_p_s[n * 30 + 7] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 2], PRIM_INT__f_s_p_s[n * 30 + 7]);

                        PRIM_INT__f_s_p_s[n * 30 + 8] = SIMINT_MUL(Q_PA[2], PRIM_INT__f_s_s_s[n * 10 + 2]);
                        PRIM_INT__f_s_p_s[n * 30 + 8] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 2], PRIM_INT__f_s_p_s[n * 30 + 8]);
                        PRIM_INT__f_s_p_s[n * 30 + 8] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__f_s_p_s[n * 30 + 8]);

                        PRIM_INT__f_s_p_s[n * 30 + 9] = SIMINT_MUL(Q_PA[0], PRIM_INT__f_s_s_s[n * 10 + 3]);
                        PRIM_INT__f_s_p_s[n * 30 + 9] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__f_s_s_s[(n+1) * 10 + 3], PRIM_INT__f_s_p_s[n * 30 + 9]);
                        PRIM_INT__f_s_p_s[n * 30 + 9] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__f_s_p_s[n * 30 + 9]);

                        PRIM_INT__f_s_p_s[n * 30 + 10] = SIMINT_MUL(Q_PA[1], PRIM_INT__f_s_s_s[n * 10 + 3]);
                        PRIM_INT__f_s_p_s[n * 30 + 10] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 3], PRIM_INT__f_s_p_s[n * 30 + 10]);
                        PRIM_INT__f_s_p_s[n * 30 + 10] = SIMINT_FMADD( vrr_const_2_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 1], PRIM_INT__f_s_p_s[n * 30 + 10]);

                        PRIM_INT__f_s_p_s[n * 30 + 11] = SIMINT_MUL(Q_PA[2], PRIM_INT__f_s_s_s[n * 10 + 3]);
                        PRIM_INT__f_s_p_s[n * 30 + 11] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 3], PRIM_INT__f_s_p_s[n * 30 + 11]);

                        PRIM_INT__f_s_p_s[n * 30 + 12] = SIMINT_MUL(Q_PA[0], PRIM_INT__f_s_s_s[n * 10 + 4]);
                        PRIM_INT__f_s_p_s[n * 30 + 12] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__f_s_s_s[(n+1) * 10 + 4], PRIM_INT__f_s_p_s[n * 30 + 12]);
                        PRIM_INT__f_s_p_s[n * 30 + 12] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 4], PRIM_INT__f_s_p_s[n * 30 + 12]);

                        PRIM_INT__f_s_p_s[n * 30 + 13] = SIMINT_MUL(Q_PA[1], PRIM_INT__f_s_s_s[n * 10 + 4]);
                        PRIM_INT__f_s_p_s[n * 30 + 13] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 4], PRIM_INT__f_s_p_s[n * 30 + 13]);
                        PRIM_INT__f_s_p_s[n * 30 + 13] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 2], PRIM_INT__f_s_p_s[n * 30 + 13]);

                        PRIM_INT__f_s_p_s[n * 30 + 14] = SIMINT_MUL(Q_PA[2], PRIM_INT__f_s_s_s[n * 10 + 4]);
                        PRIM_INT__f_s_p_s[n * 30 + 14] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 4], PRIM_INT__f_s_p_s[n * 30 + 14]);
                        PRIM_INT__f_s_p_s[n * 30 + 14] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 1], PRIM_INT__f_s_p_s[n * 30 + 14]);

                        PRIM_INT__f_s_p_s[n * 30 + 15] = SIMINT_MUL(Q_PA[0], PRIM_INT__f_s_s_s[n * 10 + 5]);
                        PRIM_INT__f_s_p_s[n * 30 + 15] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__f_s_s_s[(n+1) * 10 + 5], PRIM_INT__f_s_p_s[n * 30 + 15]);
                        PRIM_INT__f_s_p_s[n * 30 + 15] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 5], PRIM_INT__f_s_p_s[n * 30 + 15]);

                        PRIM_INT__f_s_p_s[n * 30 + 16] = SIMINT_MUL(Q_PA[1], PRIM_INT__f_s_s_s[n * 10 + 5]);
                        PRIM_INT__f_s_p_s[n * 30 + 16] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 5], PRIM_INT__f_s_p_s[n * 30 + 16]);

                        PRIM_INT__f_s_p_s[n * 30 + 17] = SIMINT_MUL(Q_PA[2], PRIM_INT__f_s_s_s[n * 10 + 5]);
                        PRIM_INT__f_s_p_s[n * 30 + 17] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 5], PRIM_INT__f_s_p_s[n * 30 + 17]);
                        PRIM_INT__f_s_p_s[n * 30 + 17] = SIMINT_FMADD( vrr_const_2_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 2], PRIM_INT__f_s_p_s[n * 30 + 17]);

                        PRIM_INT__f_s_p_s[n * 30 + 18] = SIMINT_MUL(Q_PA[0], PRIM_INT__f_s_s_s[n * 10 + 6]);
                        PRIM_INT__f_s_p_s[n * 30 + 18] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__f_s_s_s[(n+1) * 10 + 6], PRIM_INT__f_s_p_s[n * 30 + 18]);

                        PRIM_INT__f_s_p_s[n * 30 + 19] = SIMINT_MUL(Q_PA[1], PRIM_INT__f_s_s_s[n * 10 + 6]);
                        PRIM_INT__f_s_p_s[n * 30 + 19] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 6], PRIM_INT__f_s_p_s[n * 30 + 19]);
                        PRIM_INT__f_s_p_s[n * 30 + 19] = SIMINT_FMADD( vrr_const_3_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__f_s_p_s[n * 30 + 19]);

                        PRIM_INT__f_s_p_s[n * 30 + 20] = SIMINT_MUL(Q_PA[2], PRIM_INT__f_s_s_s[n * 10 + 6]);
                        PRIM_INT__f_s_p_s[n * 30 + 20] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 6], PRIM_INT__f_s_p_s[n * 30 + 20]);

                        PRIM_INT__f_s_p_s[n * 30 + 21] = SIMINT_MUL(Q_PA[0], PRIM_INT__f_s_s_s[n * 10 + 7]);
                        PRIM_INT__f_s_p_s[n * 30 + 21] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__f_s_s_s[(n+1) * 10 + 7], PRIM_INT__f_s_p_s[n * 30 + 21]);

                        PRIM_INT__f_s_p_s[n * 30 + 22] = SIMINT_MUL(Q_PA[1], PRIM_INT__f_s_s_s[n * 10 + 7]);
                        PRIM_INT__f_s_p_s[n * 30 + 22] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 7], PRIM_INT__f_s_p_s[n * 30 + 22]);
                        PRIM_INT__f_s_p_s[n * 30 + 22] = SIMINT_FMADD( vrr_const_2_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 4], PRIM_INT__f_s_p_s[n * 30 + 22]);

                        PRIM_INT__f_s_p_s[n * 30 + 23] = SIMINT_MUL(Q_PA[2], PRIM_INT__f_s_s_s[n * 10 + 7]);
                        PRIM_INT__f_s_p_s[n * 30 + 23] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 7], PRIM_INT__f_s_p_s[n * 30 + 23]);
                        PRIM_INT__f_s_p_s[n * 30 + 23] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__f_s_p_s[n * 30 + 23]);

                        PRIM_INT__f_s_p_s[n * 30 + 24] = SIMINT_MUL(Q_PA[0], PRIM_INT__f_s_s_s[n * 10 + 8]);
                        PRIM_INT__f_s_p_s[n * 30 + 24] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__f_s_s_s[(n+1) * 10 + 8], PRIM_INT__f_s_p_s[n * 30 + 24]);

                        PRIM_INT__f_s_p_s[n * 30 + 25] = SIMINT_MUL(Q_PA[1], PRIM_INT__f_s_s_s[n * 10 + 8]);
                        PRIM_INT__f_s_p_s[n * 30 + 25] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 8], PRIM_INT__f_s_p_s[n * 30 + 25]);
                        PRIM_INT__f_s_p_s[n * 30 + 25] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 5], PRIM_INT__f_s_p_s[n * 30 + 25]);

                        PRIM_INT__f_s_p_s[n * 30 + 26] = SIMINT_MUL(Q_PA[2], PRIM_INT__f_s_s_s[n * 10 + 8]);
                        PRIM_INT__f_s_p_s[n * 30 + 26] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 8], PRIM_INT__f_s_p_s[n * 30 + 26]);
                        PRIM_INT__f_s_p_s[n * 30 + 26] = SIMINT_FMADD( vrr_const_2_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 4], PRIM_INT__f_s_p_s[n * 30 + 26]);

                        PRIM_INT__f_s_p_s[n * 30 + 27] = SIMINT_MUL(Q_PA[0], PRIM_INT__f_s_s_s[n * 10 + 9]);
                        PRIM_INT__f_s_p_s[n * 30 + 27] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__f_s_s_s[(n+1) * 10 + 9], PRIM_INT__f_s_p_s[n * 30 + 27]);

                        PRIM_INT__f_s_p_s[n * 30 + 28] = SIMINT_MUL(Q_PA[1], PRIM_INT__f_s_s_s[n * 10 + 9]);
                        PRIM_INT__f_s_p_s[n * 30 + 28] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__f_s_s_s[(n+1) * 10 + 9], PRIM_INT__f_s_p_s[n * 30 + 28]);

                        PRIM_INT__f_s_p_s[n * 30 + 29] = SIMINT_MUL(Q_PA[2], PRIM_INT__f_s_s_s[n * 10 + 9]);
                        PRIM_INT__f_s_p_s[n * 30 + 29] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__f_s_s_s[(n+1) * 10 + 9], PRIM_INT__f_s_p_s[n * 30 + 29]);
                        PRIM_INT__f_s_p_s[n * 30 + 29] = SIMINT_FMADD( vrr_const_3_over_2pq, PRIM_INT__d_s_s_s[(n+1) * 6 + 5], PRIM_INT__f_s_p_s[n * 30 + 29]);

                    }


                    VRR_K_g_s_d_s(
                            PRIM_INT__g_s_d_s,
                            PRIM_INT__g_s_p_s,
                            PRIM_INT__g_s_s_s,
                            PRIM_INT__f_s_p_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2pq,
                            one_over_2q,
                            13);


                    ostei_general_vrr_K(5, 0, 3, 0, 12,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__h_s_d_s, PRIM_INT__h_s_p_s, NULL, PRIM_INT__g_s_d_s, NULL, PRIM_INT__h_s_f_s);


                    ostei_general_vrr_K(6, 0, 4, 0, 11,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__i_s_f_s, PRIM_INT__i_s_d_s, NULL, PRIM_INT__h_s_f_s, NULL, PRIM_INT__i_s_g_s);


                    ostei_general_vrr_K(7, 0, 5, 0, 10,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_g_s, PRIM_INT__k_s_f_s, NULL, PRIM_INT__i_s_g_s, NULL, PRIM_INT__k_s_h_s);



                    // Forming PRIM_INT__d_s_p_s[14 * 18];
                    for(n = 0; n < 14; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__d_s_p_s[n * 18 + 0] = SIMINT_MUL(Q_PA[0], PRIM_INT__d_s_s_s[n * 6 + 0]);
                        PRIM_INT__d_s_p_s[n * 18 + 0] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__d_s_p_s[n * 18 + 0]);
                        PRIM_INT__d_s_p_s[n * 18 + 0] = SIMINT_FMADD( vrr_const_2_over_2pq, PRIM_INT__p_s_s_s[(n+1) * 3 + 0], PRIM_INT__d_s_p_s[n * 18 + 0]);

                        PRIM_INT__d_s_p_s[n * 18 + 1] = SIMINT_MUL(Q_PA[1], PRIM_INT__d_s_s_s[n * 6 + 0]);
                        PRIM_INT__d_s_p_s[n * 18 + 1] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__d_s_p_s[n * 18 + 1]);

                        PRIM_INT__d_s_p_s[n * 18 + 2] = SIMINT_MUL(Q_PA[2], PRIM_INT__d_s_s_s[n * 6 + 0]);
                        PRIM_INT__d_s_p_s[n * 18 + 2] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__d_s_p_s[n * 18 + 2]);

                        PRIM_INT__d_s_p_s[n * 18 + 3] = SIMINT_MUL(Q_PA[0], PRIM_INT__d_s_s_s[n * 6 + 1]);
                        PRIM_INT__d_s_p_s[n * 18 + 3] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__d_s_s_s[(n+1) * 6 + 1], PRIM_INT__d_s_p_s[n * 18 + 3]);
                        PRIM_INT__d_s_p_s[n * 18 + 3] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__p_s_s_s[(n+1) * 3 + 1], PRIM_INT__d_s_p_s[n * 18 + 3]);

                        PRIM_INT__d_s_p_s[n * 18 + 4] = SIMINT_MUL(Q_PA[1], PRIM_INT__d_s_s_s[n * 6 + 1]);
                        PRIM_INT__d_s_p_s[n * 18 + 4] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__d_s_s_s[(n+1) * 6 + 1], PRIM_INT__d_s_p_s[n * 18 + 4]);
                        PRIM_INT__d_s_p_s[n * 18 + 4] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__p_s_s_s[(n+1) * 3 + 0], PRIM_INT__d_s_p_s[n * 18 + 4]);

                        PRIM_INT__d_s_p_s[n * 18 + 5] = SIMINT_MUL(Q_PA[2], PRIM_INT__d_s_s_s[n * 6 + 1]);
                        PRIM_INT__d_s_p_s[n * 18 + 5] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__d_s_s_s[(n+1) * 6 + 1], PRIM_INT__d_s_p_s[n * 18 + 5]);

                        PRIM_INT__d_s_p_s[n * 18 + 6] = SIMINT_MUL(Q_PA[0], PRIM_INT__d_s_s_s[n * 6 + 2]);
                        PRIM_INT__d_s_p_s[n * 18 + 6] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__d_s_s_s[(n+1) * 6 + 2], PRIM_INT__d_s_p_s[n * 18 + 6]);
                        PRIM_INT__d_s_p_s[n * 18 + 6] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__p_s_s_s[(n+1) * 3 + 2], PRIM_INT__d_s_p_s[n * 18 + 6]);

                        PRIM_INT__d_s_p_s[n * 18 + 7] = SIMINT_MUL(Q_PA[1], PRIM_INT__d_s_s_s[n * 6 + 2]);
                        PRIM_INT__d_s_p_s[n * 18 + 7] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__d_s_s_s[(n+1) * 6 + 2], PRIM_INT__d_s_p_s[n * 18 + 7]);

                        PRIM_INT__d_s_p_s[n * 18 + 8] = SIMINT_MUL(Q_PA[2], PRIM_INT__d_s_s_s[n * 6 + 2]);
                        PRIM_INT__d_s_p_s[n * 18 + 8] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__d_s_s_s[(n+1) * 6 + 2], PRIM_INT__d_s_p_s[n * 18 + 8]);
                        PRIM_INT__d_s_p_s[n * 18 + 8] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__p_s_s_s[(n+1) * 3 + 0], PRIM_INT__d_s_p_s[n * 18 + 8]);

                        PRIM_INT__d_s_p_s[n * 18 + 9] = SIMINT_MUL(Q_PA[0], PRIM_INT__d_s_s_s[n * 6 + 3]);
                        PRIM_INT__d_s_p_s[n * 18 + 9] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__d_s_p_s[n * 18 + 9]);

                        PRIM_INT__d_s_p_s[n * 18 + 10] = SIMINT_MUL(Q_PA[1], PRIM_INT__d_s_s_s[n * 6 + 3]);
                        PRIM_INT__d_s_p_s[n * 18 + 10] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__d_s_p_s[n * 18 + 10]);
                        PRIM_INT__d_s_p_s[n * 18 + 10] = SIMINT_FMADD( vrr_const_2_over_2pq, PRIM_INT__p_s_s_s[(n+1) * 3 + 1], PRIM_INT__d_s_p_s[n * 18 + 10]);

                        PRIM_INT__d_s_p_s[n * 18 + 11] = SIMINT_MUL(Q_PA[2], PRIM_INT__d_s_s_s[n * 6 + 3]);
                        PRIM_INT__d_s_p_s[n * 18 + 11] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__d_s_p_s[n * 18 + 11]);

                        PRIM_INT__d_s_p_s[n * 18 + 12] = SIMINT_MUL(Q_PA[0], PRIM_INT__d_s_s_s[n * 6 + 4]);
                        PRIM_INT__d_s_p_s[n * 18 + 12] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__d_s_s_s[(n+1) * 6 + 4], PRIM_INT__d_s_p_s[n * 18 + 12]);

                        PRIM_INT__d_s_p_s[n * 18 + 13] = SIMINT_MUL(Q_PA[1], PRIM_INT__d_s_s_s[n * 6 + 4]);
                        PRIM_INT__d_s_p_s[n * 18 + 13] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__d_s_s_s[(n+1) * 6 + 4], PRIM_INT__d_s_p_s[n * 18 + 13]);
                        PRIM_INT__d_s_p_s[n * 18 + 13] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__p_s_s_s[(n+1) * 3 + 2], PRIM_INT__d_s_p_s[n * 18 + 13]);

                        PRIM_INT__d_s_p_s[n * 18 + 14] = SIMINT_MUL(Q_PA[2], PRIM_INT__d_s_s_s[n * 6 + 4]);
                        PRIM_INT__d_s_p_s[n * 18 + 14] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__d_s_s_s[(n+1) * 6 + 4], PRIM_INT__d_s_p_s[n * 18 + 14]);
                        PRIM_INT__d_s_p_s[n * 18 + 14] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__p_s_s_s[(n+1) * 3 + 1], PRIM_INT__d_s_p_s[n * 18 + 14]);

                        PRIM_INT__d_s_p_s[n * 18 + 15] = SIMINT_MUL(Q_PA[0], PRIM_INT__d_s_s_s[n * 6 + 5]);
                        PRIM_INT__d_s_p_s[n * 18 + 15] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__d_s_s_s[(n+1) * 6 + 5], PRIM_INT__d_s_p_s[n * 18 + 15]);

                        PRIM_INT__d_s_p_s[n * 18 + 16] = SIMINT_MUL(Q_PA[1], PRIM_INT__d_s_s_s[n * 6 + 5]);
                        PRIM_INT__d_s_p_s[n * 18 + 16] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__d_s_s_s[(n+1) * 6 + 5], PRIM_INT__d_s_p_s[n * 18 + 16]);

                        PRIM_INT__d_s_p_s[n * 18 + 17] = SIMINT_MUL(Q_PA[2], PRIM_INT__d_s_s_s[n * 6 + 5]);
                        PRIM_INT__d_s_p_s[n * 18 + 17] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__d_s_s_s[(n+1) * 6 + 5], PRIM_INT__d_s_p_s[n * 18 + 17]);
                        PRIM_INT__d_s_p_s[n * 18 + 17] = SIMINT_FMADD( vrr_const_2_over_2pq, PRIM_INT__p_s_s_s[(n+1) * 3 + 2], PRIM_INT__d_s_p_s[n * 18 + 17]);

                    }


                    VRR_K_f_s_d_s(
                            PRIM_INT__f_s_d_s,
                            PRIM_INT__f_s_p_s,
                            PRIM_INT__f_s_s_s,
                            PRIM_INT__d_s_p_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2pq,
                            one_over_2q,
                            13);


                    VRR_K_g_s_f_s(
                            PRIM_INT__g_s_f_s,
                            PRIM_INT__g_s_d_s,
                            PRIM_INT__g_s_p_s,
                            PRIM_INT__f_s_d_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2pq,
                            one_over_2q,
                            12);


                    ostei_general_vrr_K(5, 0, 4, 0, 11,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__h_s_f_s, PRIM_INT__h_s_d_s, NULL, PRIM_INT__g_s_f_s, NULL, PRIM_INT__h_s_g_s);


                    ostei_general_vrr_K(6, 0, 5, 0, 10,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__i_s_g_s, PRIM_INT__i_s_f_s, NULL, PRIM_INT__h_s_g_s, NULL, PRIM_INT__i_s_h_s);


                    ostei_general_vrr_K(7, 0, 6, 0, 9,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_h_s, PRIM_INT__k_s_g_s, NULL, PRIM_INT__i_s_h_s, NULL, PRIM_INT__k_s_i_s);



                    // Forming PRIM_INT__p_s_p_s[14 * 9];
                    for(n = 0; n < 14; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__p_s_p_s[n * 9 + 0] = SIMINT_MUL(Q_PA[0], PRIM_INT__p_s_s_s[n * 3 + 0]);
                        PRIM_INT__p_s_p_s[n * 9 + 0] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__p_s_s_s[(n+1) * 3 + 0], PRIM_INT__p_s_p_s[n * 9 + 0]);
                        PRIM_INT__p_s_p_s[n * 9 + 0] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__p_s_p_s[n * 9 + 0]);

                        PRIM_INT__p_s_p_s[n * 9 + 1] = SIMINT_MUL(Q_PA[1], PRIM_INT__p_s_s_s[n * 3 + 0]);
                        PRIM_INT__p_s_p_s[n * 9 + 1] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__p_s_s_s[(n+1) * 3 + 0], PRIM_INT__p_s_p_s[n * 9 + 1]);

                        PRIM_INT__p_s_p_s[n * 9 + 2] = SIMINT_MUL(Q_PA[2], PRIM_INT__p_s_s_s[n * 3 + 0]);
                        PRIM_INT__p_s_p_s[n * 9 + 2] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__p_s_s_s[(n+1) * 3 + 0], PRIM_INT__p_s_p_s[n * 9 + 2]);

                        PRIM_INT__p_s_p_s[n * 9 + 3] = SIMINT_MUL(Q_PA[0], PRIM_INT__p_s_s_s[n * 3 + 1]);
                        PRIM_INT__p_s_p_s[n * 9 + 3] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__p_s_s_s[(n+1) * 3 + 1], PRIM_INT__p_s_p_s[n * 9 + 3]);

                        PRIM_INT__p_s_p_s[n * 9 + 4] = SIMINT_MUL(Q_PA[1], PRIM_INT__p_s_s_s[n * 3 + 1]);
                        PRIM_INT__p_s_p_s[n * 9 + 4] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__p_s_s_s[(n+1) * 3 + 1], PRIM_INT__p_s_p_s[n * 9 + 4]);
                        PRIM_INT__p_s_p_s[n * 9 + 4] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__p_s_p_s[n * 9 + 4]);

                        PRIM_INT__p_s_p_s[n * 9 + 5] = SIMINT_MUL(Q_PA[2], PRIM_INT__p_s_s_s[n * 3 + 1]);
                        PRIM_INT__p_s_p_s[n * 9 + 5] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__p_s_s_s[(n+1) * 3 + 1], PRIM_INT__p_s_p_s[n * 9 + 5]);

                        PRIM_INT__p_s_p_s[n * 9 + 6] = SIMINT_MUL(Q_PA[0], PRIM_INT__p_s_s_s[n * 3 + 2]);
                        PRIM_INT__p_s_p_s[n * 9 + 6] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__p_s_s_s[(n+1) * 3 + 2], PRIM_INT__p_s_p_s[n * 9 + 6]);

                        PRIM_INT__p_s_p_s[n * 9 + 7] = SIMINT_MUL(Q_PA[1], PRIM_INT__p_s_s_s[n * 3 + 2]);
                        PRIM_INT__p_s_p_s[n * 9 + 7] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__p_s_s_s[(n+1) * 3 + 2], PRIM_INT__p_s_p_s[n * 9 + 7]);

                        PRIM_INT__p_s_p_s[n * 9 + 8] = SIMINT_MUL(Q_PA[2], PRIM_INT__p_s_s_s[n * 3 + 2]);
                        PRIM_INT__p_s_p_s[n * 9 + 8] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__p_s_s_s[(n+1) * 3 + 2], PRIM_INT__p_s_p_s[n * 9 + 8]);
                        PRIM_INT__p_s_p_s[n * 9 + 8] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__p_s_p_s[n * 9 + 8]);

                    }



                    // Forming PRIM_INT__d_s_d_s[13 * 36];
                    for(n = 0; n < 13; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__d_s_d_s[n * 36 + 0] = SIMINT_MUL(Q_PA[0], PRIM_INT__d_s_p_s[n * 18 + 0]);
                        PRIM_INT__d_s_d_s[n * 36 + 0] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__d_s_p_s[(n+1) * 18 + 0], PRIM_INT__d_s_d_s[n * 36 + 0]);
                        PRIM_INT__d_s_d_s[n * 36 + 0] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__d_s_s_s[n * 6 + 0]), PRIM_INT__d_s_d_s[n * 36 + 0]);
                        PRIM_INT__d_s_d_s[n * 36 + 0] = SIMINT_FMADD( vrr_const_2_over_2pq, PRIM_INT__p_s_p_s[(n+1) * 9 + 0], PRIM_INT__d_s_d_s[n * 36 + 0]);

                        PRIM_INT__d_s_d_s[n * 36 + 3] = SIMINT_MUL(Q_PA[1], PRIM_INT__d_s_p_s[n * 18 + 1]);
                        PRIM_INT__d_s_d_s[n * 36 + 3] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__d_s_p_s[(n+1) * 18 + 1], PRIM_INT__d_s_d_s[n * 36 + 3]);
                        PRIM_INT__d_s_d_s[n * 36 + 3] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__d_s_s_s[n * 6 + 0]), PRIM_INT__d_s_d_s[n * 36 + 3]);

                        PRIM_INT__d_s_d_s[n * 36 + 5] = SIMINT_MUL(Q_PA[2], PRIM_INT__d_s_p_s[n * 18 + 2]);
                        PRIM_INT__d_s_d_s[n * 36 + 5] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__d_s_p_s[(n+1) * 18 + 2], PRIM_INT__d_s_d_s[n * 36 + 5]);
                        PRIM_INT__d_s_d_s[n * 36 + 5] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 0], PRIM_INT__d_s_s_s[n * 6 + 0]), PRIM_INT__d_s_d_s[n * 36 + 5]);

                        PRIM_INT__d_s_d_s[n * 36 + 6] = SIMINT_MUL(Q_PA[0], PRIM_INT__d_s_p_s[n * 18 + 3]);
                        PRIM_INT__d_s_d_s[n * 36 + 6] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__d_s_p_s[(n+1) * 18 + 3], PRIM_INT__d_s_d_s[n * 36 + 6]);
                        PRIM_INT__d_s_d_s[n * 36 + 6] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 1], PRIM_INT__d_s_s_s[n * 6 + 1]), PRIM_INT__d_s_d_s[n * 36 + 6]);
                        PRIM_INT__d_s_d_s[n * 36 + 6] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__p_s_p_s[(n+1) * 9 + 3], PRIM_INT__d_s_d_s[n * 36 + 6]);

                        PRIM_INT__d_s_d_s[n * 36 + 9] = SIMINT_MUL(Q_PA[1], PRIM_INT__d_s_p_s[n * 18 + 4]);
                        PRIM_INT__d_s_d_s[n * 36 + 9] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__d_s_p_s[(n+1) * 18 + 4], PRIM_INT__d_s_d_s[n * 36 + 9]);
                        PRIM_INT__d_s_d_s[n * 36 + 9] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 1], PRIM_INT__d_s_s_s[n * 6 + 1]), PRIM_INT__d_s_d_s[n * 36 + 9]);
                        PRIM_INT__d_s_d_s[n * 36 + 9] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__p_s_p_s[(n+1) * 9 + 1], PRIM_INT__d_s_d_s[n * 36 + 9]);

                        PRIM_INT__d_s_d_s[n * 36 + 11] = SIMINT_MUL(Q_PA[2], PRIM_INT__d_s_p_s[n * 18 + 5]);
                        PRIM_INT__d_s_d_s[n * 36 + 11] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__d_s_p_s[(n+1) * 18 + 5], PRIM_INT__d_s_d_s[n * 36 + 11]);
                        PRIM_INT__d_s_d_s[n * 36 + 11] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 1], PRIM_INT__d_s_s_s[n * 6 + 1]), PRIM_INT__d_s_d_s[n * 36 + 11]);

                        PRIM_INT__d_s_d_s[n * 36 + 12] = SIMINT_MUL(Q_PA[0], PRIM_INT__d_s_p_s[n * 18 + 6]);
                        PRIM_INT__d_s_d_s[n * 36 + 12] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__d_s_p_s[(n+1) * 18 + 6], PRIM_INT__d_s_d_s[n * 36 + 12]);
                        PRIM_INT__d_s_d_s[n * 36 + 12] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 2], PRIM_INT__d_s_s_s[n * 6 + 2]), PRIM_INT__d_s_d_s[n * 36 + 12]);
                        PRIM_INT__d_s_d_s[n * 36 + 12] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__p_s_p_s[(n+1) * 9 + 6], PRIM_INT__d_s_d_s[n * 36 + 12]);

                        PRIM_INT__d_s_d_s[n * 36 + 15] = SIMINT_MUL(Q_PA[1], PRIM_INT__d_s_p_s[n * 18 + 7]);
                        PRIM_INT__d_s_d_s[n * 36 + 15] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__d_s_p_s[(n+1) * 18 + 7], PRIM_INT__d_s_d_s[n * 36 + 15]);
                        PRIM_INT__d_s_d_s[n * 36 + 15] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 2], PRIM_INT__d_s_s_s[n * 6 + 2]), PRIM_INT__d_s_d_s[n * 36 + 15]);

                        PRIM_INT__d_s_d_s[n * 36 + 17] = SIMINT_MUL(Q_PA[2], PRIM_INT__d_s_p_s[n * 18 + 8]);
                        PRIM_INT__d_s_d_s[n * 36 + 17] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__d_s_p_s[(n+1) * 18 + 8], PRIM_INT__d_s_d_s[n * 36 + 17]);
                        PRIM_INT__d_s_d_s[n * 36 + 17] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 2], PRIM_INT__d_s_s_s[n * 6 + 2]), PRIM_INT__d_s_d_s[n * 36 + 17]);
                        PRIM_INT__d_s_d_s[n * 36 + 17] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__p_s_p_s[(n+1) * 9 + 2], PRIM_INT__d_s_d_s[n * 36 + 17]);

                        PRIM_INT__d_s_d_s[n * 36 + 18] = SIMINT_MUL(Q_PA[0], PRIM_INT__d_s_p_s[n * 18 + 9]);
                        PRIM_INT__d_s_d_s[n * 36 + 18] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__d_s_p_s[(n+1) * 18 + 9], PRIM_INT__d_s_d_s[n * 36 + 18]);
                        PRIM_INT__d_s_d_s[n * 36 + 18] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__d_s_s_s[n * 6 + 3]), PRIM_INT__d_s_d_s[n * 36 + 18]);

                        PRIM_INT__d_s_d_s[n * 36 + 21] = SIMINT_MUL(Q_PA[1], PRIM_INT__d_s_p_s[n * 18 + 10]);
                        PRIM_INT__d_s_d_s[n * 36 + 21] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__d_s_p_s[(n+1) * 18 + 10], PRIM_INT__d_s_d_s[n * 36 + 21]);
                        PRIM_INT__d_s_d_s[n * 36 + 21] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__d_s_s_s[n * 6 + 3]), PRIM_INT__d_s_d_s[n * 36 + 21]);
                        PRIM_INT__d_s_d_s[n * 36 + 21] = SIMINT_FMADD( vrr_const_2_over_2pq, PRIM_INT__p_s_p_s[(n+1) * 9 + 4], PRIM_INT__d_s_d_s[n * 36 + 21]);

                        PRIM_INT__d_s_d_s[n * 36 + 23] = SIMINT_MUL(Q_PA[2], PRIM_INT__d_s_p_s[n * 18 + 11]);
                        PRIM_INT__d_s_d_s[n * 36 + 23] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__d_s_p_s[(n+1) * 18 + 11], PRIM_INT__d_s_d_s[n * 36 + 23]);
                        PRIM_INT__d_s_d_s[n * 36 + 23] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 3], PRIM_INT__d_s_s_s[n * 6 + 3]), PRIM_INT__d_s_d_s[n * 36 + 23]);

                        PRIM_INT__d_s_d_s[n * 36 + 24] = SIMINT_MUL(Q_PA[0], PRIM_INT__d_s_p_s[n * 18 + 12]);
                        PRIM_INT__d_s_d_s[n * 36 + 24] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__d_s_p_s[(n+1) * 18 + 12], PRIM_INT__d_s_d_s[n * 36 + 24]);
                        PRIM_INT__d_s_d_s[n * 36 + 24] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 4], PRIM_INT__d_s_s_s[n * 6 + 4]), PRIM_INT__d_s_d_s[n * 36 + 24]);

                        PRIM_INT__d_s_d_s[n * 36 + 27] = SIMINT_MUL(Q_PA[1], PRIM_INT__d_s_p_s[n * 18 + 13]);
                        PRIM_INT__d_s_d_s[n * 36 + 27] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__d_s_p_s[(n+1) * 18 + 13], PRIM_INT__d_s_d_s[n * 36 + 27]);
                        PRIM_INT__d_s_d_s[n * 36 + 27] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 4], PRIM_INT__d_s_s_s[n * 6 + 4]), PRIM_INT__d_s_d_s[n * 36 + 27]);
                        PRIM_INT__d_s_d_s[n * 36 + 27] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__p_s_p_s[(n+1) * 9 + 7], PRIM_INT__d_s_d_s[n * 36 + 27]);

                        PRIM_INT__d_s_d_s[n * 36 + 29] = SIMINT_MUL(Q_PA[2], PRIM_INT__d_s_p_s[n * 18 + 14]);
                        PRIM_INT__d_s_d_s[n * 36 + 29] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__d_s_p_s[(n+1) * 18 + 14], PRIM_INT__d_s_d_s[n * 36 + 29]);
                        PRIM_INT__d_s_d_s[n * 36 + 29] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 4], PRIM_INT__d_s_s_s[n * 6 + 4]), PRIM_INT__d_s_d_s[n * 36 + 29]);
                        PRIM_INT__d_s_d_s[n * 36 + 29] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__p_s_p_s[(n+1) * 9 + 5], PRIM_INT__d_s_d_s[n * 36 + 29]);

                        PRIM_INT__d_s_d_s[n * 36 + 30] = SIMINT_MUL(Q_PA[0], PRIM_INT__d_s_p_s[n * 18 + 15]);
                        PRIM_INT__d_s_d_s[n * 36 + 30] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__d_s_p_s[(n+1) * 18 + 15], PRIM_INT__d_s_d_s[n * 36 + 30]);
                        PRIM_INT__d_s_d_s[n * 36 + 30] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 5], PRIM_INT__d_s_s_s[n * 6 + 5]), PRIM_INT__d_s_d_s[n * 36 + 30]);

                        PRIM_INT__d_s_d_s[n * 36 + 33] = SIMINT_MUL(Q_PA[1], PRIM_INT__d_s_p_s[n * 18 + 16]);
                        PRIM_INT__d_s_d_s[n * 36 + 33] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__d_s_p_s[(n+1) * 18 + 16], PRIM_INT__d_s_d_s[n * 36 + 33]);
                        PRIM_INT__d_s_d_s[n * 36 + 33] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 5], PRIM_INT__d_s_s_s[n * 6 + 5]), PRIM_INT__d_s_d_s[n * 36 + 33]);

                        PRIM_INT__d_s_d_s[n * 36 + 35] = SIMINT_MUL(Q_PA[2], PRIM_INT__d_s_p_s[n * 18 + 17]);
                        PRIM_INT__d_s_d_s[n * 36 + 35] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__d_s_p_s[(n+1) * 18 + 17], PRIM_INT__d_s_d_s[n * 36 + 35]);
                        PRIM_INT__d_s_d_s[n * 36 + 35] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__d_s_s_s[(n+1) * 6 + 5], PRIM_INT__d_s_s_s[n * 6 + 5]), PRIM_INT__d_s_d_s[n * 36 + 35]);
                        PRIM_INT__d_s_d_s[n * 36 + 35] = SIMINT_FMADD( vrr_const_2_over_2pq, PRIM_INT__p_s_p_s[(n+1) * 9 + 8], PRIM_INT__d_s_d_s[n * 36 + 35]);

                    }


                    VRR_K_f_s_f_s(
                            PRIM_INT__f_s_f_s,
                            PRIM_INT__f_s_d_s,
                            PRIM_INT__f_s_p_s,
                            PRIM_INT__d_s_d_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2pq,
                            one_over_2q,
                            12);


                    ostei_general_vrr_K(4, 0, 4, 0, 11,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__g_s_f_s, PRIM_INT__g_s_d_s, NULL, PRIM_INT__f_s_f_s, NULL, PRIM_INT__g_s_g_s);


                    ostei_general_vrr_K(5, 0, 5, 0, 10,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__h_s_g_s, PRIM_INT__h_s_f_s, NULL, PRIM_INT__g_s_g_s, NULL, PRIM_INT__h_s_h_s);


                    ostei_general_vrr_K(6, 0, 6, 0, 9,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__i_s_h_s, PRIM_INT__i_s_g_s, NULL, PRIM_INT__h_s_h_s, NULL, PRIM_INT__i_s_i_s);


                    ostei_general_vrr_K(7, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_i_s, PRIM_INT__k_s_h_s, NULL, PRIM_INT__i_s_i_s, NULL, PRIM_INT__k_s_k_s);



                    // Forming PRIM_INT__s_s_p_s[14 * 3];
                    for(n = 0; n < 14; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__s_s_p_s[n * 3 + 0] = SIMINT_MUL(Q_PA[0], PRIM_INT__s_s_s_s[n * 1 + 0]);
                        PRIM_INT__s_s_p_s[n * 3 + 0] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__s_s_p_s[n * 3 + 0]);

                        PRIM_INT__s_s_p_s[n * 3 + 1] = SIMINT_MUL(Q_PA[1], PRIM_INT__s_s_s_s[n * 1 + 0]);
                        PRIM_INT__s_s_p_s[n * 3 + 1] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__s_s_p_s[n * 3 + 1]);

                        PRIM_INT__s_s_p_s[n * 3 + 2] = SIMINT_MUL(Q_PA[2], PRIM_INT__s_s_s_s[n * 1 + 0]);
                        PRIM_INT__s_s_p_s[n * 3 + 2] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__s_s_p_s[n * 3 + 2]);

                    }



                    // Forming PRIM_INT__p_s_d_s[13 * 18];
                    for(n = 0; n < 13; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__p_s_d_s[n * 18 + 0] = SIMINT_MUL(Q_PA[0], PRIM_INT__p_s_p_s[n * 9 + 0]);
                        PRIM_INT__p_s_d_s[n * 18 + 0] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__p_s_p_s[(n+1) * 9 + 0], PRIM_INT__p_s_d_s[n * 18 + 0]);
                        PRIM_INT__p_s_d_s[n * 18 + 0] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_s_s[(n+1) * 3 + 0], PRIM_INT__p_s_s_s[n * 3 + 0]), PRIM_INT__p_s_d_s[n * 18 + 0]);
                        PRIM_INT__p_s_d_s[n * 18 + 0] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__s_s_p_s[(n+1) * 3 + 0], PRIM_INT__p_s_d_s[n * 18 + 0]);

                        PRIM_INT__p_s_d_s[n * 18 + 3] = SIMINT_MUL(Q_PA[1], PRIM_INT__p_s_p_s[n * 9 + 1]);
                        PRIM_INT__p_s_d_s[n * 18 + 3] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__p_s_p_s[(n+1) * 9 + 1], PRIM_INT__p_s_d_s[n * 18 + 3]);
                        PRIM_INT__p_s_d_s[n * 18 + 3] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_s_s[(n+1) * 3 + 0], PRIM_INT__p_s_s_s[n * 3 + 0]), PRIM_INT__p_s_d_s[n * 18 + 3]);

                        PRIM_INT__p_s_d_s[n * 18 + 5] = SIMINT_MUL(Q_PA[2], PRIM_INT__p_s_p_s[n * 9 + 2]);
                        PRIM_INT__p_s_d_s[n * 18 + 5] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__p_s_p_s[(n+1) * 9 + 2], PRIM_INT__p_s_d_s[n * 18 + 5]);
                        PRIM_INT__p_s_d_s[n * 18 + 5] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_s_s[(n+1) * 3 + 0], PRIM_INT__p_s_s_s[n * 3 + 0]), PRIM_INT__p_s_d_s[n * 18 + 5]);

                        PRIM_INT__p_s_d_s[n * 18 + 6] = SIMINT_MUL(Q_PA[0], PRIM_INT__p_s_p_s[n * 9 + 3]);
                        PRIM_INT__p_s_d_s[n * 18 + 6] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__p_s_p_s[(n+1) * 9 + 3], PRIM_INT__p_s_d_s[n * 18 + 6]);
                        PRIM_INT__p_s_d_s[n * 18 + 6] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_s_s[(n+1) * 3 + 1], PRIM_INT__p_s_s_s[n * 3 + 1]), PRIM_INT__p_s_d_s[n * 18 + 6]);

                        PRIM_INT__p_s_d_s[n * 18 + 9] = SIMINT_MUL(Q_PA[1], PRIM_INT__p_s_p_s[n * 9 + 4]);
                        PRIM_INT__p_s_d_s[n * 18 + 9] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__p_s_p_s[(n+1) * 9 + 4], PRIM_INT__p_s_d_s[n * 18 + 9]);
                        PRIM_INT__p_s_d_s[n * 18 + 9] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_s_s[(n+1) * 3 + 1], PRIM_INT__p_s_s_s[n * 3 + 1]), PRIM_INT__p_s_d_s[n * 18 + 9]);
                        PRIM_INT__p_s_d_s[n * 18 + 9] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__s_s_p_s[(n+1) * 3 + 1], PRIM_INT__p_s_d_s[n * 18 + 9]);

                        PRIM_INT__p_s_d_s[n * 18 + 11] = SIMINT_MUL(Q_PA[2], PRIM_INT__p_s_p_s[n * 9 + 5]);
                        PRIM_INT__p_s_d_s[n * 18 + 11] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__p_s_p_s[(n+1) * 9 + 5], PRIM_INT__p_s_d_s[n * 18 + 11]);
                        PRIM_INT__p_s_d_s[n * 18 + 11] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_s_s[(n+1) * 3 + 1], PRIM_INT__p_s_s_s[n * 3 + 1]), PRIM_INT__p_s_d_s[n * 18 + 11]);

                        PRIM_INT__p_s_d_s[n * 18 + 12] = SIMINT_MUL(Q_PA[0], PRIM_INT__p_s_p_s[n * 9 + 6]);
                        PRIM_INT__p_s_d_s[n * 18 + 12] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__p_s_p_s[(n+1) * 9 + 6], PRIM_INT__p_s_d_s[n * 18 + 12]);
                        PRIM_INT__p_s_d_s[n * 18 + 12] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_s_s[(n+1) * 3 + 2], PRIM_INT__p_s_s_s[n * 3 + 2]), PRIM_INT__p_s_d_s[n * 18 + 12]);

                        PRIM_INT__p_s_d_s[n * 18 + 15] = SIMINT_MUL(Q_PA[1], PRIM_INT__p_s_p_s[n * 9 + 7]);
                        PRIM_INT__p_s_d_s[n * 18 + 15] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__p_s_p_s[(n+1) * 9 + 7], PRIM_INT__p_s_d_s[n * 18 + 15]);
                        PRIM_INT__p_s_d_s[n * 18 + 15] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_s_s[(n+1) * 3 + 2], PRIM_INT__p_s_s_s[n * 3 + 2]), PRIM_INT__p_s_d_s[n * 18 + 15]);

                        PRIM_INT__p_s_d_s[n * 18 + 17] = SIMINT_MUL(Q_PA[2], PRIM_INT__p_s_p_s[n * 9 + 8]);
                        PRIM_INT__p_s_d_s[n * 18 + 17] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__p_s_p_s[(n+1) * 9 + 8], PRIM_INT__p_s_d_s[n * 18 + 17]);
                        PRIM_INT__p_s_d_s[n * 18 + 17] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_s_s[(n+1) * 3 + 2], PRIM_INT__p_s_s_s[n * 3 + 2]), PRIM_INT__p_s_d_s[n * 18 + 17]);
                        PRIM_INT__p_s_d_s[n * 18 + 17] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__s_s_p_s[(n+1) * 3 + 2], PRIM_INT__p_s_d_s[n * 18 + 17]);

                    }


                    VRR_K_d_s_f_s(
                            PRIM_INT__d_s_f_s,
                            PRIM_INT__d_s_d_s,
                            PRIM_INT__d_s_p_s,
                            PRIM_INT__p_s_d_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2pq,
                            one_over_2q,
                            12);


                    VRR_K_f_s_g_s(
                            PRIM_INT__f_s_g_s,
                            PRIM_INT__f_s_f_s,
                            PRIM_INT__f_s_d_s,
                            PRIM_INT__d_s_f_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2pq,
                            one_over_2q,
                            11);


                    ostei_general_vrr_K(4, 0, 5, 0, 10,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__g_s_g_s, PRIM_INT__g_s_f_s, NULL, PRIM_INT__f_s_g_s, NULL, PRIM_INT__g_s_h_s);


                    ostei_general_vrr_K(5, 0, 6, 0, 9,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__h_s_h_s, PRIM_INT__h_s_g_s, NULL, PRIM_INT__g_s_h_s, NULL, PRIM_INT__h_s_i_s);


                    ostei_general_vrr_K(6, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__i_s_i_s, PRIM_INT__i_s_h_s, NULL, PRIM_INT__h_s_i_s, NULL, PRIM_INT__i_s_k_s);


                    ostei_general_vrr_K(7, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_k_s, PRIM_INT__k_s_i_s, NULL, PRIM_INT__i_s_k_s, NULL, PRIM_INT__k_s_l_s);


                    ostei_general_vrr1_I(8, 21,
                            one_over_2p, a_over_p, aop_PQ, P_PA,
                            PRIM_INT__k_s_s_s, PRIM_INT__i_s_s_s, PRIM_INT__l_s_s_s);


                    ostei_general_vrr_K(8, 0, 1, 0, 14,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_s_s, NULL, NULL, PRIM_INT__k_s_s_s, NULL, PRIM_INT__l_s_p_s);


                    ostei_general_vrr_K(8, 0, 2, 0, 13,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_p_s, PRIM_INT__l_s_s_s, NULL, PRIM_INT__k_s_p_s, NULL, PRIM_INT__l_s_d_s);


                    ostei_general_vrr_K(8, 0, 3, 0, 12,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_d_s, PRIM_INT__l_s_p_s, NULL, PRIM_INT__k_s_d_s, NULL, PRIM_INT__l_s_f_s);


                    ostei_general_vrr_K(8, 0, 4, 0, 11,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_f_s, PRIM_INT__l_s_d_s, NULL, PRIM_INT__k_s_f_s, NULL, PRIM_INT__l_s_g_s);


                    ostei_general_vrr_K(8, 0, 5, 0, 10,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_g_s, PRIM_INT__l_s_f_s, NULL, PRIM_INT__k_s_g_s, NULL, PRIM_INT__l_s_h_s);


                    ostei_general_vrr_K(8, 0, 6, 0, 9,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_h_s, PRIM_INT__l_s_g_s, NULL, PRIM_INT__k_s_h_s, NULL, PRIM_INT__l_s_i_s);


                    ostei_general_vrr_K(8, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_i_s, PRIM_INT__l_s_h_s, NULL, PRIM_INT__k_s_i_s, NULL, PRIM_INT__l_s_k_s);



                    // Forming PRIM_INT__s_s_d_s[13 * 6];
                    for(n = 0; n < 13; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__s_s_d_s[n * 6 + 0] = SIMINT_MUL(Q_PA[0], PRIM_INT__s_s_p_s[n * 3 + 0]);
                        PRIM_INT__s_s_d_s[n * 6 + 0] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__s_s_p_s[(n+1) * 3 + 0], PRIM_INT__s_s_d_s[n * 6 + 0]);
                        PRIM_INT__s_s_d_s[n * 6 + 0] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__s_s_s_s[n * 1 + 0]), PRIM_INT__s_s_d_s[n * 6 + 0]);

                        PRIM_INT__s_s_d_s[n * 6 + 3] = SIMINT_MUL(Q_PA[1], PRIM_INT__s_s_p_s[n * 3 + 1]);
                        PRIM_INT__s_s_d_s[n * 6 + 3] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__s_s_p_s[(n+1) * 3 + 1], PRIM_INT__s_s_d_s[n * 6 + 3]);
                        PRIM_INT__s_s_d_s[n * 6 + 3] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__s_s_s_s[n * 1 + 0]), PRIM_INT__s_s_d_s[n * 6 + 3]);

                        PRIM_INT__s_s_d_s[n * 6 + 5] = SIMINT_MUL(Q_PA[2], PRIM_INT__s_s_p_s[n * 3 + 2]);
                        PRIM_INT__s_s_d_s[n * 6 + 5] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__s_s_p_s[(n+1) * 3 + 2], PRIM_INT__s_s_d_s[n * 6 + 5]);
                        PRIM_INT__s_s_d_s[n * 6 + 5] = SIMINT_FMADD( vrr_const_1_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__s_s_s_s[(n+1) * 1 + 0], PRIM_INT__s_s_s_s[n * 1 + 0]), PRIM_INT__s_s_d_s[n * 6 + 5]);

                    }



                    // Forming PRIM_INT__p_s_f_s[12 * 30];
                    for(n = 0; n < 12; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__p_s_f_s[n * 30 + 0] = SIMINT_MUL(Q_PA[0], PRIM_INT__p_s_d_s[n * 18 + 0]);
                        PRIM_INT__p_s_f_s[n * 30 + 0] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__p_s_d_s[(n+1) * 18 + 0], PRIM_INT__p_s_f_s[n * 30 + 0]);
                        PRIM_INT__p_s_f_s[n * 30 + 0] = SIMINT_FMADD( vrr_const_2_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_p_s[(n+1) * 9 + 0], PRIM_INT__p_s_p_s[n * 9 + 0]), PRIM_INT__p_s_f_s[n * 30 + 0]);
                        PRIM_INT__p_s_f_s[n * 30 + 0] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__s_s_d_s[(n+1) * 6 + 0], PRIM_INT__p_s_f_s[n * 30 + 0]);

                        PRIM_INT__p_s_f_s[n * 30 + 6] = SIMINT_MUL(Q_PA[1], PRIM_INT__p_s_d_s[n * 18 + 3]);
                        PRIM_INT__p_s_f_s[n * 30 + 6] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__p_s_d_s[(n+1) * 18 + 3], PRIM_INT__p_s_f_s[n * 30 + 6]);
                        PRIM_INT__p_s_f_s[n * 30 + 6] = SIMINT_FMADD( vrr_const_2_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_p_s[(n+1) * 9 + 1], PRIM_INT__p_s_p_s[n * 9 + 1]), PRIM_INT__p_s_f_s[n * 30 + 6]);

                        PRIM_INT__p_s_f_s[n * 30 + 9] = SIMINT_MUL(Q_PA[2], PRIM_INT__p_s_d_s[n * 18 + 5]);
                        PRIM_INT__p_s_f_s[n * 30 + 9] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__p_s_d_s[(n+1) * 18 + 5], PRIM_INT__p_s_f_s[n * 30 + 9]);
                        PRIM_INT__p_s_f_s[n * 30 + 9] = SIMINT_FMADD( vrr_const_2_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_p_s[(n+1) * 9 + 2], PRIM_INT__p_s_p_s[n * 9 + 2]), PRIM_INT__p_s_f_s[n * 30 + 9]);

                        PRIM_INT__p_s_f_s[n * 30 + 10] = SIMINT_MUL(Q_PA[0], PRIM_INT__p_s_d_s[n * 18 + 6]);
                        PRIM_INT__p_s_f_s[n * 30 + 10] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__p_s_d_s[(n+1) * 18 + 6], PRIM_INT__p_s_f_s[n * 30 + 10]);
                        PRIM_INT__p_s_f_s[n * 30 + 10] = SIMINT_FMADD( vrr_const_2_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_p_s[(n+1) * 9 + 3], PRIM_INT__p_s_p_s[n * 9 + 3]), PRIM_INT__p_s_f_s[n * 30 + 10]);

                        PRIM_INT__p_s_f_s[n * 30 + 16] = SIMINT_MUL(Q_PA[1], PRIM_INT__p_s_d_s[n * 18 + 9]);
                        PRIM_INT__p_s_f_s[n * 30 + 16] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__p_s_d_s[(n+1) * 18 + 9], PRIM_INT__p_s_f_s[n * 30 + 16]);
                        PRIM_INT__p_s_f_s[n * 30 + 16] = SIMINT_FMADD( vrr_const_2_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_p_s[(n+1) * 9 + 4], PRIM_INT__p_s_p_s[n * 9 + 4]), PRIM_INT__p_s_f_s[n * 30 + 16]);
                        PRIM_INT__p_s_f_s[n * 30 + 16] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__s_s_d_s[(n+1) * 6 + 3], PRIM_INT__p_s_f_s[n * 30 + 16]);

                        PRIM_INT__p_s_f_s[n * 30 + 19] = SIMINT_MUL(Q_PA[2], PRIM_INT__p_s_d_s[n * 18 + 11]);
                        PRIM_INT__p_s_f_s[n * 30 + 19] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__p_s_d_s[(n+1) * 18 + 11], PRIM_INT__p_s_f_s[n * 30 + 19]);
                        PRIM_INT__p_s_f_s[n * 30 + 19] = SIMINT_FMADD( vrr_const_2_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_p_s[(n+1) * 9 + 5], PRIM_INT__p_s_p_s[n * 9 + 5]), PRIM_INT__p_s_f_s[n * 30 + 19]);

                        PRIM_INT__p_s_f_s[n * 30 + 20] = SIMINT_MUL(Q_PA[0], PRIM_INT__p_s_d_s[n * 18 + 12]);
                        PRIM_INT__p_s_f_s[n * 30 + 20] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__p_s_d_s[(n+1) * 18 + 12], PRIM_INT__p_s_f_s[n * 30 + 20]);
                        PRIM_INT__p_s_f_s[n * 30 + 20] = SIMINT_FMADD( vrr_const_2_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_p_s[(n+1) * 9 + 6], PRIM_INT__p_s_p_s[n * 9 + 6]), PRIM_INT__p_s_f_s[n * 30 + 20]);

                        PRIM_INT__p_s_f_s[n * 30 + 26] = SIMINT_MUL(Q_PA[1], PRIM_INT__p_s_d_s[n * 18 + 15]);
                        PRIM_INT__p_s_f_s[n * 30 + 26] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__p_s_d_s[(n+1) * 18 + 15], PRIM_INT__p_s_f_s[n * 30 + 26]);
                        PRIM_INT__p_s_f_s[n * 30 + 26] = SIMINT_FMADD( vrr_const_2_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_p_s[(n+1) * 9 + 7], PRIM_INT__p_s_p_s[n * 9 + 7]), PRIM_INT__p_s_f_s[n * 30 + 26]);

                        PRIM_INT__p_s_f_s[n * 30 + 29] = SIMINT_MUL(Q_PA[2], PRIM_INT__p_s_d_s[n * 18 + 17]);
                        PRIM_INT__p_s_f_s[n * 30 + 29] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__p_s_d_s[(n+1) * 18 + 17], PRIM_INT__p_s_f_s[n * 30 + 29]);
                        PRIM_INT__p_s_f_s[n * 30 + 29] = SIMINT_FMADD( vrr_const_2_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__p_s_p_s[(n+1) * 9 + 8], PRIM_INT__p_s_p_s[n * 9 + 8]), PRIM_INT__p_s_f_s[n * 30 + 29]);
                        PRIM_INT__p_s_f_s[n * 30 + 29] = SIMINT_FMADD( vrr_const_1_over_2pq, PRIM_INT__s_s_d_s[(n+1) * 6 + 5], PRIM_INT__p_s_f_s[n * 30 + 29]);

                    }


                    VRR_K_d_s_g_s(
                            PRIM_INT__d_s_g_s,
                            PRIM_INT__d_s_f_s,
                            PRIM_INT__d_s_d_s,
                            PRIM_INT__p_s_f_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2pq,
                            one_over_2q,
                            11);


                    ostei_general_vrr_K(3, 0, 5, 0, 10,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__f_s_g_s, PRIM_INT__f_s_f_s, NULL, PRIM_INT__d_s_g_s, NULL, PRIM_INT__f_s_h_s);


                    ostei_general_vrr_K(4, 0, 6, 0, 9,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__g_s_h_s, PRIM_INT__g_s_g_s, NULL, PRIM_INT__f_s_h_s, NULL, PRIM_INT__g_s_i_s);


                    ostei_general_vrr_K(5, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__h_s_i_s, PRIM_INT__h_s_h_s, NULL, PRIM_INT__g_s_i_s, NULL, PRIM_INT__h_s_k_s);


                    ostei_general_vrr_K(6, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__i_s_k_s, PRIM_INT__i_s_i_s, NULL, PRIM_INT__h_s_k_s, NULL, PRIM_INT__i_s_l_s);


                    ostei_general_vrr_K(7, 0, 9, 0, 6,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_l_s, PRIM_INT__k_s_k_s, NULL, PRIM_INT__i_s_l_s, NULL, PRIM_INT__k_s_m_s);


                    ostei_general_vrr_K(8, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_k_s, PRIM_INT__l_s_i_s, NULL, PRIM_INT__k_s_k_s, NULL, PRIM_INT__l_s_l_s);


                    ostei_general_vrr1_I(9, 20,
                            one_over_2p, a_over_p, aop_PQ, P_PA,
                            PRIM_INT__l_s_s_s, PRIM_INT__k_s_s_s, PRIM_INT__m_s_s_s);


                    ostei_general_vrr_K(9, 0, 1, 0, 14,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_s_s, NULL, NULL, PRIM_INT__l_s_s_s, NULL, PRIM_INT__m_s_p_s);


                    ostei_general_vrr_K(9, 0, 2, 0, 13,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_p_s, PRIM_INT__m_s_s_s, NULL, PRIM_INT__l_s_p_s, NULL, PRIM_INT__m_s_d_s);


                    ostei_general_vrr_K(9, 0, 3, 0, 12,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_d_s, PRIM_INT__m_s_p_s, NULL, PRIM_INT__l_s_d_s, NULL, PRIM_INT__m_s_f_s);


                    ostei_general_vrr_K(9, 0, 4, 0, 11,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_f_s, PRIM_INT__m_s_d_s, NULL, PRIM_INT__l_s_f_s, NULL, PRIM_INT__m_s_g_s);


                    ostei_general_vrr_K(9, 0, 5, 0, 10,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_g_s, PRIM_INT__m_s_f_s, NULL, PRIM_INT__l_s_g_s, NULL, PRIM_INT__m_s_h_s);


                    ostei_general_vrr_K(9, 0, 6, 0, 9,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_h_s, PRIM_INT__m_s_g_s, NULL, PRIM_INT__l_s_h_s, NULL, PRIM_INT__m_s_i_s);


                    ostei_general_vrr_K(9, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_i_s, PRIM_INT__m_s_h_s, NULL, PRIM_INT__l_s_i_s, NULL, PRIM_INT__m_s_k_s);



                    // Forming PRIM_INT__s_s_f_s[12 * 10];
                    for(n = 0; n < 12; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__s_s_f_s[n * 10 + 0] = SIMINT_MUL(Q_PA[0], PRIM_INT__s_s_d_s[n * 6 + 0]);
                        PRIM_INT__s_s_f_s[n * 10 + 0] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__s_s_d_s[(n+1) * 6 + 0], PRIM_INT__s_s_f_s[n * 10 + 0]);
                        PRIM_INT__s_s_f_s[n * 10 + 0] = SIMINT_FMADD( vrr_const_2_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__s_s_p_s[(n+1) * 3 + 0], PRIM_INT__s_s_p_s[n * 3 + 0]), PRIM_INT__s_s_f_s[n * 10 + 0]);

                        PRIM_INT__s_s_f_s[n * 10 + 6] = SIMINT_MUL(Q_PA[1], PRIM_INT__s_s_d_s[n * 6 + 3]);
                        PRIM_INT__s_s_f_s[n * 10 + 6] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__s_s_d_s[(n+1) * 6 + 3], PRIM_INT__s_s_f_s[n * 10 + 6]);
                        PRIM_INT__s_s_f_s[n * 10 + 6] = SIMINT_FMADD( vrr_const_2_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__s_s_p_s[(n+1) * 3 + 1], PRIM_INT__s_s_p_s[n * 3 + 1]), PRIM_INT__s_s_f_s[n * 10 + 6]);

                        PRIM_INT__s_s_f_s[n * 10 + 9] = SIMINT_MUL(Q_PA[2], PRIM_INT__s_s_d_s[n * 6 + 5]);
                        PRIM_INT__s_s_f_s[n * 10 + 9] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__s_s_d_s[(n+1) * 6 + 5], PRIM_INT__s_s_f_s[n * 10 + 9]);
                        PRIM_INT__s_s_f_s[n * 10 + 9] = SIMINT_FMADD( vrr_const_2_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__s_s_p_s[(n+1) * 3 + 2], PRIM_INT__s_s_p_s[n * 3 + 2]), PRIM_INT__s_s_f_s[n * 10 + 9]);

                    }


                    VRR_K_p_s_g_s(
                            PRIM_INT__p_s_g_s,
                            PRIM_INT__p_s_f_s,
                            PRIM_INT__p_s_d_s,
                            PRIM_INT__s_s_f_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2pq,
                            one_over_2q,
                            11);


                    VRR_K_d_s_h_s(
                            PRIM_INT__d_s_h_s,
                            PRIM_INT__d_s_g_s,
                            PRIM_INT__d_s_f_s,
                            PRIM_INT__p_s_g_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2pq,
                            one_over_2q,
                            10);


                    ostei_general_vrr_K(3, 0, 6, 0, 9,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__f_s_h_s, PRIM_INT__f_s_g_s, NULL, PRIM_INT__d_s_h_s, NULL, PRIM_INT__f_s_i_s);


                    ostei_general_vrr_K(4, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__g_s_i_s, PRIM_INT__g_s_h_s, NULL, PRIM_INT__f_s_i_s, NULL, PRIM_INT__g_s_k_s);


                    ostei_general_vrr_K(5, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__h_s_k_s, PRIM_INT__h_s_i_s, NULL, PRIM_INT__g_s_k_s, NULL, PRIM_INT__h_s_l_s);


                    ostei_general_vrr_K(6, 0, 9, 0, 6,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__i_s_l_s, PRIM_INT__i_s_k_s, NULL, PRIM_INT__h_s_l_s, NULL, PRIM_INT__i_s_m_s);


                    ostei_general_vrr_K(7, 0, 10, 0, 5,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_m_s, PRIM_INT__k_s_l_s, NULL, PRIM_INT__i_s_m_s, NULL, PRIM_INT__k_s_n_s);


                    ostei_general_vrr_K(8, 0, 9, 0, 6,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_l_s, PRIM_INT__l_s_k_s, NULL, PRIM_INT__k_s_l_s, NULL, PRIM_INT__l_s_m_s);


                    ostei_general_vrr_K(9, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_k_s, PRIM_INT__m_s_i_s, NULL, PRIM_INT__l_s_k_s, NULL, PRIM_INT__m_s_l_s);


                    ostei_general_vrr1_I(10, 19,
                            one_over_2p, a_over_p, aop_PQ, P_PA,
                            PRIM_INT__m_s_s_s, PRIM_INT__l_s_s_s, PRIM_INT__n_s_s_s);


                    ostei_general_vrr_K(10, 0, 1, 0, 14,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_s_s, NULL, NULL, PRIM_INT__m_s_s_s, NULL, PRIM_INT__n_s_p_s);


                    ostei_general_vrr_K(10, 0, 2, 0, 13,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_p_s, PRIM_INT__n_s_s_s, NULL, PRIM_INT__m_s_p_s, NULL, PRIM_INT__n_s_d_s);


                    ostei_general_vrr_K(10, 0, 3, 0, 12,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_d_s, PRIM_INT__n_s_p_s, NULL, PRIM_INT__m_s_d_s, NULL, PRIM_INT__n_s_f_s);


                    ostei_general_vrr_K(10, 0, 4, 0, 11,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_f_s, PRIM_INT__n_s_d_s, NULL, PRIM_INT__m_s_f_s, NULL, PRIM_INT__n_s_g_s);


                    ostei_general_vrr_K(10, 0, 5, 0, 10,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_g_s, PRIM_INT__n_s_f_s, NULL, PRIM_INT__m_s_g_s, NULL, PRIM_INT__n_s_h_s);


                    ostei_general_vrr_K(10, 0, 6, 0, 9,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_h_s, PRIM_INT__n_s_g_s, NULL, PRIM_INT__m_s_h_s, NULL, PRIM_INT__n_s_i_s);


                    ostei_general_vrr_K(10, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_i_s, PRIM_INT__n_s_h_s, NULL, PRIM_INT__m_s_i_s, NULL, PRIM_INT__n_s_k_s);



                    // Forming PRIM_INT__s_s_g_s[11 * 15];
                    for(n = 0; n < 11; ++n)  // loop over orders of auxiliary function
                    {

                        PRIM_INT__s_s_g_s[n * 15 + 0] = SIMINT_MUL(Q_PA[0], PRIM_INT__s_s_f_s[n * 10 + 0]);
                        PRIM_INT__s_s_g_s[n * 15 + 0] = SIMINT_FMADD( aoq_PQ[0], PRIM_INT__s_s_f_s[(n+1) * 10 + 0], PRIM_INT__s_s_g_s[n * 15 + 0]);
                        PRIM_INT__s_s_g_s[n * 15 + 0] = SIMINT_FMADD( vrr_const_3_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__s_s_d_s[(n+1) * 6 + 0], PRIM_INT__s_s_d_s[n * 6 + 0]), PRIM_INT__s_s_g_s[n * 15 + 0]);

                        PRIM_INT__s_s_g_s[n * 15 + 10] = SIMINT_MUL(Q_PA[1], PRIM_INT__s_s_f_s[n * 10 + 6]);
                        PRIM_INT__s_s_g_s[n * 15 + 10] = SIMINT_FMADD( aoq_PQ[1], PRIM_INT__s_s_f_s[(n+1) * 10 + 6], PRIM_INT__s_s_g_s[n * 15 + 10]);
                        PRIM_INT__s_s_g_s[n * 15 + 10] = SIMINT_FMADD( vrr_const_3_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__s_s_d_s[(n+1) * 6 + 3], PRIM_INT__s_s_d_s[n * 6 + 3]), PRIM_INT__s_s_g_s[n * 15 + 10]);

                        PRIM_INT__s_s_g_s[n * 15 + 14] = SIMINT_MUL(Q_PA[2], PRIM_INT__s_s_f_s[n * 10 + 9]);
                        PRIM_INT__s_s_g_s[n * 15 + 14] = SIMINT_FMADD( aoq_PQ[2], PRIM_INT__s_s_f_s[(n+1) * 10 + 9], PRIM_INT__s_s_g_s[n * 15 + 14]);
                        PRIM_INT__s_s_g_s[n * 15 + 14] = SIMINT_FMADD( vrr_const_3_over_2q, SIMINT_FMADD(a_over_q, PRIM_INT__s_s_d_s[(n+1) * 6 + 5], PRIM_INT__s_s_d_s[n * 6 + 5]), PRIM_INT__s_s_g_s[n * 15 + 14]);

                    }


                    VRR_K_p_s_h_s(
                            PRIM_INT__p_s_h_s,
                            PRIM_INT__p_s_g_s,
                            PRIM_INT__p_s_f_s,
                            PRIM_INT__s_s_g_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2pq,
                            one_over_2q,
                            10);


                    ostei_general_vrr_K(2, 0, 6, 0, 9,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__d_s_h_s, PRIM_INT__d_s_g_s, NULL, PRIM_INT__p_s_h_s, NULL, PRIM_INT__d_s_i_s);


                    ostei_general_vrr_K(3, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__f_s_i_s, PRIM_INT__f_s_h_s, NULL, PRIM_INT__d_s_i_s, NULL, PRIM_INT__f_s_k_s);


                    ostei_general_vrr_K(4, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__g_s_k_s, PRIM_INT__g_s_i_s, NULL, PRIM_INT__f_s_k_s, NULL, PRIM_INT__g_s_l_s);


                    ostei_general_vrr_K(5, 0, 9, 0, 6,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__h_s_l_s, PRIM_INT__h_s_k_s, NULL, PRIM_INT__g_s_l_s, NULL, PRIM_INT__h_s_m_s);


                    ostei_general_vrr_K(6, 0, 10, 0, 5,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__i_s_m_s, PRIM_INT__i_s_l_s, NULL, PRIM_INT__h_s_m_s, NULL, PRIM_INT__i_s_n_s);


                    ostei_general_vrr_K(7, 0, 11, 0, 4,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_n_s, PRIM_INT__k_s_m_s, NULL, PRIM_INT__i_s_n_s, NULL, PRIM_INT__k_s_o_s);


                    ostei_general_vrr_K(8, 0, 10, 0, 5,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_m_s, PRIM_INT__l_s_l_s, NULL, PRIM_INT__k_s_m_s, NULL, PRIM_INT__l_s_n_s);


                    ostei_general_vrr_K(9, 0, 9, 0, 6,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_l_s, PRIM_INT__m_s_k_s, NULL, PRIM_INT__l_s_l_s, NULL, PRIM_INT__m_s_m_s);


                    ostei_general_vrr_K(10, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_k_s, PRIM_INT__n_s_i_s, NULL, PRIM_INT__m_s_k_s, NULL, PRIM_INT__n_s_l_s);


                    ostei_general_vrr1_I(11, 18,
                            one_over_2p, a_over_p, aop_PQ, P_PA,
                            PRIM_INT__n_s_s_s, PRIM_INT__m_s_s_s, PRIM_INT__o_s_s_s);


                    ostei_general_vrr_K(11, 0, 1, 0, 14,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_s_s, NULL, NULL, PRIM_INT__n_s_s_s, NULL, PRIM_INT__o_s_p_s);


                    ostei_general_vrr_K(11, 0, 2, 0, 13,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_p_s, PRIM_INT__o_s_s_s, NULL, PRIM_INT__n_s_p_s, NULL, PRIM_INT__o_s_d_s);


                    ostei_general_vrr_K(11, 0, 3, 0, 12,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_d_s, PRIM_INT__o_s_p_s, NULL, PRIM_INT__n_s_d_s, NULL, PRIM_INT__o_s_f_s);


                    ostei_general_vrr_K(11, 0, 4, 0, 11,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_f_s, PRIM_INT__o_s_d_s, NULL, PRIM_INT__n_s_f_s, NULL, PRIM_INT__o_s_g_s);


                    ostei_general_vrr_K(11, 0, 5, 0, 10,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_g_s, PRIM_INT__o_s_f_s, NULL, PRIM_INT__n_s_g_s, NULL, PRIM_INT__o_s_h_s);


                    ostei_general_vrr_K(11, 0, 6, 0, 9,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_h_s, PRIM_INT__o_s_g_s, NULL, PRIM_INT__n_s_h_s, NULL, PRIM_INT__o_s_i_s);


                    ostei_general_vrr_K(11, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_i_s, PRIM_INT__o_s_h_s, NULL, PRIM_INT__n_s_i_s, NULL, PRIM_INT__o_s_k_s);


                    VRR_K_s_s_h_s(
                            PRIM_INT__s_s_h_s,
                            PRIM_INT__s_s_g_s,
                            PRIM_INT__s_s_f_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2q,
                            10);


                    VRR_K_p_s_i_s(
                            PRIM_INT__p_s_i_s,
                            PRIM_INT__p_s_h_s,
                            PRIM_INT__p_s_g_s,
                            PRIM_INT__s_s_h_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2pq,
                            one_over_2q,
                            9);


                    ostei_general_vrr_K(2, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__d_s_i_s, PRIM_INT__d_s_h_s, NULL, PRIM_INT__p_s_i_s, NULL, PRIM_INT__d_s_k_s);


                    ostei_general_vrr_K(3, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__f_s_k_s, PRIM_INT__f_s_i_s, NULL, PRIM_INT__d_s_k_s, NULL, PRIM_INT__f_s_l_s);


                    ostei_general_vrr_K(4, 0, 9, 0, 6,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__g_s_l_s, PRIM_INT__g_s_k_s, NULL, PRIM_INT__f_s_l_s, NULL, PRIM_INT__g_s_m_s);


                    ostei_general_vrr_K(5, 0, 10, 0, 5,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__h_s_m_s, PRIM_INT__h_s_l_s, NULL, PRIM_INT__g_s_m_s, NULL, PRIM_INT__h_s_n_s);


                    ostei_general_vrr_K(6, 0, 11, 0, 4,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__i_s_n_s, PRIM_INT__i_s_m_s, NULL, PRIM_INT__h_s_n_s, NULL, PRIM_INT__i_s_o_s);


                    ostei_general_vrr_K(7, 0, 12, 0, 3,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_o_s, PRIM_INT__k_s_n_s, NULL, PRIM_INT__i_s_o_s, NULL, PRIM_INT__k_s_q_s);


                    ostei_general_vrr_K(8, 0, 11, 0, 4,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_n_s, PRIM_INT__l_s_m_s, NULL, PRIM_INT__k_s_n_s, NULL, PRIM_INT__l_s_o_s);


                    ostei_general_vrr_K(9, 0, 10, 0, 5,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_m_s, PRIM_INT__m_s_l_s, NULL, PRIM_INT__l_s_m_s, NULL, PRIM_INT__m_s_n_s);


                    ostei_general_vrr_K(10, 0, 9, 0, 6,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_l_s, PRIM_INT__n_s_k_s, NULL, PRIM_INT__m_s_l_s, NULL, PRIM_INT__n_s_m_s);


                    ostei_general_vrr_K(11, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_k_s, PRIM_INT__o_s_i_s, NULL, PRIM_INT__n_s_k_s, NULL, PRIM_INT__o_s_l_s);


                    ostei_general_vrr1_I(12, 17,
                            one_over_2p, a_over_p, aop_PQ, P_PA,
                            PRIM_INT__o_s_s_s, PRIM_INT__n_s_s_s, PRIM_INT__q_s_s_s);


                    ostei_general_vrr_K(12, 0, 1, 0, 14,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_s_s, NULL, NULL, PRIM_INT__o_s_s_s, NULL, PRIM_INT__q_s_p_s);


                    ostei_general_vrr_K(12, 0, 2, 0, 13,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_p_s, PRIM_INT__q_s_s_s, NULL, PRIM_INT__o_s_p_s, NULL, PRIM_INT__q_s_d_s);


                    ostei_general_vrr_K(12, 0, 3, 0, 12,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_d_s, PRIM_INT__q_s_p_s, NULL, PRIM_INT__o_s_d_s, NULL, PRIM_INT__q_s_f_s);


                    ostei_general_vrr_K(12, 0, 4, 0, 11,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_f_s, PRIM_INT__q_s_d_s, NULL, PRIM_INT__o_s_f_s, NULL, PRIM_INT__q_s_g_s);


                    ostei_general_vrr_K(12, 0, 5, 0, 10,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_g_s, PRIM_INT__q_s_f_s, NULL, PRIM_INT__o_s_g_s, NULL, PRIM_INT__q_s_h_s);


                    ostei_general_vrr_K(12, 0, 6, 0, 9,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_h_s, PRIM_INT__q_s_g_s, NULL, PRIM_INT__o_s_h_s, NULL, PRIM_INT__q_s_i_s);


                    ostei_general_vrr_K(12, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_i_s, PRIM_INT__q_s_h_s, NULL, PRIM_INT__o_s_i_s, NULL, PRIM_INT__q_s_k_s);


                    VRR_K_s_s_i_s(
                            PRIM_INT__s_s_i_s,
                            PRIM_INT__s_s_h_s,
                            PRIM_INT__s_s_g_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2q,
                            9);


                    ostei_general_vrr_K(1, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__p_s_i_s, PRIM_INT__p_s_h_s, NULL, PRIM_INT__s_s_i_s, NULL, PRIM_INT__p_s_k_s);


                    ostei_general_vrr_K(2, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__d_s_k_s, PRIM_INT__d_s_i_s, NULL, PRIM_INT__p_s_k_s, NULL, PRIM_INT__d_s_l_s);


                    ostei_general_vrr_K(3, 0, 9, 0, 6,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__f_s_l_s, PRIM_INT__f_s_k_s, NULL, PRIM_INT__d_s_l_s, NULL, PRIM_INT__f_s_m_s);


                    ostei_general_vrr_K(4, 0, 10, 0, 5,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__g_s_m_s, PRIM_INT__g_s_l_s, NULL, PRIM_INT__f_s_m_s, NULL, PRIM_INT__g_s_n_s);


                    ostei_general_vrr_K(5, 0, 11, 0, 4,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__h_s_n_s, PRIM_INT__h_s_m_s, NULL, PRIM_INT__g_s_n_s, NULL, PRIM_INT__h_s_o_s);


                    ostei_general_vrr_K(6, 0, 12, 0, 3,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__i_s_o_s, PRIM_INT__i_s_n_s, NULL, PRIM_INT__h_s_o_s, NULL, PRIM_INT__i_s_q_s);


                    ostei_general_vrr_K(7, 0, 13, 0, 2,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_q_s, PRIM_INT__k_s_o_s, NULL, PRIM_INT__i_s_q_s, NULL, PRIM_INT__k_s_r_s);


                    ostei_general_vrr_K(8, 0, 12, 0, 3,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_o_s, PRIM_INT__l_s_n_s, NULL, PRIM_INT__k_s_o_s, NULL, PRIM_INT__l_s_q_s);


                    ostei_general_vrr_K(9, 0, 11, 0, 4,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_n_s, PRIM_INT__m_s_m_s, NULL, PRIM_INT__l_s_n_s, NULL, PRIM_INT__m_s_o_s);


                    ostei_general_vrr_K(10, 0, 10, 0, 5,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_m_s, PRIM_INT__n_s_l_s, NULL, PRIM_INT__m_s_m_s, NULL, PRIM_INT__n_s_n_s);


                    ostei_general_vrr_K(11, 0, 9, 0, 6,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_l_s, PRIM_INT__o_s_k_s, NULL, PRIM_INT__n_s_l_s, NULL, PRIM_INT__o_s_m_s);


                    ostei_general_vrr_K(12, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_k_s, PRIM_INT__q_s_i_s, NULL, PRIM_INT__o_s_k_s, NULL, PRIM_INT__q_s_l_s);


                    ostei_general_vrr1_I(13, 16,
                            one_over_2p, a_over_p, aop_PQ, P_PA,
                            PRIM_INT__q_s_s_s, PRIM_INT__o_s_s_s, PRIM_INT__r_s_s_s);


                    ostei_general_vrr_K(13, 0, 1, 0, 14,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_s_s, NULL, NULL, PRIM_INT__q_s_s_s, NULL, PRIM_INT__r_s_p_s);


                    ostei_general_vrr_K(13, 0, 2, 0, 13,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_p_s, PRIM_INT__r_s_s_s, NULL, PRIM_INT__q_s_p_s, NULL, PRIM_INT__r_s_d_s);


                    ostei_general_vrr_K(13, 0, 3, 0, 12,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_d_s, PRIM_INT__r_s_p_s, NULL, PRIM_INT__q_s_d_s, NULL, PRIM_INT__r_s_f_s);


                    ostei_general_vrr_K(13, 0, 4, 0, 11,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_f_s, PRIM_INT__r_s_d_s, NULL, PRIM_INT__q_s_f_s, NULL, PRIM_INT__r_s_g_s);


                    ostei_general_vrr_K(13, 0, 5, 0, 10,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_g_s, PRIM_INT__r_s_f_s, NULL, PRIM_INT__q_s_g_s, NULL, PRIM_INT__r_s_h_s);


                    ostei_general_vrr_K(13, 0, 6, 0, 9,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_h_s, PRIM_INT__r_s_g_s, NULL, PRIM_INT__q_s_h_s, NULL, PRIM_INT__r_s_i_s);


                    ostei_general_vrr_K(13, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_i_s, PRIM_INT__r_s_h_s, NULL, PRIM_INT__q_s_i_s, NULL, PRIM_INT__r_s_k_s);


                    VRR_K_s_s_k_s(
                            PRIM_INT__s_s_k_s,
                            PRIM_INT__s_s_i_s,
                            PRIM_INT__s_s_h_s,
                            Q_PA,
                            a_over_q,
                            aoq_PQ,
                            one_over_2q,
                            8);


                    ostei_general_vrr_K(1, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__p_s_k_s, PRIM_INT__p_s_i_s, NULL, PRIM_INT__s_s_k_s, NULL, PRIM_INT__p_s_l_s);


                    ostei_general_vrr_K(2, 0, 9, 0, 6,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__d_s_l_s, PRIM_INT__d_s_k_s, NULL, PRIM_INT__p_s_l_s, NULL, PRIM_INT__d_s_m_s);


                    ostei_general_vrr_K(3, 0, 10, 0, 5,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__f_s_m_s, PRIM_INT__f_s_l_s, NULL, PRIM_INT__d_s_m_s, NULL, PRIM_INT__f_s_n_s);


                    ostei_general_vrr_K(4, 0, 11, 0, 4,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__g_s_n_s, PRIM_INT__g_s_m_s, NULL, PRIM_INT__f_s_n_s, NULL, PRIM_INT__g_s_o_s);


                    ostei_general_vrr_K(5, 0, 12, 0, 3,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__h_s_o_s, PRIM_INT__h_s_n_s, NULL, PRIM_INT__g_s_o_s, NULL, PRIM_INT__h_s_q_s);


                    ostei_general_vrr_K(6, 0, 13, 0, 2,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__i_s_q_s, PRIM_INT__i_s_o_s, NULL, PRIM_INT__h_s_q_s, NULL, PRIM_INT__i_s_r_s);


                    ostei_general_vrr_K(7, 0, 14, 0, 1,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__k_s_r_s, PRIM_INT__k_s_q_s, NULL, PRIM_INT__i_s_r_s, NULL, PRIM_INT__k_s_t_s);


                    ostei_general_vrr_K(8, 0, 13, 0, 2,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_q_s, PRIM_INT__l_s_o_s, NULL, PRIM_INT__k_s_q_s, NULL, PRIM_INT__l_s_r_s);


                    ostei_general_vrr_K(9, 0, 12, 0, 3,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_o_s, PRIM_INT__m_s_n_s, NULL, PRIM_INT__l_s_o_s, NULL, PRIM_INT__m_s_q_s);


                    ostei_general_vrr_K(10, 0, 11, 0, 4,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_n_s, PRIM_INT__n_s_m_s, NULL, PRIM_INT__m_s_n_s, NULL, PRIM_INT__n_s_o_s);


                    ostei_general_vrr_K(11, 0, 10, 0, 5,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_m_s, PRIM_INT__o_s_l_s, NULL, PRIM_INT__n_s_m_s, NULL, PRIM_INT__o_s_n_s);


                    ostei_general_vrr_K(12, 0, 9, 0, 6,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_l_s, PRIM_INT__q_s_k_s, NULL, PRIM_INT__o_s_l_s, NULL, PRIM_INT__q_s_m_s);


                    ostei_general_vrr_K(13, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_k_s, PRIM_INT__r_s_i_s, NULL, PRIM_INT__q_s_k_s, NULL, PRIM_INT__r_s_l_s);


                    ostei_general_vrr1_I(14, 15,
                            one_over_2p, a_over_p, aop_PQ, P_PA,
                            PRIM_INT__r_s_s_s, PRIM_INT__q_s_s_s, PRIM_INT__t_s_s_s);


                    ostei_general_vrr_K(14, 0, 1, 0, 14,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_s_s, NULL, NULL, PRIM_INT__r_s_s_s, NULL, PRIM_INT__t_s_p_s);


                    ostei_general_vrr_K(14, 0, 2, 0, 13,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_p_s, PRIM_INT__t_s_s_s, NULL, PRIM_INT__r_s_p_s, NULL, PRIM_INT__t_s_d_s);


                    ostei_general_vrr_K(14, 0, 3, 0, 12,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_d_s, PRIM_INT__t_s_p_s, NULL, PRIM_INT__r_s_d_s, NULL, PRIM_INT__t_s_f_s);


                    ostei_general_vrr_K(14, 0, 4, 0, 11,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_f_s, PRIM_INT__t_s_d_s, NULL, PRIM_INT__r_s_f_s, NULL, PRIM_INT__t_s_g_s);


                    ostei_general_vrr_K(14, 0, 5, 0, 10,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_g_s, PRIM_INT__t_s_f_s, NULL, PRIM_INT__r_s_g_s, NULL, PRIM_INT__t_s_h_s);


                    ostei_general_vrr_K(14, 0, 6, 0, 9,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_h_s, PRIM_INT__t_s_g_s, NULL, PRIM_INT__r_s_h_s, NULL, PRIM_INT__t_s_i_s);


                    ostei_general_vrr_K(14, 0, 7, 0, 8,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_i_s, PRIM_INT__t_s_h_s, NULL, PRIM_INT__r_s_i_s, NULL, PRIM_INT__t_s_k_s);


                    ostei_general_vrr_K(8, 0, 14, 0, 1,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__l_s_r_s, PRIM_INT__l_s_q_s, NULL, PRIM_INT__k_s_r_s, NULL, PRIM_INT__l_s_t_s);


                    ostei_general_vrr_K(9, 0, 13, 0, 2,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_q_s, PRIM_INT__m_s_o_s, NULL, PRIM_INT__l_s_q_s, NULL, PRIM_INT__m_s_r_s);


                    ostei_general_vrr_K(10, 0, 12, 0, 3,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_o_s, PRIM_INT__n_s_n_s, NULL, PRIM_INT__m_s_o_s, NULL, PRIM_INT__n_s_q_s);


                    ostei_general_vrr_K(11, 0, 11, 0, 4,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_n_s, PRIM_INT__o_s_m_s, NULL, PRIM_INT__n_s_n_s, NULL, PRIM_INT__o_s_o_s);


                    ostei_general_vrr_K(12, 0, 10, 0, 5,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_m_s, PRIM_INT__q_s_l_s, NULL, PRIM_INT__o_s_m_s, NULL, PRIM_INT__q_s_n_s);


                    ostei_general_vrr_K(13, 0, 9, 0, 6,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_l_s, PRIM_INT__r_s_k_s, NULL, PRIM_INT__q_s_l_s, NULL, PRIM_INT__r_s_m_s);


                    ostei_general_vrr_K(14, 0, 8, 0, 7,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_k_s, PRIM_INT__t_s_i_s, NULL, PRIM_INT__r_s_k_s, NULL, PRIM_INT__t_s_l_s);


                    ostei_general_vrr_K(9, 0, 14, 0, 1,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__m_s_r_s, PRIM_INT__m_s_q_s, NULL, PRIM_INT__l_s_r_s, NULL, PRIM_INT__m_s_t_s);


                    ostei_general_vrr_K(10, 0, 13, 0, 2,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_q_s, PRIM_INT__n_s_o_s, NULL, PRIM_INT__m_s_q_s, NULL, PRIM_INT__n_s_r_s);


                    ostei_general_vrr_K(11, 0, 12, 0, 3,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_o_s, PRIM_INT__o_s_n_s, NULL, PRIM_INT__n_s_o_s, NULL, PRIM_INT__o_s_q_s);


                    ostei_general_vrr_K(12, 0, 11, 0, 4,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_n_s, PRIM_INT__q_s_m_s, NULL, PRIM_INT__o_s_n_s, NULL, PRIM_INT__q_s_o_s);


                    ostei_general_vrr_K(13, 0, 10, 0, 5,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_m_s, PRIM_INT__r_s_l_s, NULL, PRIM_INT__q_s_m_s, NULL, PRIM_INT__r_s_n_s);


                    ostei_general_vrr_K(14, 0, 9, 0, 6,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_l_s, PRIM_INT__t_s_k_s, NULL, PRIM_INT__r_s_l_s, NULL, PRIM_INT__t_s_m_s);


                    ostei_general_vrr_K(10, 0, 14, 0, 1,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__n_s_r_s, PRIM_INT__n_s_q_s, NULL, PRIM_INT__m_s_r_s, NULL, PRIM_INT__n_s_t_s);


                    ostei_general_vrr_K(11, 0, 13, 0, 2,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_q_s, PRIM_INT__o_s_o_s, NULL, PRIM_INT__n_s_q_s, NULL, PRIM_INT__o_s_r_s);


                    ostei_general_vrr_K(12, 0, 12, 0, 3,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_o_s, PRIM_INT__q_s_n_s, NULL, PRIM_INT__o_s_o_s, NULL, PRIM_INT__q_s_q_s);


                    ostei_general_vrr_K(13, 0, 11, 0, 4,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_n_s, PRIM_INT__r_s_m_s, NULL, PRIM_INT__q_s_n_s, NULL, PRIM_INT__r_s_o_s);


                    ostei_general_vrr_K(14, 0, 10, 0, 5,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_m_s, PRIM_INT__t_s_l_s, NULL, PRIM_INT__r_s_m_s, NULL, PRIM_INT__t_s_n_s);


                    ostei_general_vrr_K(11, 0, 14, 0, 1,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__o_s_r_s, PRIM_INT__o_s_q_s, NULL, PRIM_INT__n_s_r_s, NULL, PRIM_INT__o_s_t_s);


                    ostei_general_vrr_K(12, 0, 13, 0, 2,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_q_s, PRIM_INT__q_s_o_s, NULL, PRIM_INT__o_s_q_s, NULL, PRIM_INT__q_s_r_s);


                    ostei_general_vrr_K(13, 0, 12, 0, 3,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_o_s, PRIM_INT__r_s_n_s, NULL, PRIM_INT__q_s_o_s, NULL, PRIM_INT__r_s_q_s);


                    ostei_general_vrr_K(14, 0, 11, 0, 4,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_n_s, PRIM_INT__t_s_m_s, NULL, PRIM_INT__r_s_n_s, NULL, PRIM_INT__t_s_o_s);


                    ostei_general_vrr_K(12, 0, 14, 0, 1,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__q_s_r_s, PRIM_INT__q_s_q_s, NULL, PRIM_INT__o_s_r_s, NULL, PRIM_INT__q_s_t_s);


                    ostei_general_vrr_K(13, 0, 13, 0, 2,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_q_s, PRIM_INT__r_s_o_s, NULL, PRIM_INT__q_s_q_s, NULL, PRIM_INT__r_s_r_s);


                    ostei_general_vrr_K(14, 0, 12, 0, 3,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_o_s, PRIM_INT__t_s_n_s, NULL, PRIM_INT__r_s_o_s, NULL, PRIM_INT__t_s_q_s);


                    ostei_general_vrr_K(13, 0, 14, 0, 1,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__r_s_r_s, PRIM_INT__r_s_q_s, NULL, PRIM_INT__q_s_r_s, NULL, PRIM_INT__r_s_t_s);


                    ostei_general_vrr_K(14, 0, 13, 0, 2,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_q_s, PRIM_INT__t_s_o_s, NULL, PRIM_INT__r_s_q_s, NULL, PRIM_INT__t_s_r_s);


                    ostei_general_vrr_K(14, 0, 14, 0, 1,
                            one_over_2q, a_over_q, one_over_2pq, aoq_PQ, Q_PA,
                            PRIM_INT__t_s_r_s, PRIM_INT__t_s_q_s, NULL, PRIM_INT__r_s_r_s, NULL, PRIM_INT__t_s_t_s);




                    ////////////////////////////////////
                    // Accumulate contracted integrals
                    ////////////////////////////////////
                    if(lastoffset == 0)
                    {
                        contract_all(1296, PRIM_INT__k_s_k_s, PRIM_PTR_INT__k_s_k_s);
                        contract_all(1620, PRIM_INT__k_s_l_s, PRIM_PTR_INT__k_s_l_s);
                        contract_all(1980, PRIM_INT__k_s_m_s, PRIM_PTR_INT__k_s_m_s);
                        contract_all(2376, PRIM_INT__k_s_n_s, PRIM_PTR_INT__k_s_n_s);
                        contract_all(2808, PRIM_INT__k_s_o_s, PRIM_PTR_INT__k_s_o_s);
                        contract_all(3276, PRIM_INT__k_s_q_s, PRIM_PTR_INT__k_s_q_s);
                        contract_all(3780, PRIM_INT__k_s_r_s, PRIM_PTR_INT__k_s_r_s);
                        contract_all(4320, PRIM_INT__k_s_t_s, PRIM_PTR_INT__k_s_t_s);
                        contract_all(1620, PRIM_INT__l_s_k_s, PRIM_PTR_INT__l_s_k_s);
                        contract_all(2025, PRIM_INT__l_s_l_s, PRIM_PTR_INT__l_s_l_s);
                        contract_all(2475, PRIM_INT__l_s_m_s, PRIM_PTR_INT__l_s_m_s);
                        contract_all(2970, PRIM_INT__l_s_n_s, PRIM_PTR_INT__l_s_n_s);
                        contract_all(3510, PRIM_INT__l_s_o_s, PRIM_PTR_INT__l_s_o_s);
                        contract_all(4095, PRIM_INT__l_s_q_s, PRIM_PTR_INT__l_s_q_s);
                        contract_all(4725, PRIM_INT__l_s_r_s, PRIM_PTR_INT__l_s_r_s);
                        contract_all(5400, PRIM_INT__l_s_t_s, PRIM_PTR_INT__l_s_t_s);
                        contract_all(1980, PRIM_INT__m_s_k_s, PRIM_PTR_INT__m_s_k_s);
                        contract_all(2475, PRIM_INT__m_s_l_s, PRIM_PTR_INT__m_s_l_s);
                        contract_all(3025, PRIM_INT__m_s_m_s, PRIM_PTR_INT__m_s_m_s);
                        contract_all(3630, PRIM_INT__m_s_n_s, PRIM_PTR_INT__m_s_n_s);
                        contract_all(4290, PRIM_INT__m_s_o_s, PRIM_PTR_INT__m_s_o_s);
                        contract_all(5005, PRIM_INT__m_s_q_s, PRIM_PTR_INT__m_s_q_s);
                        contract_all(5775, PRIM_INT__m_s_r_s, PRIM_PTR_INT__m_s_r_s);
                        contract_all(6600, PRIM_INT__m_s_t_s, PRIM_PTR_INT__m_s_t_s);
                        contract_all(2376, PRIM_INT__n_s_k_s, PRIM_PTR_INT__n_s_k_s);
                        contract_all(2970, PRIM_INT__n_s_l_s, PRIM_PTR_INT__n_s_l_s);
                        contract_all(3630, PRIM_INT__n_s_m_s, PRIM_PTR_INT__n_s_m_s);
                        contract_all(4356, PRIM_INT__n_s_n_s, PRIM_PTR_INT__n_s_n_s);
                        contract_all(5148, PRIM_INT__n_s_o_s, PRIM_PTR_INT__n_s_o_s);
                        contract_all(6006, PRIM_INT__n_s_q_s, PRIM_PTR_INT__n_s_q_s);
                        contract_all(6930, PRIM_INT__n_s_r_s, PRIM_PTR_INT__n_s_r_s);
                        contract_all(7920, PRIM_INT__n_s_t_s, PRIM_PTR_INT__n_s_t_s);
                        contract_all(2808, PRIM_INT__o_s_k_s, PRIM_PTR_INT__o_s_k_s);
                        contract_all(3510, PRIM_INT__o_s_l_s, PRIM_PTR_INT__o_s_l_s);
                        contract_all(4290, PRIM_INT__o_s_m_s, PRIM_PTR_INT__o_s_m_s);
                        contract_all(5148, PRIM_INT__o_s_n_s, PRIM_PTR_INT__o_s_n_s);
                        contract_all(6084, PRIM_INT__o_s_o_s, PRIM_PTR_INT__o_s_o_s);
                        contract_all(7098, PRIM_INT__o_s_q_s, PRIM_PTR_INT__o_s_q_s);
                        contract_all(8190, PRIM_INT__o_s_r_s, PRIM_PTR_INT__o_s_r_s);
                        contract_all(9360, PRIM_INT__o_s_t_s, PRIM_PTR_INT__o_s_t_s);
                        contract_all(3276, PRIM_INT__q_s_k_s, PRIM_PTR_INT__q_s_k_s);
                        contract_all(4095, PRIM_INT__q_s_l_s, PRIM_PTR_INT__q_s_l_s);
                        contract_all(5005, PRIM_INT__q_s_m_s, PRIM_PTR_INT__q_s_m_s);
                        contract_all(6006, PRIM_INT__q_s_n_s, PRIM_PTR_INT__q_s_n_s);
                        contract_all(7098, PRIM_INT__q_s_o_s, PRIM_PTR_INT__q_s_o_s);
                        contract_all(8281, PRIM_INT__q_s_q_s, PRIM_PTR_INT__q_s_q_s);
                        contract_all(9555, PRIM_INT__q_s_r_s, PRIM_PTR_INT__q_s_r_s);
                        contract_all(10920, PRIM_INT__q_s_t_s, PRIM_PTR_INT__q_s_t_s);
                        contract_all(3780, PRIM_INT__r_s_k_s, PRIM_PTR_INT__r_s_k_s);
                        contract_all(4725, PRIM_INT__r_s_l_s, PRIM_PTR_INT__r_s_l_s);
                        contract_all(5775, PRIM_INT__r_s_m_s, PRIM_PTR_INT__r_s_m_s);
                        contract_all(6930, PRIM_INT__r_s_n_s, PRIM_PTR_INT__r_s_n_s);
                        contract_all(8190, PRIM_INT__r_s_o_s, PRIM_PTR_INT__r_s_o_s);
                        contract_all(9555, PRIM_INT__r_s_q_s, PRIM_PTR_INT__r_s_q_s);
                        contract_all(11025, PRIM_INT__r_s_r_s, PRIM_PTR_INT__r_s_r_s);
                        contract_all(12600, PRIM_INT__r_s_t_s, PRIM_PTR_INT__r_s_t_s);
                        contract_all(4320, PRIM_INT__t_s_k_s, PRIM_PTR_INT__t_s_k_s);
                        contract_all(5400, PRIM_INT__t_s_l_s, PRIM_PTR_INT__t_s_l_s);
                        contract_all(6600, PRIM_INT__t_s_m_s, PRIM_PTR_INT__t_s_m_s);
                        contract_all(7920, PRIM_INT__t_s_n_s, PRIM_PTR_INT__t_s_n_s);
                        contract_all(9360, PRIM_INT__t_s_o_s, PRIM_PTR_INT__t_s_o_s);
                        contract_all(10920, PRIM_INT__t_s_q_s, PRIM_PTR_INT__t_s_q_s);
                        contract_all(12600, PRIM_INT__t_s_r_s, PRIM_PTR_INT__t_s_r_s);
                        contract_all(14400, PRIM_INT__t_s_t_s, PRIM_PTR_INT__t_s_t_s);
                    }
                    else
                    {
                        contract(1296, shelloffsets, PRIM_INT__k_s_k_s, PRIM_PTR_INT__k_s_k_s);
                        contract(1620, shelloffsets, PRIM_INT__k_s_l_s, PRIM_PTR_INT__k_s_l_s);
                        contract(1980, shelloffsets, PRIM_INT__k_s_m_s, PRIM_PTR_INT__k_s_m_s);
                        contract(2376, shelloffsets, PRIM_INT__k_s_n_s, PRIM_PTR_INT__k_s_n_s);
                        contract(2808, shelloffsets, PRIM_INT__k_s_o_s, PRIM_PTR_INT__k_s_o_s);
                        contract(3276, shelloffsets, PRIM_INT__k_s_q_s, PRIM_PTR_INT__k_s_q_s);
                        contract(3780, shelloffsets, PRIM_INT__k_s_r_s, PRIM_PTR_INT__k_s_r_s);
                        contract(4320, shelloffsets, PRIM_INT__k_s_t_s, PRIM_PTR_INT__k_s_t_s);
                        contract(1620, shelloffsets, PRIM_INT__l_s_k_s, PRIM_PTR_INT__l_s_k_s);
                        contract(2025, shelloffsets, PRIM_INT__l_s_l_s, PRIM_PTR_INT__l_s_l_s);
                        contract(2475, shelloffsets, PRIM_INT__l_s_m_s, PRIM_PTR_INT__l_s_m_s);
                        contract(2970, shelloffsets, PRIM_INT__l_s_n_s, PRIM_PTR_INT__l_s_n_s);
                        contract(3510, shelloffsets, PRIM_INT__l_s_o_s, PRIM_PTR_INT__l_s_o_s);
                        contract(4095, shelloffsets, PRIM_INT__l_s_q_s, PRIM_PTR_INT__l_s_q_s);
                        contract(4725, shelloffsets, PRIM_INT__l_s_r_s, PRIM_PTR_INT__l_s_r_s);
                        contract(5400, shelloffsets, PRIM_INT__l_s_t_s, PRIM_PTR_INT__l_s_t_s);
                        contract(1980, shelloffsets, PRIM_INT__m_s_k_s, PRIM_PTR_INT__m_s_k_s);
                        contract(2475, shelloffsets, PRIM_INT__m_s_l_s, PRIM_PTR_INT__m_s_l_s);
                        contract(3025, shelloffsets, PRIM_INT__m_s_m_s, PRIM_PTR_INT__m_s_m_s);
                        contract(3630, shelloffsets, PRIM_INT__m_s_n_s, PRIM_PTR_INT__m_s_n_s);
                        contract(4290, shelloffsets, PRIM_INT__m_s_o_s, PRIM_PTR_INT__m_s_o_s);
                        contract(5005, shelloffsets, PRIM_INT__m_s_q_s, PRIM_PTR_INT__m_s_q_s);
                        contract(5775, shelloffsets, PRIM_INT__m_s_r_s, PRIM_PTR_INT__m_s_r_s);
                        contract(6600, shelloffsets, PRIM_INT__m_s_t_s, PRIM_PTR_INT__m_s_t_s);
                        contract(2376, shelloffsets, PRIM_INT__n_s_k_s, PRIM_PTR_INT__n_s_k_s);
                        contract(2970, shelloffsets, PRIM_INT__n_s_l_s, PRIM_PTR_INT__n_s_l_s);
                        contract(3630, shelloffsets, PRIM_INT__n_s_m_s, PRIM_PTR_INT__n_s_m_s);
                        contract(4356, shelloffsets, PRIM_INT__n_s_n_s, PRIM_PTR_INT__n_s_n_s);
                        contract(5148, shelloffsets, PRIM_INT__n_s_o_s, PRIM_PTR_INT__n_s_o_s);
                        contract(6006, shelloffsets, PRIM_INT__n_s_q_s, PRIM_PTR_INT__n_s_q_s);
                        contract(6930, shelloffsets, PRIM_INT__n_s_r_s, PRIM_PTR_INT__n_s_r_s);
                        contract(7920, shelloffsets, PRIM_INT__n_s_t_s, PRIM_PTR_INT__n_s_t_s);
                        contract(2808, shelloffsets, PRIM_INT__o_s_k_s, PRIM_PTR_INT__o_s_k_s);
                        contract(3510, shelloffsets, PRIM_INT__o_s_l_s, PRIM_PTR_INT__o_s_l_s);
                        contract(4290, shelloffsets, PRIM_INT__o_s_m_s, PRIM_PTR_INT__o_s_m_s);
                        contract(5148, shelloffsets, PRIM_INT__o_s_n_s, PRIM_PTR_INT__o_s_n_s);
                        contract(6084, shelloffsets, PRIM_INT__o_s_o_s, PRIM_PTR_INT__o_s_o_s);
                        contract(7098, shelloffsets, PRIM_INT__o_s_q_s, PRIM_PTR_INT__o_s_q_s);
                        contract(8190, shelloffsets, PRIM_INT__o_s_r_s, PRIM_PTR_INT__o_s_r_s);
                        contract(9360, shelloffsets, PRIM_INT__o_s_t_s, PRIM_PTR_INT__o_s_t_s);
                        contract(3276, shelloffsets, PRIM_INT__q_s_k_s, PRIM_PTR_INT__q_s_k_s);
                        contract(4095, shelloffsets, PRIM_INT__q_s_l_s, PRIM_PTR_INT__q_s_l_s);
                        contract(5005, shelloffsets, PRIM_INT__q_s_m_s, PRIM_PTR_INT__q_s_m_s);
                        contract(6006, shelloffsets, PRIM_INT__q_s_n_s, PRIM_PTR_INT__q_s_n_s);
                        contract(7098, shelloffsets, PRIM_INT__q_s_o_s, PRIM_PTR_INT__q_s_o_s);
                        contract(8281, shelloffsets, PRIM_INT__q_s_q_s, PRIM_PTR_INT__q_s_q_s);
                        contract(9555, shelloffsets, PRIM_INT__q_s_r_s, PRIM_PTR_INT__q_s_r_s);
                        contract(10920, shelloffsets, PRIM_INT__q_s_t_s, PRIM_PTR_INT__q_s_t_s);
                        contract(3780, shelloffsets, PRIM_INT__r_s_k_s, PRIM_PTR_INT__r_s_k_s);
                        contract(4725, shelloffsets, PRIM_INT__r_s_l_s, PRIM_PTR_INT__r_s_l_s);
                        contract(5775, shelloffsets, PRIM_INT__r_s_m_s, PRIM_PTR_INT__r_s_m_s);
                        contract(6930, shelloffsets, PRIM_INT__r_s_n_s, PRIM_PTR_INT__r_s_n_s);
                        contract(8190, shelloffsets, PRIM_INT__r_s_o_s, PRIM_PTR_INT__r_s_o_s);
                        contract(9555, shelloffsets, PRIM_INT__r_s_q_s, PRIM_PTR_INT__r_s_q_s);
                        contract(11025, shelloffsets, PRIM_INT__r_s_r_s, PRIM_PTR_INT__r_s_r_s);
                        contract(12600, shelloffsets, PRIM_INT__r_s_t_s, PRIM_PTR_INT__r_s_t_s);
                        contract(4320, shelloffsets, PRIM_INT__t_s_k_s, PRIM_PTR_INT__t_s_k_s);
                        contract(5400, shelloffsets, PRIM_INT__t_s_l_s, PRIM_PTR_INT__t_s_l_s);
                        contract(6600, shelloffsets, PRIM_INT__t_s_m_s, PRIM_PTR_INT__t_s_m_s);
                        contract(7920, shelloffsets, PRIM_INT__t_s_n_s, PRIM_PTR_INT__t_s_n_s);
                        contract(9360, shelloffsets, PRIM_INT__t_s_o_s, PRIM_PTR_INT__t_s_o_s);
                        contract(10920, shelloffsets, PRIM_INT__t_s_q_s, PRIM_PTR_INT__t_s_q_s);
                        contract(12600, shelloffsets, PRIM_INT__t_s_r_s, PRIM_PTR_INT__t_s_r_s);
                        contract(14400, shelloffsets, PRIM_INT__t_s_t_s, PRIM_PTR_INT__t_s_t_s);
                        PRIM_PTR_INT__k_s_k_s += lastoffset*1296;
                        PRIM_PTR_INT__k_s_l_s += lastoffset*1620;
                        PRIM_PTR_INT__k_s_m_s += lastoffset*1980;
                        PRIM_PTR_INT__k_s_n_s += lastoffset*2376;
                        PRIM_PTR_INT__k_s_o_s += lastoffset*2808;
                        PRIM_PTR_INT__k_s_q_s += lastoffset*3276;
                        PRIM_PTR_INT__k_s_r_s += lastoffset*3780;
                        PRIM_PTR_INT__k_s_t_s += lastoffset*4320;
                        PRIM_PTR_INT__l_s_k_s += lastoffset*1620;
                        PRIM_PTR_INT__l_s_l_s += lastoffset*2025;
                        PRIM_PTR_INT__l_s_m_s += lastoffset*2475;
                        PRIM_PTR_INT__l_s_n_s += lastoffset*2970;
                        PRIM_PTR_INT__l_s_o_s += lastoffset*3510;
                        PRIM_PTR_INT__l_s_q_s += lastoffset*4095;
                        PRIM_PTR_INT__l_s_r_s += lastoffset*4725;
                        PRIM_PTR_INT__l_s_t_s += lastoffset*5400;
                        PRIM_PTR_INT__m_s_k_s += lastoffset*1980;
                        PRIM_PTR_INT__m_s_l_s += lastoffset*2475;
                        PRIM_PTR_INT__m_s_m_s += lastoffset*3025;
                        PRIM_PTR_INT__m_s_n_s += lastoffset*3630;
                        PRIM_PTR_INT__m_s_o_s += lastoffset*4290;
                        PRIM_PTR_INT__m_s_q_s += lastoffset*5005;
                        PRIM_PTR_INT__m_s_r_s += lastoffset*5775;
                        PRIM_PTR_INT__m_s_t_s += lastoffset*6600;
                        PRIM_PTR_INT__n_s_k_s += lastoffset*2376;
                        PRIM_PTR_INT__n_s_l_s += lastoffset*2970;
                        PRIM_PTR_INT__n_s_m_s += lastoffset*3630;
                        PRIM_PTR_INT__n_s_n_s += lastoffset*4356;
                        PRIM_PTR_INT__n_s_o_s += lastoffset*5148;
                        PRIM_PTR_INT__n_s_q_s += lastoffset*6006;
                        PRIM_PTR_INT__n_s_r_s += lastoffset*6930;
                        PRIM_PTR_INT__n_s_t_s += lastoffset*7920;
                        PRIM_PTR_INT__o_s_k_s += lastoffset*2808;
                        PRIM_PTR_INT__o_s_l_s += lastoffset*3510;
                        PRIM_PTR_INT__o_s_m_s += lastoffset*4290;
                        PRIM_PTR_INT__o_s_n_s += lastoffset*5148;
                        PRIM_PTR_INT__o_s_o_s += lastoffset*6084;
                        PRIM_PTR_INT__o_s_q_s += lastoffset*7098;
                        PRIM_PTR_INT__o_s_r_s += lastoffset*8190;
                        PRIM_PTR_INT__o_s_t_s += lastoffset*9360;
                        PRIM_PTR_INT__q_s_k_s += lastoffset*3276;
                        PRIM_PTR_INT__q_s_l_s += lastoffset*4095;
                        PRIM_PTR_INT__q_s_m_s += lastoffset*5005;
                        PRIM_PTR_INT__q_s_n_s += lastoffset*6006;
                        PRIM_PTR_INT__q_s_o_s += lastoffset*7098;
                        PRIM_PTR_INT__q_s_q_s += lastoffset*8281;
                        PRIM_PTR_INT__q_s_r_s += lastoffset*9555;
                        PRIM_PTR_INT__q_s_t_s += lastoffset*10920;
                        PRIM_PTR_INT__r_s_k_s += lastoffset*3780;
                        PRIM_PTR_INT__r_s_l_s += lastoffset*4725;
                        PRIM_PTR_INT__r_s_m_s += lastoffset*5775;
                        PRIM_PTR_INT__r_s_n_s += lastoffset*6930;
                        PRIM_PTR_INT__r_s_o_s += lastoffset*8190;
                        PRIM_PTR_INT__r_s_q_s += lastoffset*9555;
                        PRIM_PTR_INT__r_s_r_s += lastoffset*11025;
                        PRIM_PTR_INT__r_s_t_s += lastoffset*12600;
                        PRIM_PTR_INT__t_s_k_s += lastoffset*4320;
                        PRIM_PTR_INT__t_s_l_s += lastoffset*5400;
                        PRIM_PTR_INT__t_s_m_s += lastoffset*6600;
                        PRIM_PTR_INT__t_s_n_s += lastoffset*7920;
                        PRIM_PTR_INT__t_s_o_s += lastoffset*9360;
                        PRIM_PTR_INT__t_s_q_s += lastoffset*10920;
                        PRIM_PTR_INT__t_s_r_s += lastoffset*12600;
                        PRIM_PTR_INT__t_s_t_s += lastoffset*14400;
                    }

                }  // close loop over j
            }  // close loop over i
            
            //Advance to the next batch
            jstart = SIMINT_SIMD_ROUND(jend);
            
            //////////////////////////////////////////////
            // Contracted integrals: Horizontal recurrance
            //////////////////////////////////////////////


            const double hAB[3] = { P.AB_x[ab], P.AB_y[ab], P.AB_z[ab] };


            for(abcd = 0; abcd < nshellbatch; ++abcd, ++real_abcd)
            {
                const double hCD[3] = { Q.AB_x[cd+abcd], Q.AB_y[cd+abcd], Q.AB_z[cd+abcd] };

                // set up HRR pointers
                double const * restrict HRR_INT__k_s_k_s = INT__k_s_k_s + abcd * 1296;
                double const * restrict HRR_INT__k_s_l_s = INT__k_s_l_s + abcd * 1620;
                double const * restrict HRR_INT__k_s_m_s = INT__k_s_m_s + abcd * 1980;
                double const * restrict HRR_INT__k_s_n_s = INT__k_s_n_s + abcd * 2376;
                double const * restrict HRR_INT__k_s_o_s = INT__k_s_o_s + abcd * 2808;
                double const * restrict HRR_INT__k_s_q_s = INT__k_s_q_s + abcd * 3276;
                double const * restrict HRR_INT__k_s_r_s = INT__k_s_r_s + abcd * 3780;
                double const * restrict HRR_INT__k_s_t_s = INT__k_s_t_s + abcd * 4320;
                double const * restrict HRR_INT__l_s_k_s = INT__l_s_k_s + abcd * 1620;
                double const * restrict HRR_INT__l_s_l_s = INT__l_s_l_s + abcd * 2025;
                double const * restrict HRR_INT__l_s_m_s = INT__l_s_m_s + abcd * 2475;
                double const * restrict HRR_INT__l_s_n_s = INT__l_s_n_s + abcd * 2970;
                double const * restrict HRR_INT__l_s_o_s = INT__l_s_o_s + abcd * 3510;
                double const * restrict HRR_INT__l_s_q_s = INT__l_s_q_s + abcd * 4095;
                double const * restrict HRR_INT__l_s_r_s = INT__l_s_r_s + abcd * 4725;
                double const * restrict HRR_INT__l_s_t_s = INT__l_s_t_s + abcd * 5400;
                double const * restrict HRR_INT__m_s_k_s = INT__m_s_k_s + abcd * 1980;
                double const * restrict HRR_INT__m_s_l_s = INT__m_s_l_s + abcd * 2475;
                double const * restrict HRR_INT__m_s_m_s = INT__m_s_m_s + abcd * 3025;
                double const * restrict HRR_INT__m_s_n_s = INT__m_s_n_s + abcd * 3630;
                double const * restrict HRR_INT__m_s_o_s = INT__m_s_o_s + abcd * 4290;
                double const * restrict HRR_INT__m_s_q_s = INT__m_s_q_s + abcd * 5005;
                double const * restrict HRR_INT__m_s_r_s = INT__m_s_r_s + abcd * 5775;
                double const * restrict HRR_INT__m_s_t_s = INT__m_s_t_s + abcd * 6600;
                double const * restrict HRR_INT__n_s_k_s = INT__n_s_k_s + abcd * 2376;
                double const * restrict HRR_INT__n_s_l_s = INT__n_s_l_s + abcd * 2970;
                double const * restrict HRR_INT__n_s_m_s = INT__n_s_m_s + abcd * 3630;
                double const * restrict HRR_INT__n_s_n_s = INT__n_s_n_s + abcd * 4356;
                double const * restrict HRR_INT__n_s_o_s = INT__n_s_o_s + abcd * 5148;
                double const * restrict HRR_INT__n_s_q_s = INT__n_s_q_s + abcd * 6006;
                double const * restrict HRR_INT__n_s_r_s = INT__n_s_r_s + abcd * 6930;
                double const * restrict HRR_INT__n_s_t_s = INT__n_s_t_s + abcd * 7920;
                double const * restrict HRR_INT__o_s_k_s = INT__o_s_k_s + abcd * 2808;
                double const * restrict HRR_INT__o_s_l_s = INT__o_s_l_s + abcd * 3510;
                double const * restrict HRR_INT__o_s_m_s = INT__o_s_m_s + abcd * 4290;
                double const * restrict HRR_INT__o_s_n_s = INT__o_s_n_s + abcd * 5148;
                double const * restrict HRR_INT__o_s_o_s = INT__o_s_o_s + abcd * 6084;
                double const * restrict HRR_INT__o_s_q_s = INT__o_s_q_s + abcd * 7098;
                double const * restrict HRR_INT__o_s_r_s = INT__o_s_r_s + abcd * 8190;
                double const * restrict HRR_INT__o_s_t_s = INT__o_s_t_s + abcd * 9360;
                double const * restrict HRR_INT__q_s_k_s = INT__q_s_k_s + abcd * 3276;
                double const * restrict HRR_INT__q_s_l_s = INT__q_s_l_s + abcd * 4095;
                double const * restrict HRR_INT__q_s_m_s = INT__q_s_m_s + abcd * 5005;
                double const * restrict HRR_INT__q_s_n_s = INT__q_s_n_s + abcd * 6006;
                double const * restrict HRR_INT__q_s_o_s = INT__q_s_o_s + abcd * 7098;
                double const * restrict HRR_INT__q_s_q_s = INT__q_s_q_s + abcd * 8281;
                double const * restrict HRR_INT__q_s_r_s = INT__q_s_r_s + abcd * 9555;
                double const * restrict HRR_INT__q_s_t_s = INT__q_s_t_s + abcd * 10920;
                double const * restrict HRR_INT__r_s_k_s = INT__r_s_k_s + abcd * 3780;
                double const * restrict HRR_INT__r_s_l_s = INT__r_s_l_s + abcd * 4725;
                double const * restrict HRR_INT__r_s_m_s = INT__r_s_m_s + abcd * 5775;
                double const * restrict HRR_INT__r_s_n_s = INT__r_s_n_s + abcd * 6930;
                double const * restrict HRR_INT__r_s_o_s = INT__r_s_o_s + abcd * 8190;
                double const * restrict HRR_INT__r_s_q_s = INT__r_s_q_s + abcd * 9555;
                double const * restrict HRR_INT__r_s_r_s = INT__r_s_r_s + abcd * 11025;
                double const * restrict HRR_INT__r_s_t_s = INT__r_s_t_s + abcd * 12600;
                double const * restrict HRR_INT__t_s_k_s = INT__t_s_k_s + abcd * 4320;
                double const * restrict HRR_INT__t_s_l_s = INT__t_s_l_s + abcd * 5400;
                double const * restrict HRR_INT__t_s_m_s = INT__t_s_m_s + abcd * 6600;
                double const * restrict HRR_INT__t_s_n_s = INT__t_s_n_s + abcd * 7920;
                double const * restrict HRR_INT__t_s_o_s = INT__t_s_o_s + abcd * 9360;
                double const * restrict HRR_INT__t_s_q_s = INT__t_s_q_s + abcd * 10920;
                double const * restrict HRR_INT__t_s_r_s = INT__t_s_r_s + abcd * 12600;
                double const * restrict HRR_INT__t_s_t_s = INT__t_s_t_s + abcd * 14400;
                double * restrict HRR_INT__k_k_k_k = INT__k_k_k_k + real_abcd * 1679616;

                // form INT__k_p_k_s
                HRR_J_k_p(
                    HRR_INT__k_p_k_s,
                    HRR_INT__k_s_k_s,
                    HRR_INT__l_s_k_s,
                    hAB, 36);

                // form INT__k_p_l_s
                HRR_J_k_p(
                    HRR_INT__k_p_l_s,
                    HRR_INT__k_s_l_s,
                    HRR_INT__l_s_l_s,
                    hAB, 45);

                // form INT__k_p_m_s
                HRR_J_k_p(
                    HRR_INT__k_p_m_s,
                    HRR_INT__k_s_m_s,
                    HRR_INT__l_s_m_s,
                    hAB, 55);

                // form INT__k_p_n_s
                HRR_J_k_p(
                    HRR_INT__k_p_n_s,
                    HRR_INT__k_s_n_s,
                    HRR_INT__l_s_n_s,
                    hAB, 66);

                // form INT__k_p_o_s
                HRR_J_k_p(
                    HRR_INT__k_p_o_s,
                    HRR_INT__k_s_o_s,
                    HRR_INT__l_s_o_s,
                    hAB, 78);

                // form INT__k_p_q_s
                HRR_J_k_p(
                    HRR_INT__k_p_q_s,
                    HRR_INT__k_s_q_s,
                    HRR_INT__l_s_q_s,
                    hAB, 91);

                // form INT__k_p_r_s
                HRR_J_k_p(
                    HRR_INT__k_p_r_s,
                    HRR_INT__k_s_r_s,
                    HRR_INT__l_s_r_s,
                    hAB, 105);

                // form INT__k_p_t_s
                HRR_J_k_p(
                    HRR_INT__k_p_t_s,
                    HRR_INT__k_s_t_s,
                    HRR_INT__l_s_t_s,
                    hAB, 120);

                // form INT__l_p_k_s
                HRR_J_l_p(
                    HRR_INT__l_p_k_s,
                    HRR_INT__l_s_k_s,
                    HRR_INT__m_s_k_s,
                    hAB, 36);

                // form INT__l_p_l_s
                HRR_J_l_p(
                    HRR_INT__l_p_l_s,
                    HRR_INT__l_s_l_s,
                    HRR_INT__m_s_l_s,
                    hAB, 45);

                // form INT__l_p_m_s
                HRR_J_l_p(
                    HRR_INT__l_p_m_s,
                    HRR_INT__l_s_m_s,
                    HRR_INT__m_s_m_s,
                    hAB, 55);

                // form INT__l_p_n_s
                HRR_J_l_p(
                    HRR_INT__l_p_n_s,
                    HRR_INT__l_s_n_s,
                    HRR_INT__m_s_n_s,
                    hAB, 66);

                // form INT__l_p_o_s
                HRR_J_l_p(
                    HRR_INT__l_p_o_s,
                    HRR_INT__l_s_o_s,
                    HRR_INT__m_s_o_s,
                    hAB, 78);

                // form INT__l_p_q_s
                HRR_J_l_p(
                    HRR_INT__l_p_q_s,
                    HRR_INT__l_s_q_s,
                    HRR_INT__m_s_q_s,
                    hAB, 91);

                // form INT__l_p_r_s
                HRR_J_l_p(
                    HRR_INT__l_p_r_s,
                    HRR_INT__l_s_r_s,
                    HRR_INT__m_s_r_s,
                    hAB, 105);

                // form INT__l_p_t_s
                HRR_J_l_p(
                    HRR_INT__l_p_t_s,
                    HRR_INT__l_s_t_s,
                    HRR_INT__m_s_t_s,
                    hAB, 120);

                // form INT__m_p_k_s
                ostei_general_hrr_J(9, 1, 7, 0, hAB, HRR_INT__n_s_k_s, HRR_INT__m_s_k_s, HRR_INT__m_p_k_s);

                // form INT__m_p_l_s
                ostei_general_hrr_J(9, 1, 8, 0, hAB, HRR_INT__n_s_l_s, HRR_INT__m_s_l_s, HRR_INT__m_p_l_s);

                // form INT__m_p_m_s
                ostei_general_hrr_J(9, 1, 9, 0, hAB, HRR_INT__n_s_m_s, HRR_INT__m_s_m_s, HRR_INT__m_p_m_s);

                // form INT__m_p_n_s
                ostei_general_hrr_J(9, 1, 10, 0, hAB, HRR_INT__n_s_n_s, HRR_INT__m_s_n_s, HRR_INT__m_p_n_s);

                // form INT__m_p_o_s
                ostei_general_hrr_J(9, 1, 11, 0, hAB, HRR_INT__n_s_o_s, HRR_INT__m_s_o_s, HRR_INT__m_p_o_s);

                // form INT__m_p_q_s
                ostei_general_hrr_J(9, 1, 12, 0, hAB, HRR_INT__n_s_q_s, HRR_INT__m_s_q_s, HRR_INT__m_p_q_s);

                // form INT__m_p_r_s
                ostei_general_hrr_J(9, 1, 13, 0, hAB, HRR_INT__n_s_r_s, HRR_INT__m_s_r_s, HRR_INT__m_p_r_s);

                // form INT__m_p_t_s
                ostei_general_hrr_J(9, 1, 14, 0, hAB, HRR_INT__n_s_t_s, HRR_INT__m_s_t_s, HRR_INT__m_p_t_s);

                // form INT__n_p_k_s
                ostei_general_hrr_J(10, 1, 7, 0, hAB, HRR_INT__o_s_k_s, HRR_INT__n_s_k_s, HRR_INT__n_p_k_s);

                // form INT__n_p_l_s
                ostei_general_hrr_J(10, 1, 8, 0, hAB, HRR_INT__o_s_l_s, HRR_INT__n_s_l_s, HRR_INT__n_p_l_s);

                // form INT__n_p_m_s
                ostei_general_hrr_J(10, 1, 9, 0, hAB, HRR_INT__o_s_m_s, HRR_INT__n_s_m_s, HRR_INT__n_p_m_s);

                // form INT__n_p_n_s
                ostei_general_hrr_J(10, 1, 10, 0, hAB, HRR_INT__o_s_n_s, HRR_INT__n_s_n_s, HRR_INT__n_p_n_s);

                // form INT__n_p_o_s
                ostei_general_hrr_J(10, 1, 11, 0, hAB, HRR_INT__o_s_o_s, HRR_INT__n_s_o_s, HRR_INT__n_p_o_s);

                // form INT__n_p_q_s
                ostei_general_hrr_J(10, 1, 12, 0, hAB, HRR_INT__o_s_q_s, HRR_INT__n_s_q_s, HRR_INT__n_p_q_s);

                // form INT__n_p_r_s
                ostei_general_hrr_J(10, 1, 13, 0, hAB, HRR_INT__o_s_r_s, HRR_INT__n_s_r_s, HRR_INT__n_p_r_s);

                // form INT__n_p_t_s
                ostei_general_hrr_J(10, 1, 14, 0, hAB, HRR_INT__o_s_t_s, HRR_INT__n_s_t_s, HRR_INT__n_p_t_s);

                // form INT__o_p_k_s
                ostei_general_hrr_J(11, 1, 7, 0, hAB, HRR_INT__q_s_k_s, HRR_INT__o_s_k_s, HRR_INT__o_p_k_s);

                // form INT__o_p_l_s
                ostei_general_hrr_J(11, 1, 8, 0, hAB, HRR_INT__q_s_l_s, HRR_INT__o_s_l_s, HRR_INT__o_p_l_s);

                // form INT__o_p_m_s
                ostei_general_hrr_J(11, 1, 9, 0, hAB, HRR_INT__q_s_m_s, HRR_INT__o_s_m_s, HRR_INT__o_p_m_s);

                // form INT__o_p_n_s
                ostei_general_hrr_J(11, 1, 10, 0, hAB, HRR_INT__q_s_n_s, HRR_INT__o_s_n_s, HRR_INT__o_p_n_s);

                // form INT__o_p_o_s
                ostei_general_hrr_J(11, 1, 11, 0, hAB, HRR_INT__q_s_o_s, HRR_INT__o_s_o_s, HRR_INT__o_p_o_s);

                // form INT__o_p_q_s
                ostei_general_hrr_J(11, 1, 12, 0, hAB, HRR_INT__q_s_q_s, HRR_INT__o_s_q_s, HRR_INT__o_p_q_s);

                // form INT__o_p_r_s
                ostei_general_hrr_J(11, 1, 13, 0, hAB, HRR_INT__q_s_r_s, HRR_INT__o_s_r_s, HRR_INT__o_p_r_s);

                // form INT__o_p_t_s
                ostei_general_hrr_J(11, 1, 14, 0, hAB, HRR_INT__q_s_t_s, HRR_INT__o_s_t_s, HRR_INT__o_p_t_s);

                // form INT__q_p_k_s
                ostei_general_hrr_J(12, 1, 7, 0, hAB, HRR_INT__r_s_k_s, HRR_INT__q_s_k_s, HRR_INT__q_p_k_s);

                // form INT__q_p_l_s
                ostei_general_hrr_J(12, 1, 8, 0, hAB, HRR_INT__r_s_l_s, HRR_INT__q_s_l_s, HRR_INT__q_p_l_s);

                // form INT__q_p_m_s
                ostei_general_hrr_J(12, 1, 9, 0, hAB, HRR_INT__r_s_m_s, HRR_INT__q_s_m_s, HRR_INT__q_p_m_s);

                // form INT__q_p_n_s
                ostei_general_hrr_J(12, 1, 10, 0, hAB, HRR_INT__r_s_n_s, HRR_INT__q_s_n_s, HRR_INT__q_p_n_s);

                // form INT__q_p_o_s
                ostei_general_hrr_J(12, 1, 11, 0, hAB, HRR_INT__r_s_o_s, HRR_INT__q_s_o_s, HRR_INT__q_p_o_s);

                // form INT__q_p_q_s
                ostei_general_hrr_J(12, 1, 12, 0, hAB, HRR_INT__r_s_q_s, HRR_INT__q_s_q_s, HRR_INT__q_p_q_s);

                // form INT__q_p_r_s
                ostei_general_hrr_J(12, 1, 13, 0, hAB, HRR_INT__r_s_r_s, HRR_INT__q_s_r_s, HRR_INT__q_p_r_s);

                // form INT__q_p_t_s
                ostei_general_hrr_J(12, 1, 14, 0, hAB, HRR_INT__r_s_t_s, HRR_INT__q_s_t_s, HRR_INT__q_p_t_s);

                // form INT__r_p_k_s
                ostei_general_hrr_J(13, 1, 7, 0, hAB, HRR_INT__t_s_k_s, HRR_INT__r_s_k_s, HRR_INT__r_p_k_s);

                // form INT__r_p_l_s
                ostei_general_hrr_J(13, 1, 8, 0, hAB, HRR_INT__t_s_l_s, HRR_INT__r_s_l_s, HRR_INT__r_p_l_s);

                // form INT__r_p_m_s
                ostei_general_hrr_J(13, 1, 9, 0, hAB, HRR_INT__t_s_m_s, HRR_INT__r_s_m_s, HRR_INT__r_p_m_s);

                // form INT__r_p_n_s
                ostei_general_hrr_J(13, 1, 10, 0, hAB, HRR_INT__t_s_n_s, HRR_INT__r_s_n_s, HRR_INT__r_p_n_s);

                // form INT__r_p_o_s
                ostei_general_hrr_J(13, 1, 11, 0, hAB, HRR_INT__t_s_o_s, HRR_INT__r_s_o_s, HRR_INT__r_p_o_s);

                // form INT__r_p_q_s
                ostei_general_hrr_J(13, 1, 12, 0, hAB, HRR_INT__t_s_q_s, HRR_INT__r_s_q_s, HRR_INT__r_p_q_s);

                // form INT__r_p_r_s
                ostei_general_hrr_J(13, 1, 13, 0, hAB, HRR_INT__t_s_r_s, HRR_INT__r_s_r_s, HRR_INT__r_p_r_s);

                // form INT__r_p_t_s
                ostei_general_hrr_J(13, 1, 14, 0, hAB, HRR_INT__t_s_t_s, HRR_INT__r_s_t_s, HRR_INT__r_p_t_s);

                // form INT__k_d_k_s
                HRR_J_k_d(
                    HRR_INT__k_d_k_s,
                    HRR_INT__k_p_k_s,
                    HRR_INT__l_p_k_s,
                    hAB, 36);

                // form INT__k_d_l_s
                HRR_J_k_d(
                    HRR_INT__k_d_l_s,
                    HRR_INT__k_p_l_s,
                    HRR_INT__l_p_l_s,
                    hAB, 45);

                // form INT__k_d_m_s
                HRR_J_k_d(
                    HRR_INT__k_d_m_s,
                    HRR_INT__k_p_m_s,
                    HRR_INT__l_p_m_s,
                    hAB, 55);

                // form INT__k_d_n_s
                HRR_J_k_d(
                    HRR_INT__k_d_n_s,
                    HRR_INT__k_p_n_s,
                    HRR_INT__l_p_n_s,
                    hAB, 66);

                // form INT__k_d_o_s
                HRR_J_k_d(
                    HRR_INT__k_d_o_s,
                    HRR_INT__k_p_o_s,
                    HRR_INT__l_p_o_s,
                    hAB, 78);

                // form INT__k_d_q_s
                HRR_J_k_d(
                    HRR_INT__k_d_q_s,
                    HRR_INT__k_p_q_s,
                    HRR_INT__l_p_q_s,
                    hAB, 91);

                // form INT__k_d_r_s
                HRR_J_k_d(
                    HRR_INT__k_d_r_s,
                    HRR_INT__k_p_r_s,
                    HRR_INT__l_p_r_s,
                    hAB, 105);

                // form INT__k_d_t_s
                HRR_J_k_d(
                    HRR_INT__k_d_t_s,
                    HRR_INT__k_p_t_s,
                    HRR_INT__l_p_t_s,
                    hAB, 120);

                // form INT__l_d_k_s
                ostei_general_hrr_J(8, 2, 7, 0, hAB, HRR_INT__m_p_k_s, HRR_INT__l_p_k_s, HRR_INT__l_d_k_s);

                // form INT__l_d_l_s
                ostei_general_hrr_J(8, 2, 8, 0, hAB, HRR_INT__m_p_l_s, HRR_INT__l_p_l_s, HRR_INT__l_d_l_s);

                // form INT__l_d_m_s
                ostei_general_hrr_J(8, 2, 9, 0, hAB, HRR_INT__m_p_m_s, HRR_INT__l_p_m_s, HRR_INT__l_d_m_s);

                // form INT__l_d_n_s
                ostei_general_hrr_J(8, 2, 10, 0, hAB, HRR_INT__m_p_n_s, HRR_INT__l_p_n_s, HRR_INT__l_d_n_s);

                // form INT__l_d_o_s
                ostei_general_hrr_J(8, 2, 11, 0, hAB, HRR_INT__m_p_o_s, HRR_INT__l_p_o_s, HRR_INT__l_d_o_s);

                // form INT__l_d_q_s
                ostei_general_hrr_J(8, 2, 12, 0, hAB, HRR_INT__m_p_q_s, HRR_INT__l_p_q_s, HRR_INT__l_d_q_s);

                // form INT__l_d_r_s
                ostei_general_hrr_J(8, 2, 13, 0, hAB, HRR_INT__m_p_r_s, HRR_INT__l_p_r_s, HRR_INT__l_d_r_s);

                // form INT__l_d_t_s
                ostei_general_hrr_J(8, 2, 14, 0, hAB, HRR_INT__m_p_t_s, HRR_INT__l_p_t_s, HRR_INT__l_d_t_s);

                // form INT__m_d_k_s
                ostei_general_hrr_J(9, 2, 7, 0, hAB, HRR_INT__n_p_k_s, HRR_INT__m_p_k_s, HRR_INT__m_d_k_s);

                // form INT__m_d_l_s
                ostei_general_hrr_J(9, 2, 8, 0, hAB, HRR_INT__n_p_l_s, HRR_INT__m_p_l_s, HRR_INT__m_d_l_s);

                // form INT__m_d_m_s
                ostei_general_hrr_J(9, 2, 9, 0, hAB, HRR_INT__n_p_m_s, HRR_INT__m_p_m_s, HRR_INT__m_d_m_s);

                // form INT__m_d_n_s
                ostei_general_hrr_J(9, 2, 10, 0, hAB, HRR_INT__n_p_n_s, HRR_INT__m_p_n_s, HRR_INT__m_d_n_s);

                // form INT__m_d_o_s
                ostei_general_hrr_J(9, 2, 11, 0, hAB, HRR_INT__n_p_o_s, HRR_INT__m_p_o_s, HRR_INT__m_d_o_s);

                // form INT__m_d_q_s
                ostei_general_hrr_J(9, 2, 12, 0, hAB, HRR_INT__n_p_q_s, HRR_INT__m_p_q_s, HRR_INT__m_d_q_s);

                // form INT__m_d_r_s
                ostei_general_hrr_J(9, 2, 13, 0, hAB, HRR_INT__n_p_r_s, HRR_INT__m_p_r_s, HRR_INT__m_d_r_s);

                // form INT__m_d_t_s
                ostei_general_hrr_J(9, 2, 14, 0, hAB, HRR_INT__n_p_t_s, HRR_INT__m_p_t_s, HRR_INT__m_d_t_s);

                // form INT__n_d_k_s
                ostei_general_hrr_J(10, 2, 7, 0, hAB, HRR_INT__o_p_k_s, HRR_INT__n_p_k_s, HRR_INT__n_d_k_s);

                // form INT__n_d_l_s
                ostei_general_hrr_J(10, 2, 8, 0, hAB, HRR_INT__o_p_l_s, HRR_INT__n_p_l_s, HRR_INT__n_d_l_s);

                // form INT__n_d_m_s
                ostei_general_hrr_J(10, 2, 9, 0, hAB, HRR_INT__o_p_m_s, HRR_INT__n_p_m_s, HRR_INT__n_d_m_s);

                // form INT__n_d_n_s
                ostei_general_hrr_J(10, 2, 10, 0, hAB, HRR_INT__o_p_n_s, HRR_INT__n_p_n_s, HRR_INT__n_d_n_s);

                // form INT__n_d_o_s
                ostei_general_hrr_J(10, 2, 11, 0, hAB, HRR_INT__o_p_o_s, HRR_INT__n_p_o_s, HRR_INT__n_d_o_s);

                // form INT__n_d_q_s
                ostei_general_hrr_J(10, 2, 12, 0, hAB, HRR_INT__o_p_q_s, HRR_INT__n_p_q_s, HRR_INT__n_d_q_s);

                // form INT__n_d_r_s
                ostei_general_hrr_J(10, 2, 13, 0, hAB, HRR_INT__o_p_r_s, HRR_INT__n_p_r_s, HRR_INT__n_d_r_s);

                // form INT__n_d_t_s
                ostei_general_hrr_J(10, 2, 14, 0, hAB, HRR_INT__o_p_t_s, HRR_INT__n_p_t_s, HRR_INT__n_d_t_s);

                // form INT__o_d_k_s
                ostei_general_hrr_J(11, 2, 7, 0, hAB, HRR_INT__q_p_k_s, HRR_INT__o_p_k_s, HRR_INT__o_d_k_s);

                // form INT__o_d_l_s
                ostei_general_hrr_J(11, 2, 8, 0, hAB, HRR_INT__q_p_l_s, HRR_INT__o_p_l_s, HRR_INT__o_d_l_s);

                // form INT__o_d_m_s
                ostei_general_hrr_J(11, 2, 9, 0, hAB, HRR_INT__q_p_m_s, HRR_INT__o_p_m_s, HRR_INT__o_d_m_s);

                // form INT__o_d_n_s
                ostei_general_hrr_J(11, 2, 10, 0, hAB, HRR_INT__q_p_n_s, HRR_INT__o_p_n_s, HRR_INT__o_d_n_s);

                // form INT__o_d_o_s
                ostei_general_hrr_J(11, 2, 11, 0, hAB, HRR_INT__q_p_o_s, HRR_INT__o_p_o_s, HRR_INT__o_d_o_s);

                // form INT__o_d_q_s
                ostei_general_hrr_J(11, 2, 12, 0, hAB, HRR_INT__q_p_q_s, HRR_INT__o_p_q_s, HRR_INT__o_d_q_s);

                // form INT__o_d_r_s
                ostei_general_hrr_J(11, 2, 13, 0, hAB, HRR_INT__q_p_r_s, HRR_INT__o_p_r_s, HRR_INT__o_d_r_s);

                // form INT__o_d_t_s
                ostei_general_hrr_J(11, 2, 14, 0, hAB, HRR_INT__q_p_t_s, HRR_INT__o_p_t_s, HRR_INT__o_d_t_s);

                // form INT__q_d_k_s
                ostei_general_hrr_J(12, 2, 7, 0, hAB, HRR_INT__r_p_k_s, HRR_INT__q_p_k_s, HRR_INT__q_d_k_s);

                // form INT__q_d_l_s
                ostei_general_hrr_J(12, 2, 8, 0, hAB, HRR_INT__r_p_l_s, HRR_INT__q_p_l_s, HRR_INT__q_d_l_s);

                // form INT__q_d_m_s
                ostei_general_hrr_J(12, 2, 9, 0, hAB, HRR_INT__r_p_m_s, HRR_INT__q_p_m_s, HRR_INT__q_d_m_s);

                // form INT__q_d_n_s
                ostei_general_hrr_J(12, 2, 10, 0, hAB, HRR_INT__r_p_n_s, HRR_INT__q_p_n_s, HRR_INT__q_d_n_s);

                // form INT__q_d_o_s
                ostei_general_hrr_J(12, 2, 11, 0, hAB, HRR_INT__r_p_o_s, HRR_INT__q_p_o_s, HRR_INT__q_d_o_s);

                // form INT__q_d_q_s
                ostei_general_hrr_J(12, 2, 12, 0, hAB, HRR_INT__r_p_q_s, HRR_INT__q_p_q_s, HRR_INT__q_d_q_s);

                // form INT__q_d_r_s
                ostei_general_hrr_J(12, 2, 13, 0, hAB, HRR_INT__r_p_r_s, HRR_INT__q_p_r_s, HRR_INT__q_d_r_s);

                // form INT__q_d_t_s
                ostei_general_hrr_J(12, 2, 14, 0, hAB, HRR_INT__r_p_t_s, HRR_INT__q_p_t_s, HRR_INT__q_d_t_s);

                // form INT__k_f_k_s
                ostei_general_hrr_J(7, 3, 7, 0, hAB, HRR_INT__l_d_k_s, HRR_INT__k_d_k_s, HRR_INT__k_f_k_s);

                // form INT__k_f_l_s
                ostei_general_hrr_J(7, 3, 8, 0, hAB, HRR_INT__l_d_l_s, HRR_INT__k_d_l_s, HRR_INT__k_f_l_s);

                // form INT__k_f_m_s
                ostei_general_hrr_J(7, 3, 9, 0, hAB, HRR_INT__l_d_m_s, HRR_INT__k_d_m_s, HRR_INT__k_f_m_s);

                // form INT__k_f_n_s
                ostei_general_hrr_J(7, 3, 10, 0, hAB, HRR_INT__l_d_n_s, HRR_INT__k_d_n_s, HRR_INT__k_f_n_s);

                // form INT__k_f_o_s
                ostei_general_hrr_J(7, 3, 11, 0, hAB, HRR_INT__l_d_o_s, HRR_INT__k_d_o_s, HRR_INT__k_f_o_s);

                // form INT__k_f_q_s
                ostei_general_hrr_J(7, 3, 12, 0, hAB, HRR_INT__l_d_q_s, HRR_INT__k_d_q_s, HRR_INT__k_f_q_s);

                // form INT__k_f_r_s
                ostei_general_hrr_J(7, 3, 13, 0, hAB, HRR_INT__l_d_r_s, HRR_INT__k_d_r_s, HRR_INT__k_f_r_s);

                // form INT__k_f_t_s
                ostei_general_hrr_J(7, 3, 14, 0, hAB, HRR_INT__l_d_t_s, HRR_INT__k_d_t_s, HRR_INT__k_f_t_s);

                // form INT__l_f_k_s
                ostei_general_hrr_J(8, 3, 7, 0, hAB, HRR_INT__m_d_k_s, HRR_INT__l_d_k_s, HRR_INT__l_f_k_s);

                // form INT__l_f_l_s
                ostei_general_hrr_J(8, 3, 8, 0, hAB, HRR_INT__m_d_l_s, HRR_INT__l_d_l_s, HRR_INT__l_f_l_s);

                // form INT__l_f_m_s
                ostei_general_hrr_J(8, 3, 9, 0, hAB, HRR_INT__m_d_m_s, HRR_INT__l_d_m_s, HRR_INT__l_f_m_s);

                // form INT__l_f_n_s
                ostei_general_hrr_J(8, 3, 10, 0, hAB, HRR_INT__m_d_n_s, HRR_INT__l_d_n_s, HRR_INT__l_f_n_s);

                // form INT__l_f_o_s
                ostei_general_hrr_J(8, 3, 11, 0, hAB, HRR_INT__m_d_o_s, HRR_INT__l_d_o_s, HRR_INT__l_f_o_s);

                // form INT__l_f_q_s
                ostei_general_hrr_J(8, 3, 12, 0, hAB, HRR_INT__m_d_q_s, HRR_INT__l_d_q_s, HRR_INT__l_f_q_s);

                // form INT__l_f_r_s
                ostei_general_hrr_J(8, 3, 13, 0, hAB, HRR_INT__m_d_r_s, HRR_INT__l_d_r_s, HRR_INT__l_f_r_s);

                // form INT__l_f_t_s
                ostei_general_hrr_J(8, 3, 14, 0, hAB, HRR_INT__m_d_t_s, HRR_INT__l_d_t_s, HRR_INT__l_f_t_s);

                // form INT__m_f_k_s
                ostei_general_hrr_J(9, 3, 7, 0, hAB, HRR_INT__n_d_k_s, HRR_INT__m_d_k_s, HRR_INT__m_f_k_s);

                // form INT__m_f_l_s
                ostei_general_hrr_J(9, 3, 8, 0, hAB, HRR_INT__n_d_l_s, HRR_INT__m_d_l_s, HRR_INT__m_f_l_s);

                // form INT__m_f_m_s
                ostei_general_hrr_J(9, 3, 9, 0, hAB, HRR_INT__n_d_m_s, HRR_INT__m_d_m_s, HRR_INT__m_f_m_s);

                // form INT__m_f_n_s
                ostei_general_hrr_J(9, 3, 10, 0, hAB, HRR_INT__n_d_n_s, HRR_INT__m_d_n_s, HRR_INT__m_f_n_s);

                // form INT__m_f_o_s
                ostei_general_hrr_J(9, 3, 11, 0, hAB, HRR_INT__n_d_o_s, HRR_INT__m_d_o_s, HRR_INT__m_f_o_s);

                // form INT__m_f_q_s
                ostei_general_hrr_J(9, 3, 12, 0, hAB, HRR_INT__n_d_q_s, HRR_INT__m_d_q_s, HRR_INT__m_f_q_s);

                // form INT__m_f_r_s
                ostei_general_hrr_J(9, 3, 13, 0, hAB, HRR_INT__n_d_r_s, HRR_INT__m_d_r_s, HRR_INT__m_f_r_s);

                // form INT__m_f_t_s
                ostei_general_hrr_J(9, 3, 14, 0, hAB, HRR_INT__n_d_t_s, HRR_INT__m_d_t_s, HRR_INT__m_f_t_s);

                // form INT__n_f_k_s
                ostei_general_hrr_J(10, 3, 7, 0, hAB, HRR_INT__o_d_k_s, HRR_INT__n_d_k_s, HRR_INT__n_f_k_s);

                // form INT__n_f_l_s
                ostei_general_hrr_J(10, 3, 8, 0, hAB, HRR_INT__o_d_l_s, HRR_INT__n_d_l_s, HRR_INT__n_f_l_s);

                // form INT__n_f_m_s
                ostei_general_hrr_J(10, 3, 9, 0, hAB, HRR_INT__o_d_m_s, HRR_INT__n_d_m_s, HRR_INT__n_f_m_s);

                // form INT__n_f_n_s
                ostei_general_hrr_J(10, 3, 10, 0, hAB, HRR_INT__o_d_n_s, HRR_INT__n_d_n_s, HRR_INT__n_f_n_s);

                // form INT__n_f_o_s
                ostei_general_hrr_J(10, 3, 11, 0, hAB, HRR_INT__o_d_o_s, HRR_INT__n_d_o_s, HRR_INT__n_f_o_s);

                // form INT__n_f_q_s
                ostei_general_hrr_J(10, 3, 12, 0, hAB, HRR_INT__o_d_q_s, HRR_INT__n_d_q_s, HRR_INT__n_f_q_s);

                // form INT__n_f_r_s
                ostei_general_hrr_J(10, 3, 13, 0, hAB, HRR_INT__o_d_r_s, HRR_INT__n_d_r_s, HRR_INT__n_f_r_s);

                // form INT__n_f_t_s
                ostei_general_hrr_J(10, 3, 14, 0, hAB, HRR_INT__o_d_t_s, HRR_INT__n_d_t_s, HRR_INT__n_f_t_s);

                // form INT__o_f_k_s
                ostei_general_hrr_J(11, 3, 7, 0, hAB, HRR_INT__q_d_k_s, HRR_INT__o_d_k_s, HRR_INT__o_f_k_s);

                // form INT__o_f_l_s
                ostei_general_hrr_J(11, 3, 8, 0, hAB, HRR_INT__q_d_l_s, HRR_INT__o_d_l_s, HRR_INT__o_f_l_s);

                // form INT__o_f_m_s
                ostei_general_hrr_J(11, 3, 9, 0, hAB, HRR_INT__q_d_m_s, HRR_INT__o_d_m_s, HRR_INT__o_f_m_s);

                // form INT__o_f_n_s
                ostei_general_hrr_J(11, 3, 10, 0, hAB, HRR_INT__q_d_n_s, HRR_INT__o_d_n_s, HRR_INT__o_f_n_s);

                // form INT__o_f_o_s
                ostei_general_hrr_J(11, 3, 11, 0, hAB, HRR_INT__q_d_o_s, HRR_INT__o_d_o_s, HRR_INT__o_f_o_s);

                // form INT__o_f_q_s
                ostei_general_hrr_J(11, 3, 12, 0, hAB, HRR_INT__q_d_q_s, HRR_INT__o_d_q_s, HRR_INT__o_f_q_s);

                // form INT__o_f_r_s
                ostei_general_hrr_J(11, 3, 13, 0, hAB, HRR_INT__q_d_r_s, HRR_INT__o_d_r_s, HRR_INT__o_f_r_s);

                // form INT__o_f_t_s
                ostei_general_hrr_J(11, 3, 14, 0, hAB, HRR_INT__q_d_t_s, HRR_INT__o_d_t_s, HRR_INT__o_f_t_s);

                // form INT__k_g_k_s
                ostei_general_hrr_J(7, 4, 7, 0, hAB, HRR_INT__l_f_k_s, HRR_INT__k_f_k_s, HRR_INT__k_g_k_s);

                // form INT__k_g_l_s
                ostei_general_hrr_J(7, 4, 8, 0, hAB, HRR_INT__l_f_l_s, HRR_INT__k_f_l_s, HRR_INT__k_g_l_s);

                // form INT__k_g_m_s
                ostei_general_hrr_J(7, 4, 9, 0, hAB, HRR_INT__l_f_m_s, HRR_INT__k_f_m_s, HRR_INT__k_g_m_s);

                // form INT__k_g_n_s
                ostei_general_hrr_J(7, 4, 10, 0, hAB, HRR_INT__l_f_n_s, HRR_INT__k_f_n_s, HRR_INT__k_g_n_s);

                // form INT__k_g_o_s
                ostei_general_hrr_J(7, 4, 11, 0, hAB, HRR_INT__l_f_o_s, HRR_INT__k_f_o_s, HRR_INT__k_g_o_s);

                // form INT__k_g_q_s
                ostei_general_hrr_J(7, 4, 12, 0, hAB, HRR_INT__l_f_q_s, HRR_INT__k_f_q_s, HRR_INT__k_g_q_s);

                // form INT__k_g_r_s
                ostei_general_hrr_J(7, 4, 13, 0, hAB, HRR_INT__l_f_r_s, HRR_INT__k_f_r_s, HRR_INT__k_g_r_s);

                // form INT__k_g_t_s
                ostei_general_hrr_J(7, 4, 14, 0, hAB, HRR_INT__l_f_t_s, HRR_INT__k_f_t_s, HRR_INT__k_g_t_s);

                // form INT__l_g_k_s
                ostei_general_hrr_J(8, 4, 7, 0, hAB, HRR_INT__m_f_k_s, HRR_INT__l_f_k_s, HRR_INT__l_g_k_s);

                // form INT__l_g_l_s
                ostei_general_hrr_J(8, 4, 8, 0, hAB, HRR_INT__m_f_l_s, HRR_INT__l_f_l_s, HRR_INT__l_g_l_s);

                // form INT__l_g_m_s
                ostei_general_hrr_J(8, 4, 9, 0, hAB, HRR_INT__m_f_m_s, HRR_INT__l_f_m_s, HRR_INT__l_g_m_s);

                // form INT__l_g_n_s
                ostei_general_hrr_J(8, 4, 10, 0, hAB, HRR_INT__m_f_n_s, HRR_INT__l_f_n_s, HRR_INT__l_g_n_s);

                // form INT__l_g_o_s
                ostei_general_hrr_J(8, 4, 11, 0, hAB, HRR_INT__m_f_o_s, HRR_INT__l_f_o_s, HRR_INT__l_g_o_s);

                // form INT__l_g_q_s
                ostei_general_hrr_J(8, 4, 12, 0, hAB, HRR_INT__m_f_q_s, HRR_INT__l_f_q_s, HRR_INT__l_g_q_s);

                // form INT__l_g_r_s
                ostei_general_hrr_J(8, 4, 13, 0, hAB, HRR_INT__m_f_r_s, HRR_INT__l_f_r_s, HRR_INT__l_g_r_s);

                // form INT__l_g_t_s
                ostei_general_hrr_J(8, 4, 14, 0, hAB, HRR_INT__m_f_t_s, HRR_INT__l_f_t_s, HRR_INT__l_g_t_s);

                // form INT__m_g_k_s
                ostei_general_hrr_J(9, 4, 7, 0, hAB, HRR_INT__n_f_k_s, HRR_INT__m_f_k_s, HRR_INT__m_g_k_s);

                // form INT__m_g_l_s
                ostei_general_hrr_J(9, 4, 8, 0, hAB, HRR_INT__n_f_l_s, HRR_INT__m_f_l_s, HRR_INT__m_g_l_s);

                // form INT__m_g_m_s
                ostei_general_hrr_J(9, 4, 9, 0, hAB, HRR_INT__n_f_m_s, HRR_INT__m_f_m_s, HRR_INT__m_g_m_s);

                // form INT__m_g_n_s
                ostei_general_hrr_J(9, 4, 10, 0, hAB, HRR_INT__n_f_n_s, HRR_INT__m_f_n_s, HRR_INT__m_g_n_s);

                // form INT__m_g_o_s
                ostei_general_hrr_J(9, 4, 11, 0, hAB, HRR_INT__n_f_o_s, HRR_INT__m_f_o_s, HRR_INT__m_g_o_s);

                // form INT__m_g_q_s
                ostei_general_hrr_J(9, 4, 12, 0, hAB, HRR_INT__n_f_q_s, HRR_INT__m_f_q_s, HRR_INT__m_g_q_s);

                // form INT__m_g_r_s
                ostei_general_hrr_J(9, 4, 13, 0, hAB, HRR_INT__n_f_r_s, HRR_INT__m_f_r_s, HRR_INT__m_g_r_s);

                // form INT__m_g_t_s
                ostei_general_hrr_J(9, 4, 14, 0, hAB, HRR_INT__n_f_t_s, HRR_INT__m_f_t_s, HRR_INT__m_g_t_s);

                // form INT__n_g_k_s
                ostei_general_hrr_J(10, 4, 7, 0, hAB, HRR_INT__o_f_k_s, HRR_INT__n_f_k_s, HRR_INT__n_g_k_s);

                // form INT__n_g_l_s
                ostei_general_hrr_J(10, 4, 8, 0, hAB, HRR_INT__o_f_l_s, HRR_INT__n_f_l_s, HRR_INT__n_g_l_s);

                // form INT__n_g_m_s
                ostei_general_hrr_J(10, 4, 9, 0, hAB, HRR_INT__o_f_m_s, HRR_INT__n_f_m_s, HRR_INT__n_g_m_s);

                // form INT__n_g_n_s
                ostei_general_hrr_J(10, 4, 10, 0, hAB, HRR_INT__o_f_n_s, HRR_INT__n_f_n_s, HRR_INT__n_g_n_s);

                // form INT__n_g_o_s
                ostei_general_hrr_J(10, 4, 11, 0, hAB, HRR_INT__o_f_o_s, HRR_INT__n_f_o_s, HRR_INT__n_g_o_s);

                // form INT__n_g_q_s
                ostei_general_hrr_J(10, 4, 12, 0, hAB, HRR_INT__o_f_q_s, HRR_INT__n_f_q_s, HRR_INT__n_g_q_s);

                // form INT__n_g_r_s
                ostei_general_hrr_J(10, 4, 13, 0, hAB, HRR_INT__o_f_r_s, HRR_INT__n_f_r_s, HRR_INT__n_g_r_s);

                // form INT__n_g_t_s
                ostei_general_hrr_J(10, 4, 14, 0, hAB, HRR_INT__o_f_t_s, HRR_INT__n_f_t_s, HRR_INT__n_g_t_s);

                // form INT__k_h_k_s
                ostei_general_hrr_J(7, 5, 7, 0, hAB, HRR_INT__l_g_k_s, HRR_INT__k_g_k_s, HRR_INT__k_h_k_s);

                // form INT__k_h_l_s
                ostei_general_hrr_J(7, 5, 8, 0, hAB, HRR_INT__l_g_l_s, HRR_INT__k_g_l_s, HRR_INT__k_h_l_s);

                // form INT__k_h_m_s
                ostei_general_hrr_J(7, 5, 9, 0, hAB, HRR_INT__l_g_m_s, HRR_INT__k_g_m_s, HRR_INT__k_h_m_s);

                // form INT__k_h_n_s
                ostei_general_hrr_J(7, 5, 10, 0, hAB, HRR_INT__l_g_n_s, HRR_INT__k_g_n_s, HRR_INT__k_h_n_s);

                // form INT__k_h_o_s
                ostei_general_hrr_J(7, 5, 11, 0, hAB, HRR_INT__l_g_o_s, HRR_INT__k_g_o_s, HRR_INT__k_h_o_s);

                // form INT__k_h_q_s
                ostei_general_hrr_J(7, 5, 12, 0, hAB, HRR_INT__l_g_q_s, HRR_INT__k_g_q_s, HRR_INT__k_h_q_s);

                // form INT__k_h_r_s
                ostei_general_hrr_J(7, 5, 13, 0, hAB, HRR_INT__l_g_r_s, HRR_INT__k_g_r_s, HRR_INT__k_h_r_s);

                // form INT__k_h_t_s
                ostei_general_hrr_J(7, 5, 14, 0, hAB, HRR_INT__l_g_t_s, HRR_INT__k_g_t_s, HRR_INT__k_h_t_s);

                // form INT__l_h_k_s
                ostei_general_hrr_J(8, 5, 7, 0, hAB, HRR_INT__m_g_k_s, HRR_INT__l_g_k_s, HRR_INT__l_h_k_s);

                // form INT__l_h_l_s
                ostei_general_hrr_J(8, 5, 8, 0, hAB, HRR_INT__m_g_l_s, HRR_INT__l_g_l_s, HRR_INT__l_h_l_s);

                // form INT__l_h_m_s
                ostei_general_hrr_J(8, 5, 9, 0, hAB, HRR_INT__m_g_m_s, HRR_INT__l_g_m_s, HRR_INT__l_h_m_s);

                // form INT__l_h_n_s
                ostei_general_hrr_J(8, 5, 10, 0, hAB, HRR_INT__m_g_n_s, HRR_INT__l_g_n_s, HRR_INT__l_h_n_s);

                // form INT__l_h_o_s
                ostei_general_hrr_J(8, 5, 11, 0, hAB, HRR_INT__m_g_o_s, HRR_INT__l_g_o_s, HRR_INT__l_h_o_s);

                // form INT__l_h_q_s
                ostei_general_hrr_J(8, 5, 12, 0, hAB, HRR_INT__m_g_q_s, HRR_INT__l_g_q_s, HRR_INT__l_h_q_s);

                // form INT__l_h_r_s
                ostei_general_hrr_J(8, 5, 13, 0, hAB, HRR_INT__m_g_r_s, HRR_INT__l_g_r_s, HRR_INT__l_h_r_s);

                // form INT__l_h_t_s
                ostei_general_hrr_J(8, 5, 14, 0, hAB, HRR_INT__m_g_t_s, HRR_INT__l_g_t_s, HRR_INT__l_h_t_s);

                // form INT__m_h_k_s
                ostei_general_hrr_J(9, 5, 7, 0, hAB, HRR_INT__n_g_k_s, HRR_INT__m_g_k_s, HRR_INT__m_h_k_s);

                // form INT__m_h_l_s
                ostei_general_hrr_J(9, 5, 8, 0, hAB, HRR_INT__n_g_l_s, HRR_INT__m_g_l_s, HRR_INT__m_h_l_s);

                // form INT__m_h_m_s
                ostei_general_hrr_J(9, 5, 9, 0, hAB, HRR_INT__n_g_m_s, HRR_INT__m_g_m_s, HRR_INT__m_h_m_s);

                // form INT__m_h_n_s
                ostei_general_hrr_J(9, 5, 10, 0, hAB, HRR_INT__n_g_n_s, HRR_INT__m_g_n_s, HRR_INT__m_h_n_s);

                // form INT__m_h_o_s
                ostei_general_hrr_J(9, 5, 11, 0, hAB, HRR_INT__n_g_o_s, HRR_INT__m_g_o_s, HRR_INT__m_h_o_s);

                // form INT__m_h_q_s
                ostei_general_hrr_J(9, 5, 12, 0, hAB, HRR_INT__n_g_q_s, HRR_INT__m_g_q_s, HRR_INT__m_h_q_s);

                // form INT__m_h_r_s
                ostei_general_hrr_J(9, 5, 13, 0, hAB, HRR_INT__n_g_r_s, HRR_INT__m_g_r_s, HRR_INT__m_h_r_s);

                // form INT__m_h_t_s
                ostei_general_hrr_J(9, 5, 14, 0, hAB, HRR_INT__n_g_t_s, HRR_INT__m_g_t_s, HRR_INT__m_h_t_s);

                // form INT__k_i_k_s
                ostei_general_hrr_J(7, 6, 7, 0, hAB, HRR_INT__l_h_k_s, HRR_INT__k_h_k_s, HRR_INT__k_i_k_s);

                // form INT__k_i_l_s
                ostei_general_hrr_J(7, 6, 8, 0, hAB, HRR_INT__l_h_l_s, HRR_INT__k_h_l_s, HRR_INT__k_i_l_s);

                // form INT__k_i_m_s
                ostei_general_hrr_J(7, 6, 9, 0, hAB, HRR_INT__l_h_m_s, HRR_INT__k_h_m_s, HRR_INT__k_i_m_s);

                // form INT__k_i_n_s
                ostei_general_hrr_J(7, 6, 10, 0, hAB, HRR_INT__l_h_n_s, HRR_INT__k_h_n_s, HRR_INT__k_i_n_s);

                // form INT__k_i_o_s
                ostei_general_hrr_J(7, 6, 11, 0, hAB, HRR_INT__l_h_o_s, HRR_INT__k_h_o_s, HRR_INT__k_i_o_s);

                // form INT__k_i_q_s
                ostei_general_hrr_J(7, 6, 12, 0, hAB, HRR_INT__l_h_q_s, HRR_INT__k_h_q_s, HRR_INT__k_i_q_s);

                // form INT__k_i_r_s
                ostei_general_hrr_J(7, 6, 13, 0, hAB, HRR_INT__l_h_r_s, HRR_INT__k_h_r_s, HRR_INT__k_i_r_s);

                // form INT__k_i_t_s
                ostei_general_hrr_J(7, 6, 14, 0, hAB, HRR_INT__l_h_t_s, HRR_INT__k_h_t_s, HRR_INT__k_i_t_s);

                // form INT__l_i_k_s
                ostei_general_hrr_J(8, 6, 7, 0, hAB, HRR_INT__m_h_k_s, HRR_INT__l_h_k_s, HRR_INT__l_i_k_s);

                // form INT__l_i_l_s
                ostei_general_hrr_J(8, 6, 8, 0, hAB, HRR_INT__m_h_l_s, HRR_INT__l_h_l_s, HRR_INT__l_i_l_s);

                // form INT__l_i_m_s
                ostei_general_hrr_J(8, 6, 9, 0, hAB, HRR_INT__m_h_m_s, HRR_INT__l_h_m_s, HRR_INT__l_i_m_s);

                // form INT__l_i_n_s
                ostei_general_hrr_J(8, 6, 10, 0, hAB, HRR_INT__m_h_n_s, HRR_INT__l_h_n_s, HRR_INT__l_i_n_s);

                // form INT__l_i_o_s
                ostei_general_hrr_J(8, 6, 11, 0, hAB, HRR_INT__m_h_o_s, HRR_INT__l_h_o_s, HRR_INT__l_i_o_s);

                // form INT__l_i_q_s
                ostei_general_hrr_J(8, 6, 12, 0, hAB, HRR_INT__m_h_q_s, HRR_INT__l_h_q_s, HRR_INT__l_i_q_s);

                // form INT__l_i_r_s
                ostei_general_hrr_J(8, 6, 13, 0, hAB, HRR_INT__m_h_r_s, HRR_INT__l_h_r_s, HRR_INT__l_i_r_s);

                // form INT__l_i_t_s
                ostei_general_hrr_J(8, 6, 14, 0, hAB, HRR_INT__m_h_t_s, HRR_INT__l_h_t_s, HRR_INT__l_i_t_s);

                // form INT__k_k_k_s
                ostei_general_hrr_J(7, 7, 7, 0, hAB, HRR_INT__l_i_k_s, HRR_INT__k_i_k_s, HRR_INT__k_k_k_s);

                // form INT__k_k_l_s
                ostei_general_hrr_J(7, 7, 8, 0, hAB, HRR_INT__l_i_l_s, HRR_INT__k_i_l_s, HRR_INT__k_k_l_s);

                // form INT__k_k_m_s
                ostei_general_hrr_J(7, 7, 9, 0, hAB, HRR_INT__l_i_m_s, HRR_INT__k_i_m_s, HRR_INT__k_k_m_s);

                // form INT__k_k_n_s
                ostei_general_hrr_J(7, 7, 10, 0, hAB, HRR_INT__l_i_n_s, HRR_INT__k_i_n_s, HRR_INT__k_k_n_s);

                // form INT__k_k_o_s
                ostei_general_hrr_J(7, 7, 11, 0, hAB, HRR_INT__l_i_o_s, HRR_INT__k_i_o_s, HRR_INT__k_k_o_s);

                // form INT__k_k_q_s
                ostei_general_hrr_J(7, 7, 12, 0, hAB, HRR_INT__l_i_q_s, HRR_INT__k_i_q_s, HRR_INT__k_k_q_s);

                // form INT__k_k_r_s
                ostei_general_hrr_J(7, 7, 13, 0, hAB, HRR_INT__l_i_r_s, HRR_INT__k_i_r_s, HRR_INT__k_k_r_s);

                // form INT__k_k_t_s
                ostei_general_hrr_J(7, 7, 14, 0, hAB, HRR_INT__l_i_t_s, HRR_INT__k_i_t_s, HRR_INT__k_k_t_s);

                // form INT__k_k_k_p
                HRR_L_k_p(
                    HRR_INT__k_k_k_p,
                    HRR_INT__k_k_k_s,
                    HRR_INT__k_k_l_s,
                    hCD, 1296);

                // form INT__k_k_l_p
                HRR_L_l_p(
                    HRR_INT__k_k_l_p,
                    HRR_INT__k_k_l_s,
                    HRR_INT__k_k_m_s,
                    hCD, 1296);

                // form INT__k_k_m_p
                ostei_general_hrr_L(7, 7, 9, 1, hCD, HRR_INT__k_k_n_s, HRR_INT__k_k_m_s, HRR_INT__k_k_m_p);

                // form INT__k_k_n_p
                ostei_general_hrr_L(7, 7, 10, 1, hCD, HRR_INT__k_k_o_s, HRR_INT__k_k_n_s, HRR_INT__k_k_n_p);

                // form INT__k_k_o_p
                ostei_general_hrr_L(7, 7, 11, 1, hCD, HRR_INT__k_k_q_s, HRR_INT__k_k_o_s, HRR_INT__k_k_o_p);

                // form INT__k_k_q_p
                ostei_general_hrr_L(7, 7, 12, 1, hCD, HRR_INT__k_k_r_s, HRR_INT__k_k_q_s, HRR_INT__k_k_q_p);

                // form INT__k_k_r_p
                ostei_general_hrr_L(7, 7, 13, 1, hCD, HRR_INT__k_k_t_s, HRR_INT__k_k_r_s, HRR_INT__k_k_r_p);

                // form INT__k_k_k_d
                HRR_L_k_d(
                    HRR_INT__k_k_k_d,
                    HRR_INT__k_k_k_p,
                    HRR_INT__k_k_l_p,
                    hCD, 1296);

                // form INT__k_k_l_d
                ostei_general_hrr_L(7, 7, 8, 2, hCD, HRR_INT__k_k_m_p, HRR_INT__k_k_l_p, HRR_INT__k_k_l_d);

                // form INT__k_k_m_d
                ostei_general_hrr_L(7, 7, 9, 2, hCD, HRR_INT__k_k_n_p, HRR_INT__k_k_m_p, HRR_INT__k_k_m_d);

                // form INT__k_k_n_d
                ostei_general_hrr_L(7, 7, 10, 2, hCD, HRR_INT__k_k_o_p, HRR_INT__k_k_n_p, HRR_INT__k_k_n_d);

                // form INT__k_k_o_d
                ostei_general_hrr_L(7, 7, 11, 2, hCD, HRR_INT__k_k_q_p, HRR_INT__k_k_o_p, HRR_INT__k_k_o_d);

                // form INT__k_k_q_d
                ostei_general_hrr_L(7, 7, 12, 2, hCD, HRR_INT__k_k_r_p, HRR_INT__k_k_q_p, HRR_INT__k_k_q_d);

                // form INT__k_k_k_f
                ostei_general_hrr_L(7, 7, 7, 3, hCD, HRR_INT__k_k_l_d, HRR_INT__k_k_k_d, HRR_INT__k_k_k_f);

                // form INT__k_k_l_f
                ostei_general_hrr_L(7, 7, 8, 3, hCD, HRR_INT__k_k_m_d, HRR_INT__k_k_l_d, HRR_INT__k_k_l_f);

                // form INT__k_k_m_f
                ostei_general_hrr_L(7, 7, 9, 3, hCD, HRR_INT__k_k_n_d, HRR_INT__k_k_m_d, HRR_INT__k_k_m_f);

                // form INT__k_k_n_f
                ostei_general_hrr_L(7, 7, 10, 3, hCD, HRR_INT__k_k_o_d, HRR_INT__k_k_n_d, HRR_INT__k_k_n_f);

                // form INT__k_k_o_f
                ostei_general_hrr_L(7, 7, 11, 3, hCD, HRR_INT__k_k_q_d, HRR_INT__k_k_o_d, HRR_INT__k_k_o_f);

                // form INT__k_k_k_g
                ostei_general_hrr_L(7, 7, 7, 4, hCD, HRR_INT__k_k_l_f, HRR_INT__k_k_k_f, HRR_INT__k_k_k_g);

                // form INT__k_k_l_g
                ostei_general_hrr_L(7, 7, 8, 4, hCD, HRR_INT__k_k_m_f, HRR_INT__k_k_l_f, HRR_INT__k_k_l_g);

                // form INT__k_k_m_g
                ostei_general_hrr_L(7, 7, 9, 4, hCD, HRR_INT__k_k_n_f, HRR_INT__k_k_m_f, HRR_INT__k_k_m_g);

                // form INT__k_k_n_g
                ostei_general_hrr_L(7, 7, 10, 4, hCD, HRR_INT__k_k_o_f, HRR_INT__k_k_n_f, HRR_INT__k_k_n_g);

                // form INT__k_k_k_h
                ostei_general_hrr_L(7, 7, 7, 5, hCD, HRR_INT__k_k_l_g, HRR_INT__k_k_k_g, HRR_INT__k_k_k_h);

                // form INT__k_k_l_h
                ostei_general_hrr_L(7, 7, 8, 5, hCD, HRR_INT__k_k_m_g, HRR_INT__k_k_l_g, HRR_INT__k_k_l_h);

                // form INT__k_k_m_h
                ostei_general_hrr_L(7, 7, 9, 5, hCD, HRR_INT__k_k_n_g, HRR_INT__k_k_m_g, HRR_INT__k_k_m_h);

                // form INT__k_k_k_i
                ostei_general_hrr_L(7, 7, 7, 6, hCD, HRR_INT__k_k_l_h, HRR_INT__k_k_k_h, HRR_INT__k_k_k_i);

                // form INT__k_k_l_i
                ostei_general_hrr_L(7, 7, 8, 6, hCD, HRR_INT__k_k_m_h, HRR_INT__k_k_l_h, HRR_INT__k_k_l_i);

                // form INT__k_k_k_k
                ostei_general_hrr_L(7, 7, 7, 7, hCD, HRR_INT__k_k_l_i, HRR_INT__k_k_k_i, HRR_INT__k_k_k_k);


            }  // close HRR loop


        }   // close loop cdbatch

        istart = iend;
    }  // close loop over ab

    return P.nshell12_clip * Q.nshell12_clip;
}

